/*
ssc (static site checker)
Copyright (c) 2020,2021 Dylan Harris
https://dylanharris.org/

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public Licence as published by
the Free Software Foundation, either version 3 of the Licence, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public Licence for more details.

You should have received a copy of the GNU General Public
Licence along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "main/standard.h"
#include "main/context.h"
#include "schema/schema_type.h"
#include "utility/quote.h"
#include "microformat/def.h"

#define SF_ENUMERATION          0x10000000
#define SF_EXTERNAL_ENUMERATION 0x20000000
#define SF_NO_ITEMID            0x40000000
#define SF_DEPRECATED           0x80000000
#define SF_SIMPLE_MASK          0x0FFFFFFF
#define SF_NO_SIMPLE_TYPE       0
#define MAKE_SIMPLE_TYPE(T)     static_cast < flags_t > (T)

struct symbol_entry < schema_version, e_schema_type, e_schema, s_schema > schema_type_symbol_table [] =
{   // common tag
    { { s_ctag, 1, 0 }, { 0, 0 }, "AuthorTag", ctag_author, s_ctag },
    { { s_ctag, 1, 0 }, { 0, 0 }, "AutoTag", ctag_auto, s_ctag },
    { { s_ctag, 1, 0 }, { 0, 0 }, "ReaderTag", ctag_reader, s_ctag },
    { { s_ctag, 1, 0 }, { 0, 0 }, "Tag", ctag_tag, s_ctag },

    // DCAM
    { { s_dcam, 1, 0 }, { 0, 0 }, "VocabularyEncodingScheme", dcam_vocabularyencodingscheme, s_dcam },

    // DCMI
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Collection", dcmi_collection, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Dataset", dcmi_dataset, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Event", dcmi_event, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Image", dcmi_image, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "InteractiveResource", dcmi_interactiveresource, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "MovingImage", dcmi_movingimage, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "PhysicalObject", dcmi_physicalobject, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Service", dcmi_service, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Software", dcmi_software, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Sound", dcmi_sound, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "StillImage", dcmi_stillimage, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Text", dcmi_text, s_dcmi },

    // dublin core terms
    { { s_dct, 1, 0 }, { 0, 0 }, "Agent", dct_agent, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "AgentClass", dct_agentclass, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "BibliographicResource", dct_bibliographicresource, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "FileFormat", dct_fileformat, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "Frequency", dct_frequency, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "Jurisdiction", dct_jurisdiction, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "LicenseDocument", dct_licencedocument, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "LinguisticSystem", dct_linguisticsystem, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "Location", dct_location, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "LocationPeriodOrJurisdiction", dct_locationperiodorjurisdiction, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "MediaType", dct_mediatype, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "MediaTypeOrExtent", dct_mediatypeorextent, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "MethodOfAccrual", dct_methodofaccrual, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "MethodOfInstruction", dct_methodofinstruction, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "PeriodOfTime", dct_periodoftime, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "PhysicalMedium", dct_physicalmedium, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "PhysicalResource", dct_physicalresource, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "Policy", dct_policy, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "ProvenanceStatement", dct_provenancestatement, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "RightsStatement", dct_rightsstatement, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "SizeOrDuration", dct_sizeorduration, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "Standard", dct_standard, s_dct },

    // foaf
    { { s_foaf, 0, 1 }, { 0, 0 }, "Agent", foaf_agent, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "Document", foaf_document, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "Group", foaf_group, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "Image", foaf_image, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "LabelProperty", foaf_labelproperty, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "OnlineAccount", foaf_onlineaccount, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "OnlineChatAccount", foaf_onlinechataccount, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "OnlineEcommerceAccount", foaf_onlineecommerceaccount, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "OnlineGamingAccount", foaf_onlinegamingaccount, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "Organization", foaf_organisation, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "Person", foaf_person, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "PersonalProfileDocument", foaf_personalprofiledocument, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "Project", foaf_project, s_foaf },

    // good relations
    { { s_gr, 1, 0, SV_DEPRECATED }, { 0, 0 }, "ActualProductOrServiceInstance", gr_actualproductorserviceinstance, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Brand", gr_brand, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "BusinessEntity", gr_businessentity, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "BusinessEntityType", gr_businessentitytype, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "BusinessFunction", gr_businessfunction, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DayOfWeek", gr_dayofweek, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryChargeSpecification", gr_deliverychargespecification, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryMethod", gr_deliverymethod, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryModeParcelService", gr_deliverymodeparcelservice, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Individual", gr_individual, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "License", gr_licence, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Location", gr_location, s_gr },
    { { s_gr, 1, 0, SV_DEPRECATED }, { 0, 0 }, "LocationOfSalesOrServiceProvisioning", gr_locationofsalesorserviceprovisioning, s_gr },
    { { s_gr, 1, 0, SV_DEPRECATED }, { 0, 0 }, "N-Ary-Relations", gr_n_ary_relations, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Offering", gr_offering, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "OpeningHoursSpecification", gr_openinghoursspecification, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PaymentChargeSpecification", gr_paymentchargespecification, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PaymentMethod", gr_paymentmethod, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PaymentMethodCreditCard", gr_paymentmethodcreditcard, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PriceSpecification", gr_pricespecification, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "ProductOrService", gr_productorservice, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "ProductOrServiceModel", gr_productorservicemodel, s_gr },
    { { s_gr, 1, 0, SV_DEPRECATED }, { 0, 0 }, "ProductOrServicesSomeInstancesPlaceholder", gr_productorservicessomeinstancesplaceholder, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "QualitativeValue", gr_qualitativevalue, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "QuantitativeValue", gr_quantitativevalue, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "QuantitativeValueFloat", gr_quantitativevaluefloat, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "QuantitativeValueInteger", gr_quantitativevalueinteger, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "SomeItems", gr_someitems, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "TypeAndQuantityNode", gr_typeandquantitynode, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "UnitPriceSpecification", gr_unitpricespecification, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "WarrantyPromise", gr_warrantypromise, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "WarrantyScope", gr_warrantyscope, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "AmericanExpress", gr_americanexpress, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Business", gr_business, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "ByBankTransferInAdvance", gr_bybanktransferinadvance, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "ByInvoice", gr_byinvoice, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Cash", gr_cash, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "CheckInAdvance", gr_checkinadvance, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "COD", gr_cod, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "ConstructionInstallation", gr_constructioninstallation, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryModeDirectDownload", gr_deliverymodedirectdownload, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryModeFreight", gr_deliverymodefreight, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryModeMail", gr_deliverymodemail, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryModeOwnFleet", gr_deliverymodeownfleet, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryModePickUp", gr_deliverymodepickup, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DHL", gr_dhl, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DinersClub", gr_dinersclub, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DirectDebit", gr_directdebit, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Discover", gr_discover, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Dispose", gr_dispose, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Enduser", gr_enduser, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "FederalExpress", gr_federalexpress, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Friday", gr_friday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "GoogleCheckout", gr_googlecheckout, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "JCB", gr_jcb, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Labor-BringIn", gr_labour_bringin, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "LeaseOut", gr_leaseout, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Maintain", gr_maintain, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "MasterCard", gr_mastercard, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Monday", gr_monday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PartsAndLabor-BringIn", gr_partsandlabour_bringin, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PartsAndLabor-PickUp", gr_partsandlabour_pickup, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PayPal", gr_paypal, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PaySwarm", gr_payswarm, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "ProvideService", gr_provideservice, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PublicHolidays", gr_publicholidays, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PublicInstitution", gr_publicinstitution, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Repair", gr_repair, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Reseller", gr_reseller, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Saturday", gr_saturday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Sell", gr_sell, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Sunday", gr_sunday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Thursday", gr_thursday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Tuesday", gr_tuesday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "UPS", gr_ups, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "VISA", gr_visa, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Wednesday", gr_wednesday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Buy", gr_buy, s_gr },

    // ical
    { { s_ical, 1, 0 }, { 0, 0 }, "Vevent", ical_vevent, s_ical },
    { { s_ical, 1, 0 }, { 0, 0 }, "Vtimezone", ical_vtimezone, s_ical },

    // https://web.archive.org/web/20101126105442/http://microformats.org/wiki/microdata
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_ADR, mft_adr, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_GEO, mft_geo, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_ATOM, mft_hatom, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_AUDIO, mft_haudio, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/hcalendar", mft_hcalendar, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_ENTRY, mft_hentry, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_LISTING, mft_hlisting, s_microformats, SF_NO_ITEMID | SF_DEPRECATED }, // no XMDP, but ...
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_MEDIA, mft_hmedia, s_microformats, SF_NO_ITEMID | SF_DEPRECATED }, // no XMDP, but ...
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_NEWS, mft_hnews, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_PRODUCT, mft_hproduct, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_RECIPE, mft_hrecipe, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_RESUME, mft_hresume, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_REVIEW, mft_hreview, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_REVIEW_AGGREGATE, mft_haggregate, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },

    // https://microformats.org/wiki/microdata (April 2021)
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_ADR, mft2_hadr, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_BREADCRUMB, mft2_hbreadcrumb, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_CARD, mft2_hcard, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_CITE, mft2_hcite, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_ENTRY, mft2_hentry, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_EVENT, mft2_hevent, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_FEED, mft2_hfeed, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_GEO, mft2_hgeo, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_ITEM, mft2_hitem, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_LISTING, mft2_hlisting, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_PRODUCT, mft2_hproduct, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_RECIPE, mft2_hrecipe, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_RESUME, mft2_hresume, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_REVIEW, mft2_hreview, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_AGGREGATE, mft2_haggregate, s_microformats },

    // poetry
    { { s_poetry, 1, 0 }, { 0, 0 }, "Dialect", poetry_dialect, s_poetry },
    { { s_poetry, 1, 0 }, { 0, 0 }, "Form", poetry_form, s_poetry },
    { { s_poetry, 1, 0 }, { 0, 0 }, "Poem", poetry_poem, s_poetry },
    { { s_poetry, 1, 0 }, { 0, 0 }, "Period", poetry_period, s_poetry },
    { { s_poetry, 1, 0 }, { 0, 0 }, "Stanza", poetry_stanza, s_poetry },

    // RDF
    { { s_rdf, 1, 0 }, { 0, 0 }, "Alt", rdf_alt, s_rdf },
    { { s_rdf, 1, 0 }, { 0, 0 }, "Bag", rdf_bag, s_rdf },
    { { s_rdf, 1, 0 }, { 0, 0 }, "Compoundliteral", rdf_compoundliteral, s_rdf, MAKE_SIMPLE_TYPE (t_text) },
    { { s_rdf, 1, 0 }, { 0, 0 }, "HTML", rdf_html, s_rdf, s_rdf, MAKE_SIMPLE_TYPE (t_xmlliteral) },
    { { s_rdf, 1, 0 }, { 0, 0 }, "JSON", rdf_json, s_rdf, MAKE_SIMPLE_TYPE (t_xmlliteral) },
    { { s_rdf, 1, 0 }, { 0, 0 }, "langString", rdf_langstring, s_rdf, MAKE_SIMPLE_TYPE (t_lang) },
    { { s_rdf, 1, 0 }, { 0, 0 }, "List", rdf_list, s_rdf },
    { { s_rdf, 1, 0 }, { 0, 0 }, "Plainliteral", rdf_plainliteral, s_rdf, MAKE_SIMPLE_TYPE (t_text) },
    { { s_rdf, 1, 0 }, { 0, 0 }, "Property", rdf_property, s_rdf },
    { { s_rdf, 1, 0 }, { 0, 0 }, "Seq", rdf_seq, s_rdf },
    { { s_rdf, 1, 0 }, { 0, 0 }, "Statement", rdf_statement, s_rdf },
    { { s_rdf, 1, 0 }, { 0, 0 }, "XMLLiteral", rdf_xmlliteral, s_rdf, MAKE_SIMPLE_TYPE (t_xmlliteral) },

    // RDFs
    { { s_rdfs, 1, 0 }, { 0, 0 }, "Class", rdfs_class, s_rdfs },
    { { s_rdfs, 1, 0 }, { 0, 0 }, "Container", rdfs_container, s_rdfs },
    { { s_rdfs, 1, 0 }, { 0, 0 }, "ContainerMembershipProperty", rdfs_containermembershipproperty, s_rdfs },
    { { s_rdfs, 1, 0 }, { 0, 0 }, "Datatype", rdfs_datatype, s_rdfs },
    { { s_rdfs, 1, 0 }, { 0, 0 }, "Literal", rdfs_literal, s_rdfs },
    { { s_rdfs, 1, 0 }, { 0, 0 }, "Resource", rdfs_resource, s_rdfs },

    // schema.org
    { { 2, 0 }, { 0, 0 }, "APIReference", sch_apireference },
    { { 3, 5 }, { 0, 0 }, "3DModel", sch_3dmodel },
    { { 3, 6 }, { 6, 0 }, "Abdomen", sch_abdomen },
    { { 2, 0 }, { 0, 0 }, "AboutPage", sch_aboutpage },
    { { 2, 0 }, { 0, 0 }, "AcceptAction", sch_acceptaction },
    { { 3, 1 }, { 0, 0 }, "Accommodation", sch_accommodation },
    { { 2, 0 }, { 0, 0 }, "AccountingService", sch_accountingservice },
    { { 2, 0 }, { 0, 0 }, "AchieveAction", sch_achieveaction },
    { { 2, 0 }, { 0, 0 }, "Action", sch_action },
    { { 3, 5 }, { 0, 0 }, "ActionAccessSpecification", sch_actionaccessspecification },
    { { 2, 0 }, { 0, 0 }, "ActionStatusType", sch_actionstatustype, s_schema, SF_ENUMERATION },
    { { 11, 0 }, { 0, 0 }, "ActivationFee", sch_activationfee },
    { { 2, 0 }, { 0, 0 }, "ActivateAction", sch_activateaction },
    { { 3, 5 }, { 0, 0 }, "ActiveActionStatus", sch_activeactionstatus },
    { { 3, 6 }, { 6, 0 }, "ActiveNotRecruiting", sch_activenotrecruiting },
    { { 2, 0 }, { 0, 0 }, "AddAction", sch_addaction },
    { { 2, 0 }, { 0, 0 }, "AdministrativeArea", sch_administrativearea },
    { { 2, 0 }, { 0, 0 }, "AdultEntertainment", sch_adultentertainment },
    { { 3, 5 }, { 0, 0 }, "AdvertiserContentArticle", sch_advertisercontentarticle },
    { { 3, 5 }, { 0, 0 }, "AerobicActivity", sch_aerobicactivity },
    { { 2, 0 }, { 0, 0 }, "AggregateOffer", sch_aggregateoffer },
    { { 2, 0 }, { 0, 0 }, "AggregateRating", sch_aggregaterating },
    { { 2, 0 }, { 0, 0 }, "AgreeAction", sch_agreeaction },
    { { 2, 0 }, { 0, 0 }, "Airline", sch_airline },
    { { 2, 0 }, { 0, 0 }, "Airport", sch_airport },
    { { 3, 5 }, { 0, 0 }, "AlbumRelease", sch_albumrelease },
    { { 2, 0 }, { 0, 0 }, "AlignmentObject", sch_alignmentobject },
    { { 12, 0 }, { 0, 0 }, "AllergiesHealthAspect", sch_allergieshealthaspect },
    { { 2, 0 }, { 0, 0 }, "AllocateAction", sch_allocateaction },
    { { 3, 6 }, { 6, 0 }, "AllWheelDriveConfiguration", sch_allwheeldriveconfiguration },
    { { 11, 0 }, { 0, 0 }, "AmpStory", sch_ampstory },
    { { 3, 5 }, { 0, 0 }, "AMRadioChannel", sch_amradiochannel },
    { { 2, 0 }, { 0, 0 }, "AmusementPark", sch_amusementpark },
    { { 3, 5 }, { 0, 0 }, "AnaerobicActivity", sch_anaerobicactivity },
    { { 3, 5 }, { 0, 0 }, "AnalysisNewsArticle", sch_analysisnewsarticle },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "AnatomicalStructure", sch_anatomicalstructure },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "AnatomicalSystem", sch_anatomicalsystem },
    { { 3, 6 }, { 6, 0 }, "Anesthesia", sch_anaesthesia },
    { { 2, 0 }, { 0, 0 }, "Animalshelter", sch_animalshelter },
    { { 2, 0 }, { 0, 0 }, "Answer", sch_answer },
    { { 3, 1 }, { 0, 0 }, "Apartment", sch_apartment },
    { { 2, 0 }, { 0, 0 }, "Apartmentcomplex", sch_apartmentcomplex },
    { { 3, 6 }, { 6, 0 }, "Appearance", sch_appearance },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "AppendAction", sch_appendaction },
    { { 2, 0 }, { 0, 0 }, "ApplyAction", sch_applyaction },
    { { 2, 0 }, { 0, 0 }, "ApprovedIndication", sch_approvedindication },
    { { 2, 0 }, { 0, 0 }, "Aquarium", sch_aquarium },
    { { 3, 5 }, { 0, 0 }, "ArchiveComponent", sch_archivecomponent },
    { { 3, 5 }, { 0, 0 }, "ArchiveOrganization", sch_archiveorganisation },
    { { 2, 0 }, { 0, 0 }, "ArriveAction", sch_arriveaction },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Artery", sch_artery },
    { { 2, 0 }, { 0, 0 }, "ArtGallery", sch_artgallery },
    { { 2, 0 }, { 0, 0 }, "Article", sch_article },
    { { 2, 0 }, { 0, 0 }, "AskAction", sch_askaction },
    { { 3, 5 }, { 0, 0 }, "AskPublicNewsArticle", sch_askpublicnewsarticle },
    { { 2, 0 }, { 0, 0 }, "AssessAction", sch_assessaction },
    { { 2, 0 }, { 0, 0 }, "AssignAction", sch_assignaction },
    { { 3, 5 }, { 0, 0 }, "Atlas", sch_atlas },
    { { 2, 0 }, { 0, 0 }, "Attorney", sch_attorney },
    { { 2, 0 }, { 0, 0 }, "Audience", sch_audience },
    { { 3, 5 }, { 0, 0 }, "Audiobook", sch_audiobook },
    { { 3, 5 }, { 0, 0 }, "AudiobookFormat", sch_audiobookformat },
    { { 2, 0 }, { 0, 0 }, "AudioObject", sch_audioobject },
    { { 13, 0 }, { 0, 0 }, "AudioObjectSnapshot", sch_audioobjectsnapshot },
    { { 7, 0 }, { 0, 0 }, "AuthenticContent", sch_authenticcontent },
    { { 3, 5 }, { 0, 0 }, "AuthoritativeLegalValue", sch_authoritativelegalvalue },
    { { 2, 0 }, { 0, 0 }, "AuthorizeAction", sch_authoriseaction },
    { { 2, 0 }, { 0, 0 }, "AutoBodyShop", sch_autobodyshop },
    { { 2, 0 }, { 0, 0 }, "AutoDealer", sch_autodealer },
    { { 2, 0 }, { 0, 0 }, "AutomatedTeller", sch_automatedteller },
    { { 2, 0 }, { 0, 0 }, "AutomotiveBusiness", sch_automotivebusiness },
    { { 2, 0 }, { 0, 0 }, "AutoPartsStore", sch_autopartsstore },
    { { 2, 0 }, { 0, 0 }, "AutoRental", sch_autorental },
    { { 2, 0 }, { 0, 0 }, "AutoRepair", sch_autorepair },
    { { 2, 0 }, { 0, 0 }, "AutoWash", sch_autowash },
    { { 3, 6 }, { 6, 0 }, "Ayurvedic", sch_pixiegiven },
    { { 3, 5 }, { 0, 0 }, "BackgroundNewsArticle", sch_backgroundnewsarticle },
    { { 12, 0 }, { 0, 0 }, "BackOrder", sch_backorder },
    { { 3, 6 }, { 6, 0 }, "Bacteria", sch_bacteria },
    { { 2, 0 }, { 0, 0 }, "Bakery", sch_bakery },
    { { 3, 5 }, { 0, 0 }, "Balance", sch_balance },
    { { 3, 0 }, { 0, 0 }, "BankAccount", sch_bankaccount },
    { { 2, 0 }, { 0, 0 }, "BankOrCreditUnion", sch_bankorcreditunion },
    { { 2, 2 }, { 0, 0 }, "BarCode", sch_barcode },
    { { 2, 0 }, { 0, 0 }, "BarOrPub", sch_barorpub },
    { { 7, 4 }, { 0, 0 }, "BasicIncome", sch_basicincome },
    { { 2, 0 }, { 0, 0 }, "Beach", sch_beach },
    { { 2, 0 }, { 0, 0 }, "BeautySalon", sch_beautysalon },
    { { 2, 0 }, { 0, 0 }, "BedAndBreakfast", sch_bedandbreakfast },
    { { 3, 1 }, { 0, 0 }, "BedDetails", sch_beddetails },
    { { 3, 5 }, { 0, 0 }, "BedType", sch_bedtype },
    { { 2, 0 }, { 0, 0 }, "BefriendAction", sch_befriendaction },
    { { 5, 0 }, { 0, 0 }, "BenefitsHealthAspect", sch_benefitshealthaspect },
    { { 2, 0 }, { 0, 0 }, "BikeStore", sch_bikestore },
    { { 13, 0 }, { 0, 0 }, "BioChemEntity", sch_biochementity },
    { { 2, 0 }, { 0, 0 }, "Blog", sch_blog },
    { { 2, 0 }, { 0, 0 }, "BlogPosting", sch_blogposting },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "BloodTest", sch_bloodtest },
    { { 2, 0 }, { 0, 0 }, "BoardingPolicyType", sch_boardingpolicytype, s_schema, SF_ENUMERATION },
    { { 9, 0 }, { 0, 0 }, "BoatReservation", sch_boatreservation },
    { { 9, 0 }, { 0, 0 }, "BoatTerminal", sch_boatterminal },
    { { 9, 0 }, { 0, 0 }, "BoatTrip", sch_boattrip },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementArm", sch_bodymeasurementarm },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementBust", sch_bodymeasurementbust },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementChest", sch_bodymeasurementchest },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementFoot", sch_bodymeasurementfoot },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementHand", sch_bodymeasurementhand },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementHead", sch_bodymeasurementhead },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementHeight", sch_bodymeasurementheight },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementHips", sch_bodymeasurementhips },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementInsideLeg", sch_bodymeasurementinsideleg },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementTypeEnumeration", sch_bodymeasurementtypeenumeration, s_schema, SF_ENUMERATION },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementNeck", sch_bodymeasurementneck },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementUnderbust", sch_bodymeasurementunderbust },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementWaist", sch_bodymeasurementwaist },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementWeight", sch_bodymeasurementweight },
    { { 2, 0 }, { 0, 0 }, "BodyOfWater", sch_bodyofwater },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Bone", sch_bone },
    { { 2, 0 }, { 0, 0 }, "Book", sch_book },
    { { 2, 0 }, { 0, 0 }, "BookFormatType", sch_bookformattype, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "BookMarkAction", sch_bookmarkaction },
    { { 2, 0 }, { 0, 0 }, "BookSeries", sch_bookseries },
    { { 2, 0 }, { 0, 0 }, "BookStore", sch_bookstore },
    { { 2, 0 }, { 0, 0 }, "Boolean", sch_boolean, s_schema, SF_ENUMERATION | MAKE_SIMPLE_TYPE (t_bool) },
    { { 2, 0 }, { 0, 0 }, "BorrowAction", sch_borrowaction },
    { { 2, 0 }, { 0, 0 }, "BowlingAlley", sch_bowlingalley },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "BrainStructure", sch_brainstructure },
    { { 2, 0 }, { 0, 0 }, "Brand", sch_brand },
    { { 2, 0 }, { 0, 0 }, "BreadCrumblist", sch_breadcrumblist },
    { { 2, 0 }, { 0, 0 }, "Brewery", sch_brewery },
    { { 2, 1 }, { 0, 0 }, "Bridge", sch_bridge },
    { { 2, 0 }, { 0, 0 }, "BroadcastChannel", sch_broadcastchannel },
    { { 2, 0 }, { 0, 0 }, "BroadcastEvent", sch_broadcastevent },
    { { 3, 5 }, { 0, 0 }, "BroadcastFrequencySpecification", sch_broadcastfrequencyspecification },
    { { 3, 5 }, { 0, 0 }, "BroadcastRelease", sch_broadcastrelease },
    { { 2, 0 }, { 0, 0 }, "BroadcastService", sch_broadcastservice },
    { { 3, 5 }, { 0, 0 }, "BrokerageAccount", sch_brokerageaccount },
    { { 2, 0 }, { 0, 0 }, "BuddhistTemple", sch_buddhisttemple },
    { { 2, 0 }, { 0, 0 }, "BusinessAudience", sch_businessaudience },
    { { 2, 0 }, { 0, 0 }, "BusinessEntityType", sch_businessentitytype, s_schema, SF_EXTERNAL_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "BusinessEvent", sch_businessevent },
    { { 2, 0 }, { 0, 0 }, "BusinessFunction", sch_businessfunction, s_schema, SF_EXTERNAL_ENUMERATION },
    { { 7, 4 }, { 0, 0 }, "BusinessSupport", sch_businesssupport },
    { { 3, 5 }, { 0, 0 }, "BusOrCoach", sch_busorcoach },
    { { 2, 0 }, { 0, 0 }, "BusReservation", sch_busreservation },
    { { 2, 0 }, { 0, 0 }, "BusStation", sch_busstation },
    { { 2, 0 }, { 0, 0 }, "BusStop", sch_busstop },
    { { 2, 0 }, { 0, 0 }, "BusTrip", sch_bustrip },
    { { 2, 0 }, { 0, 0 }, "BuyAction", sch_buyaction },
    { { 2, 0 }, { 0, 0 }, "CableOrSatelliteService", sch_cableorsatelliteservice },
    { { 2, 0 }, { 0, 0 }, "CafeOrCoffeeShop", sch_cafeorcoffeeshop },
    { { 2, 0 }, { 0, 0 }, "Campground", sch_campground },
    { { 3, 1 }, { 0, 0 }, "CampingPitch", sch_campingpitch },
    { { 2, 0 }, { 0, 0 }, "Canal", sch_canal },
    { { 2, 0 }, { 0, 0 }, "CancelAction", sch_cancelaction },
    { { 2, 0 }, { 0, 0 }, "Car", sch_car },
    { { 3, 6 }, { 6, 0 }, "Cardiovascular", sch_cardiovascular },
    { { 3, 6 }, { 6, 0 }, "CardiovascularExam", sch_cardiovascularexam },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "CarUsageType", sch_carusagetype, s_schema, SF_ENUMERATION },
    { { 3, 6 }, { 6, 0 }, "CaseSeries", sch_caseseries },
    { { 2, 0 }, { 0, 0 }, "Casino", sch_casino },
    { { 3, 5 }, { 0, 0 }, "CassetteFormat", sch_cassetteformat },
    { { 3, 5 }, { 0, 0 }, "CategoryCode", sch_categorycode },
    { { 3, 5 }, { 0, 0 }, "CategoryCodeSet", sch_categorycodeset },
    { { 2, 0 }, { 0, 0 }, "CatholicChurch", sch_catholicchurch },
    { { 5, 0 }, { 0, 0 }, "CausesHealthAspect", sch_causeshealthaspect },
    { { 7, 3 }, { 0, 0 }, "CDCPMDRecord", sch_cdcpmdrecord },
    { { 3, 5 }, { 0, 0 }, "CDFormat", sch_cdformat },
    { { 2, 0 }, { 0, 0 }, "Cemetery", sch_cemetery },
    { { 3, 5 }, { 0, 0 }, "Chapter", sch_chapter },
    { { 9, 0 }, { 0, 0 }, "CharitableIncorporatedOrganization", sch_charitableincorporatedorganization },
    { { 2, 0 }, { 0, 0 }, "CheckSction", sch_checkaction },
    { { 2, 0 }, { 0, 0 }, "CheckInAction", sch_checkinaction },
    { { 2, 0 }, { 0, 0 }, "CheckOutAction", sch_checkoutaction },
    { { 2, 0 }, { 0, 0 }, "CheckOutPage", sch_checkoutpage },
    { { 13, 0 }, { 0, 0 }, "ChemicalSubstance", sch_chemicalsubstance },
    { { 2, 0 }, { 0, 0 }, "ChildCare", sch_childcare },
    { { 2, 0 }, { 0, 0 }, "ChildrensEvent", sch_childrensevent },
    { { 3, 6 }, { 6, 0 }, "Chiropractic", sch_harmful },
    { { 2, 0 }, { 0, 0 }, "ChooseAction", sch_chooseaction },
    { { 2, 0 }, { 0, 0 }, "Church", sch_church },
    { { 2, 0 }, { 0, 0 }, "City", sch_city },
    { { 2, 0 }, { 0, 0 }, "CityHall", sch_cityhall },
    { { 2, 0 }, { 0, 0 }, "CivicStructure", sch_civicstructure },
    { { 3, 5 }, { 0, 0 }, "Claim", sch_claim },
    { { 3, 2 }, { 0, 0 }, "ClaimReview", sch_claimreview },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Class", sch_class },
    { { 11, 0 }, { 0, 0 }, "CleaningFee", sch_cleaningfee },
    { { 3, 6 }, { 6, 0 }, "Clinician", sch_clinician },
    { { 2, 0 }, { 0, 0 }, "Clip", sch_clip },
    { { 2, 0 }, { 0, 0 }, "ClothingStore", sch_clothingstore },
    { { 2, 0 }, { 2, 1 }, "Code", sch_code },
    { { 3, 6 }, { 6, 0 }, "CohortStudy", sch_cohortstudy },
    { { 3, 5 }, { 0, 0 }, "Collection", sch_collection },
    { { 2, 0 }, { 0, 0 }, "CollectionPage", sch_collectionpage },
    { { 2, 0 }, { 0, 0 }, "CollegeOrUniversity", sch_collegeoruniversity },
    { { 2, 0 }, { 0, 0 }, "ComedyClub", sch_comedyclub },
    { { 2, 0 }, { 0, 0 }, "ComedyEvent", sch_comedyevent },
    { { 3, 5 }, { 0, 0 }, "ComicCoverArt", sch_comiccoverart },
    { { 3, 5 }, { 0, 0 }, "ComicIssue", sch_comicissue },
    { { 3, 5 }, { 0, 0 }, "ComicSeries", sch_comicseries },
    { { 3, 5 }, { 0, 0 }, "ComicStory", sch_comicstory },
    { { 2, 0 }, { 0, 0 }, "Comment", sch_comment },
    { { 2, 0 }, { 0, 0 }, "CommentAction", sch_commentaction },
    { { 3, 5 }, { 0, 0 }, "CommentPermission", sch_commentpermission },
    { { 2, 0 }, { 0, 0 }, "CommunicateAction", sch_communicateaction },
    { { 3, 5 }, { 0, 0 }, "CommunityHealth", sch_communityhealth },
    { { 3, 5 }, { 0, 0 }, "CompilationAlbum", sch_compilationalbum },
    { { 3, 6 }, { 6, 0 }, "Completed", sch_completed },
    { { 3, 5 }, { 0, 0 }, "CompletedActionStatus", sch_completedactionstatus },
    { { 3, 5 }, { 0, 0 }, "CompleteDatafeed", sch_completedatafeed },
    { { 3, 0 }, { 0, 0 }, "CompoundPriceSpecification", sch_compoundpricespecification },
    { { 3, 0 }, { 0, 0 }, "ComputerLanguage", sch_computerlanguage },
    { { 2, 0 }, { 0, 0 }, "ComputerStore", sch_computerstore },
    { { 2, 0 }, { 0, 0 }, "ConfirmAction", sch_confirmaction },
    { { 3, 5 }, { 0, 0 }, "Consortium", sch_consortium },
    { { 2, 0 }, { 0, 0 }, "ConsumeAction", sch_consumeaction },
    { { 2, 0 }, { 0, 0 }, "ContactPage", sch_contactpage },
    { { 2, 0 }, { 0, 0 }, "ContactPoint", sch_contactpoint },
    { { 2, 0 }, { 0, 0 }, "ContactPointOption", sch_contactpointoption, s_schema, SF_ENUMERATION },
    { { 5, 0 }, { 0, 0 }, "ContagiousnessHealthAspect", sch_contagiousnesshealthaspect },
    { { 2, 0 }, { 0, 0 }, "Continent", sch_continent },
    { { 2, 0 }, { 0, 0 }, "ControlAction", sch_controlaction },
    { { 2, 0 }, { 0, 0 }, "ConvenienceStore", sch_conveniencestore },
    { { 3, 0 }, { 0, 0 }, "Conversation", sch_conversation },
    { { 2, 0 }, { 0, 0 }, "CookAction", sch_cookaction },
    { { 3, 5 }, { 0, 0 }, "Coop", sch_coop },
    { { 2, 0 }, { 0, 0 }, "Corporation", sch_corporation },
    { { 3, 5 }, { 0, 0 }, "CorrectionComment", sch_correctioncomment },
    { { 2, 0 }, { 0, 0 }, "Country", sch_country },
    { { 3, 2 }, { 0, 0 }, "Course", sch_course },
    { { 3, 2 }, { 0, 0 }, "CourseInstance", sch_courseinstance },
    { { 2, 0 }, { 0, 0 }, "CourtHouse", sch_courthouse },
    { { 3, 5 }, { 0, 0 }, "CoverArt", sch_coverart },
    { { 7, 0 }, { 0, 0 }, "CovidTestingFacility", sch_covidtestingfacility },
    { { 2, 0 }, { 0, 0 }, "CreateAction", sch_createaction },
    { { 2, 0 }, { 0, 0 }, "CreativeWork", sch_creativework },
    { { 2, 0 }, { 0, 0 }, "CreativeWorkSeason", sch_creativeworkseason },
    { { 2, 0 }, { 0, 0 }, "CreativeWorkSeries", sch_creativeworkseries },
    { { 2, 0 }, { 0, 0 }, "CreditCard", sch_creditcard },
    { { 2, 0 }, { 0, 0 }, "Crematorium", sch_crematorium },
    { { 3, 5 }, { 0, 0 }, "CriticReview", sch_criticreview },
    { { 3, 6 }, { 6, 0 }, "CrossSectional", sch_crosssectional },
    { { 3, 5 }, { 0, 0 }, "CssSelectorType", sch_cssselectortype },
    { { 3, 6 }, { 6, 0 }, "CT", sch_ct },
    { { 3, 1 }, { 0, 0 }, "CurrencyConversionService", sch_currencyconversionservice },
    { { 3, 5 }, { 0, 0 }, "DamagedCondition", sch_damagedcondition },
    { { 2, 0 }, { 0, 0 }, "DanceEvent", sch_danceevent },
    { { 2, 0 }, { 0, 0 }, "DanceGroup", sch_dancegroup },
    { { 2, 0 }, { 0, 0 }, "DataCatalog", sch_datacatalogue },
    { { 2, 0 }, { 0, 0 }, "DataDownload", sch_datadownload },
    { { 3, 0 }, { 0, 0 }, "DataFeed", sch_datafeed },
    { { 3, 0 }, { 0, 0 }, "DataFeedItem", sch_datafeeditem },
    { { 2, 0 }, { 0, 0 }, "Dataset", sch_dataset },
    { { 2, 0 }, { 0, 0 }, "DataType", sch_datatype },
    { { 2, 0 }, { 0, 0 }, "Date", sch_date, s_schema, MAKE_SIMPLE_TYPE (t_just_date) },
    { { 2, 0 }, { 3, 4 }, "DatedMoneySpecification", sch_datedmoneyspecification },
    { { 2, 0 }, { 0, 0 }, "DateTime", sch_datetime, s_schema, MAKE_SIMPLE_TYPE (t_datetime) },
    { { 2, 0 }, { 0, 0 }, "DayOfWeek", sch_dayofweek, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "DaySpa", sch_dayspa },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "DDxElement", sch_ddxelement },
    { { 2, 0 }, { 0, 0 }, "DeactivateAction", sch_deactivateaction },
    { { 12, 0 }, { 0, 0 }, "DecontextualizedContent", sch_decontextualisedcontent },
    { { 2, 0 }, { 0, 0 }, "DefenceEstablishment", sch_defenceestablishment },
    { { 7, 4 }, { 0, 0 }, "DefinedRegion", sch_definedregion },
    { { 3, 5 }, { 0, 0 }, "DefinedTerm", sch_definedterm },
    { { 3, 5 }, { 0, 0 }, "DefinedTermSet", sch_definedtermset },
    { { 3, 5 }, { 0, 0 }, "DefinitiveLegalValue", sch_definitivelegalvalue },
    { { 2, 0 }, { 0, 0 }, "DeleteAction", sch_deleteaction },
    { { 2, 0 }, { 0, 0 }, "DeliveryChargeSpecification", sch_deliverychargespecification },
    { { 2, 0 }, { 0, 0 }, "DeliveryEvent", sch_deliveryevent },
    { { 2, 0 }, { 0, 0 }, "DeliveryMethod", sch_deliverymethod, s_schema, SF_ENUMERATION },
    { { 8, 0 }, { 0, 0 }, "DeliveryTimeSettings", sch_deliverytimesettings },
    { { 2, 0 }, { 0, 0 }, "Demand", sch_demand },
    { { 3, 5 }, { 0, 0 }, "DemoAlbum", sch_demoalbum },
    { { 2, 0 }, { 0, 0 }, "Dentist", sch_dentist },
    { { 3, 6 }, { 6, 0 }, "Dentistry", sch_dentistry },
    { { 2, 0 }, { 0, 0 }, "DepartAction", sch_departaction },
    { { 2, 0 }, { 0, 0 }, "DepartmentStore", sch_departmentstore },
    { { 3, 0 }, { 0, 0 }, "DepositAccount", sch_depositaccount },
    { { 3, 5 }, { 0, 0 }, "Dermatology", sch_dermatology },
    { { 3, 5 }, { 0, 0 }, "DiabeticDiet", sch_diabeticdiet },
    { { 3, 6 }, { 6, 0 }, "Diagnostic", sch_diagnostic },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "DiagnosticLab", sch_diagnosticlab },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "DiagnosticProcedure", sch_diagnosticprocedure },
    { { 2, 0 }, { 0, 0 }, "Diet", sch_diet },
    { { 2, 0 }, { 0, 0 }, "DietarySupplement", sch_dietarysupplement },
    { { 3, 5 }, { 0, 0 }, "DietNutrition", sch_dietnutrition },
    { { 3, 5 }, { 0, 0 }, "DigitalAudioTapeFormat", sch_digitalaudiotapeformat },
    { { 3, 0 }, { 0, 0 }, "DigitalDocument", sch_digitaldocument },
    { { 3, 0 }, { 0, 0 }, "DigitalDocumentPermission", sch_digitaldocumentpermission },
    { { 3, 0 }, { 0, 0 }, "DigitalDocumentPermissionType", sch_digitaldocumentpermissiontype, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "DigitalFormat", sch_digitalformat },
    { { 7, 4 }, { 0, 0 }, "DisabilitySupport", sch_disabilitysupport },
    { { 2, 0 }, { 0, 0 }, "DisagreeAction", sch_disagreeaction },
    { { 3, 5 }, { 0, 0 }, "Discontinued", sch_discontinued },
    { { 2, 0 }, { 0, 0 }, "DiscoverAction", sch_discoveraction },
    { { 2, 1 }, { 0, 0 }, "DiscussionForumPosting", sch_discussionforumposting },
    { { 2, 0 }, { 0, 0 }, "DislikeAction", sch_dislikeaction },
    { { 2, 0 }, { 0, 0 }, "Distance", sch_distance },
    { { 11, 0 }, { 0, 0 }, "DistanceFee", sch_distancefee },
    { { 3, 5 }, { 0, 0 }, "Distillery", sch_distillery },
    { { 3, 5 }, { 0, 0 }, "DJMixAlbum", sch_djmixalbum },
    { { 2, 0 }, { 0, 0 }, "DonateAction", sch_donateaction },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "DoseSchedule", sch_doseschedule },
    { { 3, 5 }, { 0, 0 }, "DoubleBlindedTrial", sch_doubleblindedtrial },
    { { 2, 0 }, { 0, 0 }, "DownloadaAtion", sch_downloadaction },
    { { 11, 0 }, { 0, 0 }, "Downpayment", sch_downpayment },
    { { 2, 0 }, { 0, 0 }, "DrawAction", sch_drawaction },
    { { 3, 5 }, { 0, 0 }, "Drawing", sch_drawing },
    { { 2, 0 }, { 0, 0 }, "DrinkAction", sch_drinkaction },
    { { 2, 0 }, { 0, 0 }, "DriveWheelConfigurationValue", sch_drivewheelconfigurationvalue, s_schema, SF_ENUMERATION },
    { { 3, 6 }, { 6, 0 }, "DrivingSchoolVehicleUsage", sch_drivingschoolvehicleusage },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Drug", sch_drug },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "DrugClass", sch_drugclass },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "DrugCost", sch_drugcost },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "DrugCostCategory", sch_drugcostcategory, s_schema, SF_ENUMERATION },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "DrugLegalStatus", sch_druglegalstatus },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "DrugPregnancyCategory", sch_drugpregnancycategory, s_schema, SF_ENUMERATION },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "DrugPrescriptionStatus", sch_drugprescriptionstatus },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "DrugStrength", sch_drugstrength },
    { { 2, 0 }, { 0, 0 }, "DryCleaningOrLaundry", sch_drycleaningorlaundry },
    { { 2, 0 }, { 0, 0 }, "Duration", sch_duration, s_schema, MAKE_SIMPLE_TYPE (t_duration) },
    { { 3, 5 }, { 0, 0 }, "DVDFormat", sch_dvdformat },
    { { 3, 6 }, { 6, 0 }, "Ear", sch_ear },
    { { 2, 0 }, { 0, 0 }, "EatAction", sch_eataction },
    { { 3, 5 }, { 0, 0 }, "EBook", sch_ebook },
    { { 12, 0 }, { 0, 0 }, "EditedOrCroppedContent", sch_editedorcroppedcontent },
    { { 2, 0 }, { 0, 0 }, "EducationalAudience", sch_educationalaudience },
    { { 3, 5 }, { 0, 0 }, "EducationalOccupationalCredential", sch_educationaloccupationalcredential },
    { { 3, 9 }, { 0, 0 }, "EducationalOccupationalProgram", sch_educationaloccupationalprogramme },
    { { 2, 0 }, { 0, 0 }, "EducationalOrganization", sch_educationalorganisation },
    { { 2, 0 }, { 0, 0 }, "EducationEvent", sch_educationevent },
    { { 12, 0 }, { 0, 0 }, "EffectivenessHealthAspect", sch_effectivenesshealthaspect },
    { { 2, 0 }, { 0, 0 }, "Electrician", sch_electrician },
    { { 2, 0 }, { 0, 0 }, "ElectronicsStore", sch_electronicsstore },
    { { 2, 0 }, { 0, 0 }, "ElementarySchool", sch_elementaryschool },
    { { 2, 0 }, { 0, 0 }, "EmailMessage", sch_emailmessage },
    { { 2, 0 }, { 0, 0 }, "Embassy", sch_embassy },
    { { 3, 5 }, { 0, 0 }, "Emergency", sch_emergency },
    { { 2, 0 }, { 0, 0 }, "EmergencyService", sch_emergencyservice },
    { { 2, 0 }, { 0, 0 }, "EmployeeRole", sch_employeerole },
    { { 3, 5 }, { 0, 0 }, "EmployerAggregateRating", sch_employeraggregaterating },
    { { 3, 5 }, { 0, 0 }, "EmployerReview", sch_employerreview },
    { { 2, 0 }, { 0, 0 }, "EmploymentAgency", sch_employmentagency },
    { { 3, 6 }, { 6, 0 }, "Endocrine", sch_endocrine },
    { { 2, 0 }, { 0, 0 }, "EndorseAction", sch_endorseaction },
    { { 3, 5 }, { 0, 0 }, "EndorsementRating", sch_endorsementrating },
    { { 2, 0 }, { 0, 0 }, "Energy", sch_energy },
    { { 10, 0 }, { 0, 0 }, "EnergyConsumptionDetails", sch_energyconsumptiondetails },
    { { 10, 0 }, { 0, 0 }, "EnergyEfficiencyEnumeration", sch_energyefficiencyenumeration, s_schema, SF_ENUMERATION },
    { { 10, 0 }, { 0, 0 }, "EnergyStarCertified", sch_energystarcertified },
    { { 2, 0 }, { 0, 0 }, "EngineSpecification", sch_enginespecification },
    { { 3, 6 }, { 6, 0 }, "EnrollingByInvitation", sch_enrollingbyinvitation },
    { { 2, 0 }, { 0, 0 }, "EntertainmentBusiness", sch_entertainmentbusiness },
    { { 2, 0 }, { 0, 0 }, "EntryPoint", sch_entrypoint },
    { { 2, 0 }, { 0, 0 }, "Enumeration", sch_enumeration },
    { { 2, 0 }, { 0, 0 }, "Episode", sch_episode },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryA", sch_euenergyefficiencycategorya },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryA1Plus", sch_euenergyefficiencycategorya1plus },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryA2Plus", sch_euenergyefficiencycategorya2plus },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryA3Plus", sch_euenergyefficiencycategorya3plus },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryB", sch_euenergyefficiencycategoryb },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryC", sch_euenergyefficiencycategoryc },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryD", sch_euenergyefficiencycategoryd },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryE", sch_euenergyefficiencycategorye },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryF", sch_euenergyefficiencycategoryf },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryG", sch_euenergyefficiencycategoryg },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyEnumeration", sch_euenergyefficiencyenumeration, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "EPRelease", sch_eprelease },
    { { 2, 0 }, { 0, 0 }, "Event", sch_event },
    { { 7, 0 }, { 0, 0 }, "EventAttendanceModeEnumeration", sch_eventattendancemodeenumeration, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "EventCancelled", sch_eventcancelled },
    { { 7, 0 }, { 0, 0 }, "EventMovedOnline", sch_eventmovedonline },
    { { 3, 5 }, { 0, 0 }, "EventPostponed", sch_eventpostponed },
    { { 3, 5 }, { 0, 0 }, "EventRescheduled", sch_eventrescheduled },
    { { 2, 0 }, { 0, 0 }, "EventReservation", sch_eventreservation },
    { { 3, 5 }, { 0, 0 }, "EventScheduled", sch_eventscheduled },
    { { 3, 5 }, { 0, 0 }, "EventSeries", sch_eventseries },
    { { 2, 0 }, { 0, 0 }, "EventStatusType", sch_eventstatustype, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "EventVenue", sch_eventvenue },
    { { 3, 6 }, { 6, 0 }, "EvidenceLevelA", sch_evidencelevela },
    { { 3, 6 }, { 6, 0 }, "EvidenceLevelB", sch_evidencelevelb },
    { { 3, 6 }, { 6, 0 }, "EvidenceLevelC", sch_evidencelevelc },
    { { 3, 5 }, { 0, 0 }, "ExchangeRateSpecification", sch_exchangeratespecification },
    { { 3, 8 }, { 0, 0 }, "ExchangeRefund", sch_exchangerefund },
    { { 2, 0 }, { 0, 0 }, "ExerciseAction", sch_exerciseaction },
    { { 2, 0 }, { 0, 0 }, "ExerciseGym", sch_exercisegym },
    { { 2, 0 }, { 0, 0 }, "ExercisePlan", sch_exerciseplan },
    { { 2, 2 }, { 0, 0 }, "ExhibitionEvent", sch_exhibitionevent },
    { { 3, 6 }, { 6, 0 }, "Eye", sch_eye },
    { { 3, 5 }, { 0, 0 }, "FailedActionStatus", sch_failedactionstatus },
    { { 2, 0 }, { 0, 0 }, "False", sch_false },
    { { 3, 5 }, { 0, 0 }, "FAQPage", sch_faqpage },
    { { 2, 0 }, { 0, 0 }, "FastFoodRestaurant", sch_fastfoodrestaurant },
    { { 3, 6 }, { 6, 0 }, "FDACategoryA", sch_fdacategorya },
    { { 3, 6 }, { 6, 0 }, "FDACategoryB", sch_fdacategoryb },
    { { 3, 6 }, { 6, 0 }, "FDACategoryC", sch_fdacategoryc },
    { { 3, 6 }, { 6, 0 }, "FDACategoryD", sch_fdacategoryd },
    { { 3, 6 }, { 6, 0 }, "FDACategoryX", sch_fdacategoryx },
    { { 3, 6 }, { 6, 0 }, "FDANotEvaluated", sch_fdanotevaluated },
    { { 3, 5 }, { 0, 0 }, "Female", sch_female },
    { { 2, 0 }, { 0, 0 }, "Festival", sch_festival },
    { { 2, 0 }, { 0, 0 }, "FilmAction", sch_filmaction },
    { { 3, 0 }, { 0, 0 }, "FinancialProduct", sch_financialproduct },
    { { 2, 0 }, { 0, 0 }, "FinancialService", sch_financialservice },
    { { 2, 0 }, { 0, 0 }, "FindAction", sch_findaction },
    { { 2, 0 }, { 0, 0 }, "FireStation", sch_firestation },
    { { 3, 5 }, { 0, 0 }, "Flexibility", sch_flexibility },
    { { 2, 0 }, { 0, 0 }, "Flight", sch_flight },
    { { 2, 0 }, { 0, 0 }, "FlightReservation", sch_flightreservation },
    { { 2, 0 }, { 0, 0 }, "Float", sch_float, s_schema, MAKE_SIMPLE_TYPE (t_real) },
    { { 6, 0 }, { 0, 0 }, "FloorPlan", sch_floorplan },
    { { 2, 0 }, { 0, 0 }, "Florist", sch_florist },
    { { 3, 5 }, { 0, 0 }, "FMRadioChannel", sch_fmradiochannel },
    { { 2, 0 }, { 0, 0 }, "FollowAction", sch_followaction },
    { { 2, 0 }, { 0, 0 }, "FoodEstablishment", sch_foodestablishment },
    { { 2, 0 }, { 0, 0 }, "FoodEstablishmentReservation", sch_foodestablishmentreservation },
    { { 2, 0 }, { 0, 0 }, "FoodEvent", sch_foodevent },
    { { 3, 1 }, { 0, 0 }, "FoodService", sch_foodservice },
    { { 3, 6 }, { 6, 0 }, "FourWheelDriveConfiguration", sch_fourwheeldriveconfiguration },
    { { 13, 0 }, { 0, 0 }, "FreeReturn", sch_freereturn },
    { { 3, 5 }, { 0, 0 }, "Friday", sch_friday },
    { { 3, 6 }, { 6, 0 }, "FrontWheelDriveConfiguration", sch_frontwheeldriveconfiguration },
    { { 3, 8 }, { 0, 0 }, "FullRefund", sch_fullrefund },
    { { 3, 5 }, { 0, 0 }, "FundingAgency", sch_fundingagency },
    { { 3, 5 }, { 0, 0 }, "FundingScheme", sch_fundingscheme },
    { { 3, 6 }, { 6, 0 }, "Fungus", sch_fungus },
    { { 2, 0 }, { 0, 0 }, "FurnitureStore", sch_furniturestore },
    { { 2, 0 }, { 0, 0 }, "Game", sch_game },
    { { 2, 0 }, { 0, 0 }, "GamePlayMode", sch_gameplaymode, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "GameServer", sch_gameserver },
    { { 2, 0 }, { 0, 0 }, "GameServerStatus", sch_gameserverstatus, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "GardenStore", sch_gardenstore },
    { { 2, 0 }, { 0, 0 }, "GasStation", sch_gasstation },
    { { 3, 6 }, { 6, 0 }, "Gastroenterologic", sch_gastroenterologic },
    { { 2, 0 }, { 0, 0 }, "GatedResidenceCommunity", sch_gatedresidencecommunity },
    { { 3, 0 }, { 0, 0 }, "GenderType", sch_gendertype, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "GeneralContractor", sch_generalcontractor },
    { { 13, 0 }, { 0, 0 }, "Gene", sch_gene },
    { { 3, 6 }, { 6, 0 }, "Genetic", sch_genetic },
    { { 3, 6 }, { 6, 0 }, "Genitourinary", sch_genitourinary },
    { { 2, 2 }, { 0, 0 }, "GeoCircle", sch_geocircle },
    { { 2, 0 }, { 0, 0 }, "GeoCoordinates", sch_geocoordinates },
    { { 2, 0 }, { 0, 0 }, "GeoShape", sch_geoshape },
    { { 3, 5 }, { 0, 0 }, "GeospatialGeometry", sch_geospatialgeometry },
    { { 3, 5 }, { 0, 0 }, "Geriatric", sch_geriatric },
    { { 12, 0 }, { 0, 0 }, "GettingAccessHealthAspect", sch_gettingaccesshealthaspect },
    { { 2, 0 }, { 0, 0 }, "GiveAction", sch_giveaction },
    { { 3, 5 }, { 0, 0 }, "GlutenFreeDiet", sch_glutenfreediet },
    { { 2, 0 }, { 0, 0 }, "GolfCourse", sch_golfcourse },
    { { 7, 4 }, { 0, 0 }, "GovernmentBenefitsType", sch_governmentbenefitstype, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "GovernmentBuilding", sch_governmentbuilding },
    { { 2, 0 }, { 0, 0 }, "GovernmentOffice", sch_governmentoffice },
    { { 2, 0 }, { 0, 0 }, "GovernmentOrganization", sch_governmentorganisation },
    { { 2, 0 }, { 0, 0 }, "GovernmentPermit", sch_governmentpermit },
    { { 2, 0 }, { 0, 0 }, "GovernmentService", sch_governmentservice },
    { { 3, 5 }, { 0, 0 }, "Grant", sch_grant },
    { { 3, 5 }, { 0, 0 }, "GraphicNovel", sch_graphicnovel },
    { { 2, 0 }, { 0, 0 }, "GroceryStore", sch_grocerystore },
    { { 3, 5 }, { 0, 0 }, "GroupBoardingPolicy", sch_groupboardingpolicy },
    { { 6, 0 }, { 0, 0 }, "Guide", sch_guide },
    { { 3, 5 }, { 0, 0 }, "Gynecologic", sch_gynaecologic },
    { { 7, 4 }, { 0, 0 }, "Hackathon", sch_hackathon },
    { { 2, 0 }, { 0, 0 }, "HairSalon", sch_hairsalon },
    { { 3, 5 }, { 0, 0 }, "HalalDiet", sch_halaldiet },
    { { 3, 5 }, { 0, 0 }, "Hardcover", sch_hardcover },
    { { 2, 0 }, { 0, 0 }, "HardwareStore", sch_hardwarestore },
    { { 3, 6 }, { 6, 0 }, "Head", sch_head },
    { { 2, 0 }, { 0, 0 }, "HealthAndBeautyBusiness", sch_healthandbeautybusiness },
    { { 5, 0 }, { 0, 0 }, "HealthAspectEnumeration", sch_healthaspectenumeration, s_schema, SF_ENUMERATION },
    { { 7, 4 }, { 0, 0 }, "HealthCare", sch_healthcare },
    { { 2, 0 }, { 0, 0 }, "HealthClub", sch_healthclub },
    { { 3, 5 }, { 0, 0 }, "HealthInsurancePlan", sch_dystopianhealthinsuranceplan },
    { { 3, 5 }, { 0, 0 }, "HealthPlanCostSharingSpecification", sch_dystopianhealthplancostsharingspecification },
    { { 3, 5 }, { 0, 0 }, "HealthPlanFormulary", sch_dystopianhealthplanformulary },
    { { 3, 5 }, { 0, 0 }, "HealthPlanNetwork", sch_dystopianhealthplannetwork },
    { { 5, 0 }, { 0, 0 }, "HealthTopicContent", sch_healthtopiccontent },
    { { 3, 5 }, { 0, 0 }, "HearingImpairedSupported", sch_hearingimpairedsupported },
    { { 3, 6 }, { 6, 0 }, "Hematologic", sch_haematologic },
    { { 2, 0 }, { 0, 0 }, "HighSchool", sch_highschool },
    { { 3, 5 }, { 0, 0 }, "HinduDiet", sch_hindudiet },
    { { 2, 0 }, { 0, 0 }, "HinduTemple", sch_hindutemple },
    { { 2, 0 }, { 0, 0 }, "HobbyShop", sch_hobbyshop },
    { { 2, 0 }, { 0, 0 }, "HomeAndConstructionBusiness", sch_homeandconstructionbusiness },
    { { 2, 0 }, { 0, 0 }, "HomeGoodsStore", sch_homegoodsstore },
    { { 3, 6 }, { 6, 0 }, "Homeopathic", sch_magic },
    { { 2, 0 }, { 0, 0 }, "Hospital", sch_hospital },
    { { 2, 0 }, { 0, 0 }, "Hostel", sch_hostel },
    { { 2, 0 }, { 0, 0 }, "Hotel", sch_hotel },
    { { 3, 1 }, { 0, 0 }, "HotelRoom", sch_hotelroom },
    { { 3, 1 }, { 0, 0 }, "House", sch_house },
    { { 2, 0 }, { 0, 0 }, "HousePainter", sch_housepainter },
    { { 12, 0 }, { 0, 0 }, "HowItWorksHealthAspect", sch_howitworkshealthaspect },
    { { 5, 0 }, { 0, 0 }, "HowOrWhereHealthAspect", sch_howorwherehealthaspect },
    { { 3, 3 }, { 0, 0 }, "HowTo", sch_howto },
    { { 3, 3 }, { 0, 0 }, "HowToDirection", sch_howtodirection },
    { { 3, 3 }, { 0, 0 }, "HowToItem", sch_howtoitem },
    { { 3, 3 }, { 0, 0 }, "HowToSection", sch_howtosection },
    { { 3, 3 }, { 0, 0 }, "HowToStep", sch_howtostep },
    { { 3, 3 }, { 0, 0 }, "HowToSupply", sch_howtosupply },
    { { 3, 3 }, { 0, 0 }, "HowToTip", sch_howtotip },
    { { 3, 3 }, { 0, 0 }, "HowToTool", sch_howtotool },
    { { 2, 0 }, { 0, 0 }, "HVACBusiness", sch_hvacbusiness },
    { { 11, 0 }, { 0, 0 }, "HyperToc", sch_hypertoc },
    { { 11, 0 }, { 0, 0 }, "HyperTocEntry", sch_hypertocentry },
    { { 2, 0 }, { 0, 0 }, "IceCreamShop", sch_icecreamshop },
    { { 2, 0 }, { 0, 0 }, "IgnoreAction", sch_ignoreaction },
    { { 2, 0 }, { 0, 0 }, "ImageGallery", sch_imagegallery },
    { { 2, 0 }, { 0, 0 }, "ImageObject", sch_imageobject },
    { { 13, 0 }, { 0, 0 }, "ImageObjectSnapshot", sch_imageobjectsnapshot },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "ImagingTest", sch_imagingtest },
    { { 2, 0 }, { 0, 0 }, "IndividualProduct", sch_individualproduct },
    { { 3, 6 }, { 6, 0 }, "Infectious", sch_infectious },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "InfectiousAgentClass", sch_infectiousagentclass, s_schema, SF_ENUMERATION },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "InfectiousDisease", sch_infectiousdisease },
    { { 3, 5 }, { 0, 0 }, "InForce", sch_inforce },
    { { 2, 0 }, { 0, 0 }, "InformAction", sch_informaction },
    { { 12, 0 }, { 0, 0 }, "IngredientsHealthAspect", sch_ingredientshealthaspect },
    { { 2, 0 }, { 0, 0 }, "InsertAction", sch_insertaction },
    { { 2, 0 }, { 0, 0 }, "InstallAction", sch_installaction },
    { { 11, 0 }, { 0, 0 }, "Installment", sch_installment },
    { { 3, 5 }, { 0, 0 }, "InStock", sch_instock },
    { { 3, 5 }, { 0, 0 }, "InStoreOnly", sch_instoreonly },
    { { 2, 0 }, { 0, 0 }, "InsuranceAgency", sch_insuranceagency },
    { { 2, 0 }, { 0, 0 }, "Intangible", sch_intangible },
    { { 2, 0 }, { 0, 0 }, "Integer", sch_integer, s_schema, MAKE_SIMPLE_TYPE (t_integer) },
    { { 2, 0 }, { 0, 0 }, "InteractAction", sch_interactaction },
    { { 2, 1 }, { 0, 0 }, "InteractionCounter", sch_interactioncounter },
    { { 3, 5 }, { 0, 0 }, "InternationalTrial", sch_internationaltrial },
    { { 2, 0 }, { 0, 0 }, "InternetCafe", sch_internetcafe },
    { { 3, 5 }, { 0, 0 }, "InvestmentFund", sch_investmentfund },
    { { 3, 1 }, { 0, 0 }, "InvestmentOrDeposit", sch_investmentordeposit },
    { { 2, 0 }, { 0, 0 }, "InviteAction", sch_inviteaction },
    { { 2, 0 }, { 0, 0 }, "Invoice", sch_invoice },
    { { 11, 0 }, { 0, 0 }, "InvoicePrice", sch_invoiceprice },
    { { 2, 0 }, { 0, 0 }, "ItemAvailability", sch_itemavailability, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "ItemList", sch_itemlist },
    { { 3, 5 }, { 0, 0 }, "ItemListOrderAscending", sch_itemlistorderascending },
    { { 3, 5 }, { 0, 0 }, "ItemListOrderDescending", sch_itemlistorderdescending },
    { { 2, 0 }, { 0, 0 }, "ItemListOrderType", sch_itemlistordertype, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "ItemListUnordered", sch_itemlistunordered },
    { { 2, 0 }, { 0, 0 }, "ItemPage", sch_itempage },
    { { 2, 0 }, { 0, 0 }, "JewelryStore", sch_jewelrystore },
    { { 2, 0 }, { 0, 0 }, "JobPosting", sch_jobposting },
    { { 2, 0 }, { 0, 0 }, "JoinAction", sch_joinaction },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Joint", sch_joint },
    { { 3, 5 }, { 0, 0 }, "KosherDiet", sch_kosherdiet },
    { { 3, 6 }, { 6, 0 }, "LaboratoryScience", sch_laboratoryscience },
    { { 2, 0 }, { 0, 0 }, "LakeBodyOfWater", sch_lakebodyofwater },
    { { 2, 0 }, { 0, 0 }, "Landform", sch_landform },
    { { 2, 0 }, { 0, 0 }, "LandmarksOrHistoricalBuildings", sch_landmarksorhistoricalbuildings },
    { { 2, 0 }, { 0, 0 }, "Language", sch_language },
    { { 3, 5 }, { 0, 0 }, "LaserDiscFormat", sch_laserdiscformat },
    { { 9, 0 }, { 0, 0 }, "LearningResource", sch_learningresource },
    { { 2, 0 }, { 0, 0 }, "LeaveAction", sch_leaveaction },
    { { 3, 6 }, { 6, 0 }, "LeftHandDriving", sch_lefthanddriving },
    { { 3, 5 }, { 0, 0 }, "LegalForceStatus", sch_legalforcestatus, s_schema, SF_ENUMERATION },
    { { 2, 2 }, { 0, 0 }, "LegalService", sch_legalservice },
    { { 3, 5 }, { 0, 0 }, "LegalValueLevel", sch_legalvaluelevel, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "Legislation", sch_legislation },
    { { 3, 5 }, { 0, 0 }, "LegislationObject", sch_legislationobject },
    { { 2, 0 }, { 0, 0 }, "LegislativeBuilding", sch_legislativebuilding },
    { { 3, 5 }, { 0, 0 }, "LeisureTimeActivity", sch_leisuretimeactivity },
    { { 2, 0 }, { 0, 0 }, "LendAction", sch_lendaction },
    { { 2, 0 }, { 0, 0 }, "Library", sch_library },
    { { 3, 5 }, { 0, 0 }, "LibrarySystem", sch_librarysystem },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "LifestyleModification", sch_lifestylemodification },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Ligament", sch_ligament },
    { { 2, 0 }, { 0, 0 }, "LikeAction", sch_likeaction },
    { { 3, 5 }, { 0, 0 }, "LimitedAvailability", sch_limitedavailability },
    { { 9, 0 }, { 0, 0 }, "LimitedByGuaranteeCharity", sch_limitedbyguaranteecharity },
    { { 3, 5 }, { 0, 0 }, "LinkRole", sch_linkrole },
    { { 2, 0 }, { 0, 0 }, "LiquorStore", sch_liquorstore },
    { { 2, 0 }, { 0, 0 }, "ListenAction", sch_listenaction },
    { { 2, 0 }, { 0, 0 }, "ListItem", sch_listitem },
    { { 11, 0 }, { 0, 0 }, "ListPrice", sch_listprice },
    { { 2, 0 }, { 0, 0 }, "LiteraryEvent", sch_literaryevent },
    { { 3, 5 }, { 0, 0 }, "LiveAlbum", sch_livealbum },
    { { 2, 1 }, { 0, 0 }, "LiveBlogPosting", sch_liveblogposting },
    { { 5, 0 }, { 0, 0 }, "LivingWithHealthAspect", sch_livingwithhealthaspect },
    { { 3, 0 }, { 0, 0 }, "LoanOrCredit", sch_loanorcredit },
    { { 2, 0 }, { 0, 0 }, "LocalBusiness", sch_localbusiness },
    { { 3, 1 }, { 0, 0 }, "LocationFeatureSpecification", sch_locationfeaturespecification },
    { { 2, 0 }, { 0, 0 }, "LockerDelivery", sch_lockerdelivery },
    { { 2, 0 }, { 0, 0 }, "Locksmith", sch_locksmith },
    { { 2, 0 }, { 0, 0 }, "LodgingBusiness", sch_lodgingbusiness },
    { { 2, 0 }, { 0, 0 }, "LodgingReservation", sch_lodgingreservation },
    { { 3, 6 }, { 6, 0 }, "Longitudinal", sch_longitudinal },
    { { 2, 0 }, { 0, 0 }, "LoseAction", sch_loseaction },
    { { 3, 5 }, { 0, 0 }, "LowCalorieDiet", sch_lowcaloriediet },
    { { 3, 5 }, { 0, 0 }, "LowFatDiet", sch_lowfatdiet },
    { { 3, 5 }, { 0, 0 }, "LowLactoseDiet", sch_lowlactosediet },
    { { 3, 5 }, { 0, 0 }, "LowSaltDiet", sch_lowsaltdiet },
    { { 3, 6 }, { 6, 0 }, "Lung", sch_lung },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "LymphaticVessel", sch_lymphaticvessel },
    { { 3, 5 }, { 0, 0 }, "Male", sch_male },
    { { 3, 5 }, { 0, 0 }, "Manuscript", sch_manuscript },
    { { 2, 0 }, { 0, 0 }, "Map", sch_map },
    { { 2, 0 }, { 0, 0 }, "MapCategoryType", sch_mapcategorytype, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "MarryAction", sch_marryaction },
    { { 2, 0 }, { 0, 0 }, "Mass", sch_mass },
    { { 11, 0 }, { 0, 0 }, "MathSolver", sch_mathsolver },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MaximumDoseSchedule", sch_maximumdoseschedule },
    { { 5, 0 }, { 0, 0 }, "MayTreatHealthAspect", sch_maytreathealthaspect },
    { { 12, 0 }, { 0, 0 }, "MeasurementTypeEnumeration", sch_measurementtypeenumeration, s_schema, SF_ENUMERATION },
    { { 6, 0 }, { 0, 0 }, "MediaGallery", sch_mediagallery },
    { { 7, 0 }, { 0, 0 }, "MediaManipulationRatingEnumeration", sch_mediamanipulationratingenumeration, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "MediaObject", sch_mediaobject },
    { { 7, 0 }, { 0, 0 }, "MediaReview", sch_mediareview },
    { { 13, 0 }, { 0, 0 }, "MediaReviewItem", sch_mediareviewitem },
    { { 3, 5 }, { 0, 0 }, "MediaSubscription", sch_mediasubscription },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalAudience", sch_medicalaudience },
    { { 9, 0 }, { 0, 0 }, "MedicalAudienceType", sch_medicalaudiencetype, s_schema, SF_ENUMERATION },
    { { 3, 0, SV_NOT_3134 }, { 0, 0 }, "MedicalBusiness", sch_medicalbusiness },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalCause", sch_medicalcause },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalClinic", sch_medicalclinic },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalCode", sch_medicalcode },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalCondition", sch_medicalcondition },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalConditionStage", sch_medicalconditionstage },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalContraindication", sch_medicalcontraindication },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalDevice", sch_medicaldevice },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalDevicePurpose", sch_medicaldevicepurpose, s_schema, SF_ENUMERATION },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalEntity", sch_medicalentity },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalEnumeration", sch_medicalenumeration },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalEvidenceLevel", sch_medicalevidencelevel, s_schema, SF_ENUMERATION },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalGuideline", sch_medicalguideline },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalGuidelineContraindication", sch_medicalguidelinecontraindication },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalGuidelineRecommendation", sch_medicalguidelinerecommendation },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalImagingTechnique", sch_medicalimagingtechnique, s_schema, SF_ENUMERATION },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalIndication", sch_medicalindication },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalIntangible", sch_medicalintangible },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalObservationalStudy", sch_medicalobservationalstudy },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalObservationalStudyDesign", sch_medicalobservationalstudydesign, s_schema, SF_ENUMERATION },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalOrganization", sch_medicalorganisation },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalProcedure", sch_medicalprocedure },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalProcedureType", sch_medicalproceduretype, s_schema, SF_ENUMERATION },
    { { 3, 6 }, { 6, 0 }, "MedicalResearcher", sch_medicalresearcher },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalRiskCalculator", sch_medicalriskcalculator },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalRiskEstimator", sch_medicalriskestimator },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalRiskFactor", sch_medicalriskfactor },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalRiskScore", sch_medicalriskscore },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalScholarlyArticle", sch_medicalscholarlyarticle },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalSign", sch_medicalsign },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalSignOrSymptom", sch_medicalsignorsymptom },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalSpecialty", sch_medicalspeciality, s_schema, SF_ENUMERATION },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalStudy", sch_medicalstudy },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalStudyStatus", sch_medicalstudystatus, s_schema, SF_ENUMERATION },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalSymptom", sch_medicalsymptom },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalTest", sch_medicaltest },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalTestPanel", sch_medicaltestpanel },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalTherapy", sch_medicaltherapy },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalTrial", sch_medicaltrial },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalTrialDesign", sch_medicaltrialdesign, s_schema, SF_ENUMERATION },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicalWebPage", sch_medicalwebpage },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "MedicineSystem", sch_medicinesystem, s_schema, SF_ENUMERATION },
    { { 3, 1 }, { 0, 0 }, "MeetingRoom", sch_meetingroom },
    { { 2, 0 }, { 0, 0 }, "MensClothingStore", sch_mensclothingstore },
    { { 3, 2 }, { 0, 0 }, "Menu", sch_menu },
    { { 3, 2 }, { 0, 0 }, "MenuItem", sch_menuitem },
    { { 3, 2 }, { 0, 0 }, "MenuSection", sch_menusection },
    { { 6, 0 }, { 0, 0 }, "MerchantReturnEnumeration", sch_merchantreturnenumeration, s_schema, SF_ENUMERATION },
    { { 6, 0 }, { 0, 0 }, "MerchantReturnFiniteReturnWindow", sch_merchantreturnfinitereturnwindow },
    { { 6, 0 }, { 0, 0 }, "MerchantReturnNotPermitted", sch_merchantreturnnotpermitted },
    { { 6, 0 }, { 0, 0 }, "MerchantReturnPolicy", sch_merchantreturnpolicy },
    { { 13, 0 }, { 0, 0 }, "MerchantReturnPolicySeasonalOverride", sch_merchantreturnpolicyseasonaloverride },
    { { 6, 0 }, { 0, 0 }, "MerchantReturnUnlimitedWindow", sch_merchantreturnunlimitedwindow },
    { { 6, 0 }, { 0, 0 }, "MerchantReturnUnspecified", sch_merchantreturnunspecified },
    { { 3, 0 }, { 0, 0 }, "Message", sch_message },
    { { 2, 0 }, { 0, 0 }, "MiddleSchool", sch_middleschool },
    { { 3, 5 }, { 0, 0 }, "Midwifery", sch_midwifery },
    { { 11, 0 }, { 0, 0 }, "MinimumAdvertisedPrice", sch_minimumadvertisedprice },
    { { 5, 0 }, { 0, 0 }, "MisconceptionsHealthAspect", sch_misconceptionshealthaspect },
    { { 7, 0 }, { 0, 0 }, "MissingContext", sch_missingcontext },
    { { 7, 0 }, { 0, 0 }, "MixedEventAttendanceMode", sch_mixedeventattendancemode },
    { { 3, 5 }, { 0, 0 }, "MixtapeAlbum", sch_mixtapealbum },
    { { 2, 0 }, { 0, 0 }, "MobileApplication", sch_mobileapplication },
    { { 2, 0 }, { 0, 0 }, "MobilePhoneStore", sch_mobilephonestore },
    { { 13, 0 }, { 0, 0 }, "MolecularEntity", sch_molecularentity },
    { { 3, 5 }, { 0, 0 }, "Monday", sch_monday },
    { { 3, 0 }, { 0, 0 }, "MonetaryAmount", sch_monetaryamount },
    { { 3, 5 }, { 0, 0 }, "MonetaryAmountDistribution", sch_monetaryamountdistribution },
    { { 3, 5 }, { 0, 0 }, "MonetaryGrant", sch_monetarygrant },
    { { 3, 5 }, { 0, 0 }, "MoneyTransfer", sch_moneytransfer },
    { { 3, 5 }, { 0, 0 }, "MortgageLoan", sch_mortgageloan },
    { { 2, 0 }, { 0, 0 }, "Mosque", sch_mosque },
    { { 2, 0 }, { 0, 0 }, "Motel", sch_motel },
    { { 3, 5 }, { 0, 0 }, "Motorcycle", sch_motorcycle },
    { { 2, 0 }, { 0, 0 }, "MotorcycleDealer", sch_motorcycledealer },
    { { 2, 0 }, { 0, 0 }, "MotorcycleRepair", sch_motorcyclerepair },
    { { 3, 5 }, { 0, 0 }, "MotorizedBicycle", sch_motorisedbicycle },
    { { 2, 0 }, { 0, 0 }, "Mountain", sch_mountain },
    { { 2, 0 }, { 0, 0 }, "MoveAction", sch_moveaction },
    { { 2, 0 }, { 0, 0 }, "Movie", sch_movie },
    { { 2, 0 }, { 0, 0 }, "MovieClip", sch_movieclip },
    { { 2, 0 }, { 0, 0 }, "MovieRentalStore", sch_movierentalstore },
    { { 2, 0 }, { 0, 0 }, "MovieSeries", sch_movieseries },
    { { 2, 0 }, { 0, 0 }, "MovieTheater", sch_movietheatre },
    { { 2, 0 }, { 0, 0 }, "MovingCompany", sch_movingcompany },
    { { 3, 6 }, { 6, 0 }, "Mri", sch_mri },
    { { 11, 0 }, { 0, 0 }, "MSRP", sch_msrp },
    { { 3, 6 }, { 6, 0 }, "MulticellularParasite", sch_multicellularparasite },
    { { 3, 5 }, { 0, 0 }, "MulticenterTrial", sch_multicentretrial },
    { { 3, 5 }, { 0, 0 }, "Multiplayer", sch_multiplayer },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Muscle", sch_muscle },
    { { 3, 6 }, { 6, 0 }, "Musculoskeletal", sch_musculoskeletal },
    { { 3, 6 }, { 6, 0 }, "MusculoskeletalExam", sch_musculoskeletalexam },
    { { 2, 0 }, { 0, 0 }, "Museum", sch_museum },
    { { 2, 0 }, { 0, 0 }, "MusicAlbum", sch_musicalbum },
    { { 2, 0 }, { 0, 0 }, "MusicAlbumProductionType", sch_musicalbumproductiontype, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "MusicAlbumReleaseType", sch_musicalbumreleasetype, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "MusicComposition", sch_musiccomposition },
    { { 2, 0 }, { 0, 0 }, "MusicEvent", sch_musicevent },
    { { 2, 0 }, { 0, 0 }, "MusicGroup", sch_musicgroup },
    { { 2, 0 }, { 0, 0 }, "MusicPlaylist", sch_musicplaylist },
    { { 2, 0 }, { 0, 0 }, "MusicRecording", sch_musicrecording },
    { { 2, 0 }, { 0, 0 }, "MusicRelease", sch_musicrelease },
    { { 2, 0 }, { 0, 0 }, "MusicReleaseFormatType", sch_musicreleaseformattype, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "MusicStore", sch_musicstore },
    { { 2, 0 }, { 0, 0 }, "MusicVenue", sch_musicvenue },
    { { 2, 0 }, { 0, 0 }, "MusicVideoObject", sch_musicvideoobject },
    { { 2, 0 }, { 0, 0 }, "NailSalon", sch_nailsalon },
    { { 3, 6 }, { 6, 0 }, "Neck", sch_neck },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Nerve", sch_nerve },
    { { 3, 6 }, { 6, 0 }, "Neuro", sch_neuro },
    { { 3, 6 }, { 6, 0 }, "Neurologic", sch_neurologic },
    { { 3, 5 }, { 0, 0 }, "NewCondition", sch_newcondition },
    { { 2, 0 }, { 0, 0 }, "NewsArticle", sch_newsarticle },
    { { 3, 5 }, { 0, 0 }, "NewsMediaOrganization", sch_newsmediaorganisation },
    { { 3, 5 }, { 0, 0 }, "Newspaper", sch_newspaper },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Ngo", sch_ngo },
    { { 2, 0 }, { 0, 0 }, "Nightclub", sch_nightclub },
    { { 8, 0 }, { 0, 0 }, "NlNonprofitType", sch_nlnonprofittype, s_schema, SF_ENUMERATION },
    { { 3, 6 }, { 6, 0 }, "NoninvasiveProcedure", sch_noninvasiveprocedure },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501a", sch_nonprofit501a },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c1", sch_nonprofit501c1 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c10", sch_nonprofit501c10 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c11", sch_nonprofit501c11 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c12", sch_nonprofit501c12 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c13", sch_nonprofit501c13 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c14", sch_nonprofit501c14 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c15", sch_nonprofit501c15 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c16", sch_nonprofit501c16 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c17", sch_nonprofit501c17 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c18", sch_nonprofit501c18 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c19", sch_nonprofit501c19 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c2", sch_nonprofit501c2 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c20", sch_nonprofit501c20 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c21", sch_nonprofit501c21 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c22", sch_nonprofit501c22 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c23", sch_nonprofit501c23 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c24", sch_nonprofit501c24 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c25", sch_nonprofit501c25 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c26", sch_nonprofit501c26 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c27", sch_nonprofit501c27 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c28", sch_nonprofit501c28 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c3", sch_nonprofit501c3 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c4", sch_nonprofit501c4 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c5", sch_nonprofit501c5 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c6", sch_nonprofit501c6 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c7", sch_nonprofit501c7 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c8", sch_nonprofit501c8 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c9", sch_nonprofit501c9 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501d", sch_nonprofit501d },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501e", sch_nonprofit501e },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501f", sch_nonprofit501f },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501k", sch_nonprofit501k },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501n", sch_nonprofit501n },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501q", sch_nonprofit501q },
    { { 9, 0 }, { 0, 0 }, "Nonprofit527", sch_nonprofit527 },
    { { 9, 0 }, { 0, 0 }, "Nonprofitanbi", sch_nonprofitanbi },
    { { 9, 0 }, { 0, 0 }, "Nonprofitsbbi", sch_nonprofitsbbi },
    { { 8, 0 }, { 0, 0 }, "Nonprofittype", sch_nonprofittype },
    { { 3, 6 }, { 6, 0 }, "Nose", sch_nose },
    { { 2, 0 }, { 0, 0 }, "Notary", sch_notary },
    { { 3, 0 }, { 0, 0 }, "NoteDigitalDocument", sch_notedigitaldocument },
    { { 3, 5 }, { 0, 0 }, "NotInForce", sch_notinforce },
    { { 3, 6 }, { 6, 0 }, "NotYetRecruiting", sch_notyetrecruiting },
    { { 2, 0 }, { 0, 0 }, "Number", sch_number, s_schema, MAKE_SIMPLE_TYPE (t_fixedpoint) },
    { { 3, 5 }, { 0, 0 }, "Nursing", sch_nursing },
    { { 2, 0 }, { 0, 0 }, "NutritionInformation", sch_nutritioninformation },
    { { 3, 9 }, { 0, 0 }, "Observation", sch_observation },
    { { 3, 6 }, { 6, 0 }, "Observational", sch_observational },
    { { 3, 5 }, { 0, 0 }, "Obstetric", sch_obstetric },
    { { 3, 5 }, { 0, 0 }, "Occupation", sch_occupation },
    { { 3, 5 }, { 0, 0 }, "OccupationalActivity", sch_occupationalactivity },
    { { 12, 0 }, { 0, 0 }, "OccupationalExperienceRequirements", sch_occupationalexperiencerequirements },
    { { 3, 5 }, { 0, 0 }, "OccupationalTherapy", sch_occupationaltherapy },
    { { 2, 0 }, { 0, 0 }, "OceanBodyOfWater", sch_oceanbodyofwater },
    { { 2, 0 }, { 0, 0 }, "Offer", sch_offer },
    { { 2, 2 }, { 0, 0 }, "OfferCatalog", sch_offercatalogue },
    { { 4, 0 }, { 0, 0 }, "OfferForLease", sch_offerforlease },
    { { 4, 0 }, { 0, 0 }, "OfferForPurchase", sch_offerforpurchase },
    { { 2, 0 }, { 0, 0 }, "OfferItemCondition", sch_offeritemcondition, s_schema, SF_ENUMERATION },
    { { 7, 2 }, { 0, 0 }, "OfferShippingDetails", sch_offershippingdetails },
    { { 2, 0 }, { 0, 0 }, "OfficeEquipmentStore", sch_officeequipmentstore },
    { { 3, 5 }, { 0, 0 }, "OfficiallegalValue", sch_officiallegalvalue },
    { { 7, 0 }, { 0, 0 }, "OfflineEventAttendanceMode", sch_offlineeventattendancemode },
    { { 3, 5 }, { 0, 0 }, "OfflinePermanently", sch_offlinepermanently },
    { { 3, 5 }, { 0, 0 }, "OfflineTemporarily", sch_offlinetemporarily },
    { { 3, 5 }, { 0, 0 }, "Oncologic", sch_oncologic },
    { { 2, 0 }, { 0, 0 }, "OnDemandEvent", sch_ondemandevent },
    { { 7, 4 }, { 0, 0 }, "OneTimePayments", sch_onetimepayments },
    { { 3, 5 }, { 0, 0 }, "Online", sch_online },
    { { 7, 0 }, { 0, 0 }, "OnlineEventAttendanceMode", sch_onlineeventattendancemode },
    { { 3, 5 }, { 0, 0 }, "OnlineFull", sch_onlinefull },
    { { 3, 5 }, { 0, 0 }, "OnlineOnly", sch_onlineonly },
    { { 3, 5 }, { 0, 0 }, "OnsitePickup", sch_onsitepickup },
    { { 2, 0 }, { 0, 0 }, "OpeningHoursSpecification", sch_openinghoursspecification },
    { { 3, 5 }, { 0, 0 }, "OpenTrial", sch_opentrial },
    { { 3, 5 }, { 0, 0 }, "OpinionNewsArticle", sch_opinionnewsarticle },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Optician", sch_optician },
    { { 3, 5 }, { 0, 0 }, "Optometric", sch_optometric },
    { { 2, 0 }, { 0, 0 }, "Order", sch_order },
    { { 2, 0 }, { 0, 0 }, "Orderaction", sch_orderaction },
    { { 3, 5 }, { 0, 0 }, "OrderCancelled", sch_ordercancelled },
    { { 3, 5 }, { 0, 0 }, "OrderDelivered", sch_orderdelivered },
    { { 3, 5 }, { 0, 0 }, "OrderIntransit", sch_orderintransit },
    { { 2, 0 }, { 0, 0 }, "OrderItem", sch_orderitem },
    { { 3, 5 }, { 0, 0 }, "OrderPaymentDue", sch_orderpaymentdue },
    { { 3, 5 }, { 0, 0 }, "OrderPickupAvailable", sch_orderpickupavailable },
    { { 3, 5 }, { 0, 0 }, "OrderProblem", sch_orderproblem },
    { { 3, 5 }, { 0, 0 }, "OrderProcessing", sch_orderprocessing },
    { { 3, 5 }, { 0, 0 }, "OrderReturned", sch_orderreturned },
    { { 2, 0 }, { 0, 0 }, "OrderStatus", sch_orderstatus, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "Organization", sch_organisation },
    { { 2, 0 }, { 0, 0 }, "OrganizationRole", sch_organisationrole },
    { { 2, 0 }, { 0, 0 }, "OrganizeAction", sch_organiseaction },
    { { 12, 0 }, { 0, 0 }, "OriginalMediaContent", sch_originalmediacontent },
    { { 3, 8 }, { 0, 0 }, "OriginalShippingSees", sch_originalshippingfees },
    { { 3, 6 }, { 6, 0 }, "Osteopathic", sch_osteopathic },
    { { 3, 6 }, { 6, 0 }, "Otc", sch_otc },
    { { 3, 5 }, { 0, 0 }, "Otolaryngologic", sch_otolaryngologic },
    { { 2, 0 }, { 0, 0 }, "OutletStore", sch_outletstore },
    { { 3, 5 }, { 0, 0 }, "OutOfStock", sch_outofstock },
    { { 5, 0 }, { 0, 0 }, "OverviewHealthAspect", sch_overviewhealthaspect },
    { { 2, 0 }, { 0, 0 }, "OwnershipInfo", sch_ownershipinfo },
    { { 7, 4 }, { 0, 0 }, "PaidLeave", sch_paidleave },
    { { 2, 0 }, { 0, 0 }, "PaintAction", sch_paintaction },
    { { 2, 0 }, { 0, 0 }, "Painting", sch_painting },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "PalliativeProcedure", sch_palliativeprocedure },
    { { 3, 5 }, { 0, 0 }, "Paperback", sch_paperback },
    { { 2, 0 }, { 0, 0 }, "ParcelDelivery", sch_parceldelivery },
    { { 2, 0 }, { 0, 0 }, "ParcelService", sch_parcelservice },
    { { 7, 4 }, { 0, 0 }, "ParentalSupport", sch_parentalsupport },
    { { 2, 0 }, { 0, 0 }, "ParentAudience", sch_parentaudience },
    { { 2, 0 }, { 0, 0 }, "Park", sch_park },
    { { 2, 0 }, { 0, 0 }, "ParkingFacility", sch_parkingfacility },
    { { 3, 5 }, { 0, 0 }, "ParkingMap", sch_parkingmap },
    { { 3, 5 }, { 0, 0 }, "PartiallyInforce", sch_partiallyinforce },
    { { 3, 6 }, { 6, 0 }, "Pathology", sch_pathology },
    { { 2, 0 }, { 0, 0 }, "PathologyTest", sch_pathologytest },
    { { 3, 5 }, { 0, 0 }, "Patient", sch_patient },
    { { 5, 0 }, { 0, 0 }, "PatientExperienceHealthAspect", sch_patientexperiencehealthaspect },
    { { 2, 0 }, { 0, 0 }, "Pawnshop", sch_pawnshop },
    { { 2, 0 }, { 0, 0 }, "PayAction", sch_payaction },
    { { 3, 5 }, { 0, 0 }, "Paymentautomaticallyapplied", sch_paymentautomaticallyapplied },
    { { 3, 0 }, { 0, 0 }, "Paymentcard", sch_paymentcard },
    { { 2, 0 }, { 0, 0 }, "Paymentchargespecification", sch_paymentchargespecification },
    { { 3, 5 }, { 0, 0 }, "Paymentcomplete", sch_paymentcomplete },
    { { 3, 5 }, { 0, 0 }, "Paymentdeclined", sch_paymentdeclined },
    { { 3, 5 }, { 0, 0 }, "Paymentdue", sch_paymentdue },
    { { 2, 0 }, { 0, 0 }, "Paymentmethod", sch_paymentmethod, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "Paymentpastdue", sch_paymentpastdue },
    { { 3, 1 }, { 0, 0 }, "Paymentservice", sch_paymentservice },
    { { 2, 1 }, { 0, 0 }, "Paymentstatustype", sch_paymentstatustype, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "Pediatric", sch_paediatric },
    { { 2, 0 }, { 0, 0 }, "Peopleaudience", sch_peopleaudience },
    { { 3, 6 }, { 6, 0 }, "Percutaneousprocedure", sch_percutaneousprocedure },
    { { 2, 0 }, { 0, 0 }, "Performaction", sch_performaction },
    { { 2, 0 }, { 0, 0 }, "Performancerole", sch_performancerole },
    { { 2, 0 }, { 0, 0 }, "Performingartstheater", sch_tyreshop },
    { { 2, 0 }, { 0, 0 }, "Performinggroup", sch_performinggroup },
    { { 2, 0 }, { 0, 0 }, "Periodical", sch_periodical },
    { { 2, 0 }, { 0, 0 }, "Permit", sch_permit },
    { { 2, 0 }, { 0, 0 }, "Person", sch_person },
    { { 3, 6 }, { 6, 0 }, "Pet", sch_pet },
    { { 2, 0 }, { 0, 0 }, "Petstore", sch_petstore },
    { { 2, 0 }, { 0, 0 }, "Pharmacy", sch_pharmacy },
    { { 3, 6 }, { 6, 0 }, "PharmacySpeciality", sch_pharmacyspeciality },
    { { 2, 0 }, { 0, 0 }, "Photograph", sch_photograph },
    { { 2, 0 }, { 0, 0 }, "PhotographAction", sch_photographaction },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "PhysicalActivity", sch_physicalactivity },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "PhysicalActivityCategory", sch_physicalactivitycategory, s_schema, SF_ENUMERATION },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "PhysicalExam", sch_physicalexam, s_schema, SF_ENUMERATION },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "PhysicalTherapy", sch_physicaltherapy },
    { { 2, 0 }, { 0, 0 }, "Physician", sch_physician },
    { { 3, 5 }, { 0, 0 }, "Physiotherapy", sch_physiotherapy },
    { { 2, 0 }, { 0, 0 }, "Place", sch_place },
    { { 3, 5 }, { 0, 0 }, "Placebocontrolledtrial", sch_placebocontrolledtrial },
    { { 2, 0 }, { 0, 0 }, "Placeofworship", sch_placeofworship },
    { { 2, 0 }, { 0, 0 }, "Planaction", sch_planaction },
    { { 3, 5 }, { 0, 0 }, "Plasticsurgery", sch_plasticsurgery },
    { { 3, 5 }, { 0, 0 }, "Play", sch_play },
    { { 2, 0 }, { 0, 0 }, "PlayAction", sch_playaction },
    { { 2, 0 }, { 0, 0 }, "Playground", sch_playground },
    { { 2, 0 }, { 0, 0 }, "Plumber", sch_plumber },
    { { 4, 0 }, { 0, 0 }, "PodcastEpisode", sch_podcastepisode },
    { { 4, 0 }, { 0, 0 }, "PodcastSeason", sch_podcastseason },
    { { 4, 0 }, { 0, 0 }, "PodcastSeries", sch_podcastseries },
    { { 3, 5 }, { 0, 0 }, "Podiatric", sch_podiatric },
    { { 2, 0 }, { 0, 0 }, "PoliceStation", sch_policestation },
    { { 2, 0 }, { 0, 0 }, "Pond", sch_pond },
    { { 2, 0 }, { 0, 0 }, "PostalAddress", sch_postaladdress },
    { { 8, 0 }, { 0, 0 }, "PostalcodeRangeSpecification", sch_postalcoderangespecification },
    { { 3, 5 }, { 0, 0 }, "Poster", sch_poster },
    { { 2, 0 }, { 0, 0 }, "PostOffice", sch_postoffice },
    { { 3, 5 }, { 0, 0 }, "Potentialactionstatus", sch_potentialactionstatus },
    { { 12, 0 }, { 0, 0 }, "PregnancyHealthAspect", sch_pregnancyhealthaspect },
    { { 3, 5 }, { 0, 0 }, "Preorder", sch_preorder },
    { { 3, 5 }, { 0, 0 }, "Preorderaction", sch_preorderaction },
    { { 2, 0 }, { 0, 0 }, "Prependaction", sch_prependaction },
    { { 3, 5 }, { 0, 0 }, "Presale", sch_presale },
    { { 2, 0 }, { 0, 0 }, "Preschool", sch_preschool },
    { { 3, 6 }, { 6, 0 }, "Prescriptiononly", sch_prescriptiononly },
    { { 3, 0 }, { 0, 0 }, "Presentationdigitaldocument", sch_presentationdigitaldocument },
    { { 5, 0 }, { 0, 0 }, "Preventionhealthaspect", sch_preventionhealthaspect },
    { { 2, 0 }, { 0, 0 }, "Preventionindication", sch_preventionindication },
    { { 11, 0 }, { 0, 0 }, "PriceComponentTypeEnumeration", sch_pricecomponenttypeenumeration },
    { { 2, 0 }, { 0, 0 }, "PriceSpecification", sch_pricespecification },
    { { 11, 0 }, { 0, 0 }, "PriceTypeEnumeration", sch_pricetypeenumeration },
    { { 3, 5 }, { 0, 0 }, "Primarycare", sch_primarycare },
    { { 3, 6 }, { 6, 0 }, "Prion", sch_prion },
    { { 2, 0 }, { 0, 0 }, "Product", sch_product },
    { { 9, 0 }, { 0, 0 }, "ProductCollection", sch_productcollection },
    { { 9, 0 }, { 0, 0 }, "ProductGroup", sch_productgroup },
    { { 2, 0 }, { 0, 0 }, "ProductModel", sch_productmodel },
    { { 3, 8 }, { 0, 0 }, "ProductReturnEnumeration", sch_productreturnenumeration },
    { { 3, 8 }, { 0, 0 }, "ProductReturnFiniteReturnWindow", sch_productreturnfinitereturnwindow },
    { { 3, 8 }, { 0, 0 }, "ProductReturnNotpermitted", sch_productreturnnotpermitted },
    { { 3, 8 }, { 0, 0 }, "ProductReturnPolicy", sch_productreturnpolicy },
    { { 3, 8 }, { 0, 0 }, "ProductReturnUnlimitedWindow", sch_productreturnunlimitedwindow },
    { { 3, 8 }, { 0, 0 }, "ProductReturnUnspecified", sch_productreturnunspecified },
    { { 2, 0 }, { 0, 0 }, "ProfessionalService", sch_professionalservice },
    { { 2, 0 }, { 0, 0 }, "ProfilePage", sch_profilepage },
    { { 5, 0 }, { 0, 0 }, "PrognosisHealthAspect", sch_prognosishealthaspect },
    { { 2, 0 }, { 0, 0 }, "ProgramMembership", sch_programmemembership },
    { { 3, 5 }, { 0, 0 }, "Project", sch_project },
    { { 6, 0 }, { 0, 0 }, "PronounceableText", sch_pronounceabletext },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Property", sch_property },
    { { 2, 0 }, { 0, 0 }, "PropertyValue", sch_propertyvalue },
    { { 2, 0 }, { 0, 0 }, "PropertyValueSpecification", sch_propertyvaluespecification },
    { { 13, 0 }, { 0, 0 }, "Protein", sch_protein },
    { { 3, 6 }, { 6, 0 }, "Protozoa", sch_protozoa },
    { { 3, 5 }, { 0, 0 }, "Psychiatric", sch_psychiatric },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "PsychologicalTreatment", sch_psychologicaltreatment },
    { { 2, 0 }, { 0, 0 }, "Publicationevent", sch_publicationevent },
    { { 2, 0 }, { 0, 0 }, "Publicationissue", sch_publicationissue },
    { { 2, 0 }, { 0, 0 }, "Publicationvolume", sch_publicationvolume },
    { { 3, 5 }, { 0, 0 }, "Publichealth", sch_publichealth },
    { { 3, 5 }, { 0, 0 }, "Publicholidays", sch_publicholidays },
    { { 2, 0 }, { 0, 0 }, "Publicswimmingpool", sch_publicswimmingpool },
    { { 3, 5 }, { 0, 0 }, "Publictoilet", sch_publictoilet },
    { { 3, 6 }, { 6, 0 }, "Pulmonary", sch_pulmonary },
    { { 2, 0 }, { 0, 0 }, "Qapage", sch_qapage },
    { { 2, 0 }, { 0, 0 }, "QualitativeValue", sch_qualitativevalue },
    { { 2, 0 }, { 0, 0 }, "QuantitativeValue", sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, "QuantitativeValueDistribution", sch_quantitativevaluedistribution },
    { { 2, 0 }, { 0, 0 }, "Quantity", sch_quantity },
    { { 2, 0 }, { 0, 0 }, "Question", sch_question },
    { { 9, 0 }, { 0, 0 }, "Quiz", sch_quiz },
    { { 3, 5 }, { 0, 0 }, "Quotation", sch_quotation },
    { { 2, 0 }, { 0, 0 }, "Quoteaction", sch_quoteaction },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "RadiationTherapy", sch_radiationtherapy },
    { { 3, 5 }, { 0, 0 }, "Radiobroadcastservice", sch_radiobroadcastservice },
    { { 2, 0 }, { 0, 0 }, "Radiochannel", sch_radiochannel },
    { { 2, 0 }, { 0, 0 }, "Radioclip", sch_radioclip },
    { { 2, 0 }, { 0, 0 }, "Radioepisode", sch_radioepisode },
    { { 3, 6 }, { 6, 0 }, "Radiography", sch_radiography },
    { { 2, 0 }, { 0, 0 }, "Radioseason", sch_radioseason },
    { { 2, 0 }, { 0, 0 }, "Radioseries", sch_radioseries },
    { { 2, 0 }, { 0, 0 }, "Radiostation", sch_radiostation },
    { { 3, 5 }, { 0, 0 }, "Randomizedtrial", sch_randomisedtrial },
    { { 2, 0 }, { 0, 0 }, "Rating", sch_rating },
    { { 2, 0 }, { 0, 0 }, "Reactaction", sch_reactaction },
    { { 2, 0 }, { 0, 0 }, "Readaction", sch_readaction },
    { { 3, 5 }, { 0, 0 }, "Readpermission", sch_readpermission },
    { { 2, 0 }, { 0, 0 }, "Realestateagent", sch_realestateagent },
    { { 4, 0 }, { 0, 0 }, "Realestatelisting", sch_realestatelisting },
    { { 3, 6 }, { 6, 0 }, "Rearwheeldriveconfiguration", sch_rearwheeldriveconfiguration },
    { { 2, 0 }, { 0, 0 }, "Receiveaction", sch_receiveaction },
    { { 2, 0 }, { 0, 0 }, "Recipe", sch_recipe },
    { { 6, 0 }, { 0, 0 }, "Recommendation", sch_recommendation },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Recommendeddoseschedule", sch_recommendeddoseschedule },
    { { 3, 6 }, { 6, 0 }, "Recruiting", sch_recruiting },
    { { 2, 0 }, { 0, 0 }, "Recyclingcenter", sch_recyclingcentre },
    { { 3, 8 }, { 0, 0 }, "Refundtypeenumeration", sch_refundtypeenumeration, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "Refurbishedcondition", sch_refurbishedcondition },
    { { 2, 0 }, { 0, 0 }, "Registeraction", sch_registeraction },
    { { 3, 6 }, { 6, 0 }, "Registry", sch_registry },
    { { 3, 6 }, { 6, 0 }, "Reimbursementcap", sch_reimbursementcap },
    { { 2, 0 }, { 0, 0 }, "Rejectaction", sch_rejectaction },
    { { 5, 0 }, { 0, 0 }, "Relatedtopicshealthaspect", sch_relatedtopicshealthaspect },
    { { 3, 5 }, { 0, 0 }, "Remixalbum", sch_remixalbum },
    { { 3, 6 }, { 6, 0 }, "Renal", sch_renal },
    { { 2, 0 }, { 0, 0 }, "Rentaction", sch_rentaction },
    { { 2, 0 }, { 0, 0 }, "Rentalcarreservation", sch_rentalcarreservation },
    { { 3, 6 }, { 6, 0 }, "Rentalvehicleusage", sch_rentalvehicleusage },
    { { 3, 5 }, { 0, 0 }, "Repaymentspecification", sch_repaymentspecification },
    { { 2, 0 }, { 0, 0 }, "Replaceaction", sch_replaceaction },
    { { 2, 0 }, { 0, 0 }, "Replyaction", sch_replyaction },
    { { 2, 1 }, { 0, 0 }, "Report", sch_report },
    { { 3, 5 }, { 0, 0 }, "Reportagenewsarticle", sch_reportagenewsarticle },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Reporteddoseschedule", sch_reporteddoseschedule },
    { { 3, 5 }, { 0, 0 }, "Researchproject", sch_researchproject },
    { { 2, 0 }, { 0, 0 }, "Reservation", sch_reservation },
    { { 3, 5 }, { 0, 0 }, "Reservationcancelled", sch_reservationcancelled },
    { { 3, 5 }, { 0, 0 }, "Reservationconfirmed", sch_reservationconfirmed },
    { { 3, 5 }, { 0, 0 }, "Reservationhold", sch_reservationhold },
    { { 2, 0 }, { 0, 0 }, "Reservationpackage", sch_reservationpackage },
    { { 3, 5 }, { 0, 0 }, "Reservationpending", sch_reservationpending },
    { { 2, 0 }, { 0, 0 }, "Reservationstatustype", sch_reservationstatustype, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "Reserveaction", sch_reserveaction },
    { { 2, 0 }, { 0, 0 }, "Reservoir", sch_reservoir },
    { { 2, 0 }, { 0, 0 }, "Residence", sch_residence },
    { { 3, 1 }, { 0, 0 }, "Resort", sch_resort },
    { { 3, 5 }, { 0, 0 }, "Respiratorytherapy", sch_respiratorytherapy },
    { { 2, 0 }, { 0, 0 }, "Restaurant", sch_restaurant },
    { { 3, 8 }, { 0, 0 }, "Restockingfees", sch_restockingfees },
    { { 3, 0 }, { 0, 0 }, "RestrictedDiet", sch_restricteddiet, s_schema, SF_ENUMERATION },
    { { 3, 6 }, { 6, 0 }, "Resultsavailable", sch_resultsavailable },
    { { 3, 6 }, { 6, 0 }, "Resultsnotavailable", sch_resultsnotavailable },
    { { 2, 0 }, { 0, 0 }, "Resumeaction", sch_resumeaction },
    { { 3, 6 }, { 6, 0 }, "Retail", sch_retail },
    { { 2, 0 }, { 0, 0 }, "ReturnAction", sch_returnaction },
    { { 13, 0 }, { 0, 0 }, "ReturnAtKiosk", sch_returnatkiosk },
    { { 13, 0 }, { 0, 0 }, "ReturnByMail", sch_returnbymail },
    { { 3, 8 }, { 0, 0 }, "ReturnFeesEnumeration", sch_returnfeesenumeration, s_schema, SF_ENUMERATION },
    { { 13, 0 }, { 0, 0 }, "ReturnFeesCustomerResponsibility", sch_returnfeescustomerresponsibility },
    { { 13, 0 }, { 0, 0 }, "ReturnInStore", sch_returninstore },
    { { 13, 0 }, { 0, 0 }, "ReturnLabelCustomerResponsibility", sch_returnlabelcustomerresponsibility },
    { { 13, 0 }, { 0, 0 }, "ReturnLabelDownloadAndPrint", sch_returnlabeldownloadandprint },
    { { 13, 0 }, { 0, 0 }, "ReturnLabelInBox", sch_returnlabeldownloadandprint },
    { { 13, 0 }, { 0, 0 }, "ReturnLabelSourceEnumeration", sch_returnlabelsourceenumeration, s_schema, SF_ENUMERATION },
    { { 13, 0 }, { 0, 0 }, "ReturnMethodEnumeration", sch_returnmethodenumeration, s_schema, SF_ENUMERATION },
    { { 3, 8 }, { 0, 0 }, "ReturnShippingFees", sch_returnshippingfees },
    { { 2, 0 }, { 0, 0 }, "Review", sch_review },
    { { 2, 0 }, { 0, 0 }, "Reviewaction", sch_reviewaction },
    { { 3, 5 }, { 0, 0 }, "Reviewnewsarticle", sch_reviewnewsarticle },
    { { 3, 6 }, { 6, 0 }, "Rheumatologic", sch_rheumatologic },
    { { 3, 6 }, { 6, 0 }, "Righthanddriving", sch_righthanddriving },
    { { 5, 0 }, { 0, 0 }, "Risksorcomplicationshealthaspect", sch_risksorcomplicationshealthaspect },
    { { 2, 0 }, { 0, 0 }, "Riverbodyofwater", sch_riverbodyofwater },
    { { 2, 0 }, { 0, 0 }, "Role", sch_role },
    { { 2, 0 }, { 0, 0 }, "Roofingcontractor", sch_roofingcontractor },
    { { 3, 1 }, { 0, 0 }, "Room", sch_room },
    { { 2, 0 }, { 0, 0 }, "Rsvpaction", sch_rsvpaction },
    { { 3, 5 }, { 0, 0 }, "Rsvpresponsemaybe", sch_rsvpresponsemaybe },
    { { 3, 5 }, { 0, 0 }, "Rsvpresponseno", sch_rsvpresponseno },
    { { 2, 0 }, { 0, 0 }, "Rsvpresponsetype", sch_rsvpresponsetype, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "Rsvpresponseyes", sch_rsvpresponseyes },
    { { 2, 0 }, { 0, 0 }, "Rvpark", sch_rvpark },
    { { 12, 0 }, { 0, 0 }, "SafetyHealthAspect", sch_safetyhealthaspect },
    { { 2, 0 }, { 0, 0 }, "SaleEvent", sch_saleevent },
    { { 11, 0 }, { 0, 0 }, "SalePrice", sch_saleprice },
    { { 12, 0 }, { 0, 0 }, "SatireOrParodyContent", sch_satireorparodycontent },
    { { 3, 5 }, { 0, 0 }, "SatiricalArticle", sch_satiricalarticle },
    { { 3, 5 }, { 0, 0 }, "Saturday", sch_saturday },
    { { 3, 5 }, { 0, 0 }, "Schedule", sch_schedule },
    { { 2, 0 }, { 0, 0 }, "Scheduleaction", sch_scheduleaction },
    { { 2, 0 }, { 0, 0 }, "Scholarlyarticle", sch_scholarlyarticle },
    { { 2, 0 }, { 0, 0 }, "School", sch_school },
    { { 7, 1 }, { 0, 0 }, "Schooldistrict", sch_schooldistrict },
    { { 2, 0 }, { 0, 0 }, "Screeningevent", sch_screeningevent },
    { { 5, 0 }, { 0, 0 }, "Screeninghealthaspect", sch_screeninghealthaspect },
    { { 2, 0 }, { 0, 0 }, "Sculpture", sch_sculpture },
    { { 2, 0 }, { 0, 0 }, "Seabodyofwater", sch_seabodyofwater },
    { { 2, 0 }, { 0, 0 }, "Searchaction", sch_searchaction },
    { { 2, 0 }, { 0, 0 }, "Searchresultspage", sch_searchresultspage },
    { { 2, 0 }, { 3, 4 }, "Season", sch_season },
    { { 2, 0 }, { 0, 0 }, "Seat", sch_seat },
    { { 3, 5 }, { 0, 0 }, "Seatingmap", sch_seatingmap },
    { { 5, 0 }, { 0, 0 }, "Seedoctorhealthaspect", sch_seedoctorhealthaspect },
    { { 11, 0 }, { 0, 0 }, "SeekToAction", sch_seektoaction },
    { { 5, 0 }, { 0, 0 }, "Selfcarehealthaspect", sch_selfcarehealthaspect },
    { { 2, 0 }, { 0, 0 }, "Selfstorage", sch_selfstorage },
    { { 2, 0 }, { 0, 0 }, "Sellaction", sch_sellaction },
    { { 2, 0 }, { 0, 0 }, "Sendaction", sch_sendaction },
    { { 2, 0 }, { 0, 0 }, "Series", sch_series },
    { { 2, 0 }, { 0, 0 }, "Service", sch_service },
    { { 2, 0 }, { 0, 0 }, "ServiceChannel", sch_servicechannel },
    { { 2, 0 }, { 0, 0 }, "ShareAction", sch_shareaction },
    { { 3, 5 }, { 0, 0 }, "SheetMusic", sch_sheetmusic },
    { { 8, 0 }, { 0, 0 }, "ShippingDeliverytime", sch_shippingdeliverytime },
    { { 8, 0 }, { 0, 0 }, "ShippingRatesettings", sch_shippingratesettings },
    { { 2, 0 }, { 0, 0 }, "Shoestore", sch_shoestore },
    { { 2, 0 }, { 0, 0 }, "ShoppingCenter", sch_shoppingcentre },
    { { 3, 5 }, { 0, 0 }, "ShortStory", sch_shortstory },
    { { 5, 0 }, { 0, 0 }, "Sideeffectshealthaspect", sch_sideeffectshealthaspect },
    { { 3, 5 }, { 0, 0 }, "Singleblindedtrial", sch_singleblindedtrial },
    { { 3, 5 }, { 0, 0 }, "Singlecentertrial", sch_singlecentretrial },
    { { 2, 0 }, { 0, 0 }, "Singlefamilyresidence", sch_singlefamilyresidence },
    { { 3, 5 }, { 0, 0 }, "Singleplayer", sch_singleplayer },
    { { 3, 5 }, { 0, 0 }, "Singlerelease", sch_singlerelease },
    { { 2, 0 }, { 0, 0 }, "Sitenavigationelement", sch_sitenavigationelement },
    { { 12, 0 }, { 0, 0 }, "SizeGroupEnumeration", sch_sizegroupenumeration, s_schema, SF_ENUMERATION },
    { { 12, 0 }, { 0, 0 }, "SizeSpecification", sch_sizespecification },
    { { 12, 0 }, { 0, 0 }, "SizeSystemEnumeration", sch_sizesystemenumeration, s_schema, SF_ENUMERATION },
    { { 12, 0 }, { 0, 0 }, "SizeSystemImperial", sch_sizesystemimperial },
    { { 12, 0 }, { 0, 0 }, "SizeSystemMetric", sch_sizesystemmetric },
    { { 3, 6 }, { 6, 0 }, "Skin", sch_skin },
    { { 2, 0 }, { 0, 0 }, "SkiResort", sch_skiresort },
    { { 2, 0 }, { 0, 0 }, "SocialEvent", sch_socialevent },
    { { 2, 1 }, { 0, 0 }, "SocialMediaPosting", sch_socialmediaposting },
    { { 2, 0 }, { 0, 0 }, "SoftwareApplication", sch_softwareapplication },
    { { 2, 0 }, { 0, 0 }, "SoftwareSourceCode", sch_softwaresourcecode },
    { { 3, 5 }, { 0, 0 }, "Soldout", sch_soldout },
    { { 11, 0 }, { 0, 0 }, "SolveMathAction", sch_solvemathaction },
    { { 2, 0 }, { 0, 0 }, "Someproducts", sch_someproducts },
    { { 3, 5 }, { 0, 0 }, "Soundtrackalbum", sch_soundtrackalbum },
    { { 3, 5 }, { 0, 0 }, "Speakablespecification", sch_speakablespecification },
    { { 7, 0 }, { 0, 0 }, "Specialannouncement", sch_specialannouncement },
    { { 2, 0 }, { 0, 0 }, "Specialty", sch_speciality, s_schema, SF_ENUMERATION },
    { { 3, 6 }, { 6, 0 }, "Speechpathology", sch_speechpathology },
    { { 3, 5 }, { 0, 0 }, "Spokenwordalbum", sch_spokenwordalbum },
    { { 2, 0 }, { 0, 0 }, "Sportinggoodsstore", sch_sportinggoodsstore },
    { { 2, 0 }, { 0, 0 }, "Sportsactivitylocation", sch_sportsactivitylocation },
    { { 2, 0 }, { 0, 0 }, "Sportsclub", sch_sportsclub },
    { { 2, 0 }, { 0, 0 }, "Sportsevent", sch_sportsevent },
    { { 2, 0 }, { 0, 0 }, "SportsOrganization", sch_sportsorganisation },
    { { 2, 0 }, { 0, 0 }, "Sportsteam", sch_sportsteam },
    { { 3, 0 }, { 0, 0 }, "Spreadsheetdigitaldocument", sch_spreadsheetdigitaldocument },
    { { 11, 0 }, { 0, 0 }, "SRP", sch_srp },
    { { 2, 0 }, { 0, 0 }, "StadiumOrArena", sch_stadiumorarena },
    { { 12, 0 }, { 0, 0 }, "StagedContent", sch_stagedcontent },
    { { 5, 0 }, { 0, 0 }, "StagesHealthAspect", sch_stageshealthaspect },
    { { 2, 0 }, { 0, 0 }, "State", sch_state },
    { { 13, 0 }, { 0, 0 }, "Statement", sch_statement },
    { { 3, 9 }, { 0, 0 }, "StatisticalPopulation", sch_statisticalpopulation },
    { { 9, 0 }, { 0, 0 }, "StatusEnumeration", sch_statusenumeration },
    { { 2, 1 }, { 0, 0 }, "SteeringPositionValue", sch_steeringpositionvalue, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "Store", sch_store },
    { { 3, 8 }, { 0, 0 }, "Storecreditrefund", sch_storecreditrefund },
    { { 3, 5 }, { 0, 0 }, "Strengthtraining", sch_strengthtraining },
    { { 2, 0 }, { 0, 0 }, "Structuredvalue", sch_structuredvalue },
    { { 3, 5 }, { 0, 0 }, "Studioalbum", sch_studioalbum },
    { { 3, 5 }, { 0, 0 }, "Stupidtype", sch_stupidtype },
    { { 2, 0 }, { 0, 0 }, "Subscribeaction", sch_subscribeaction },
    { { 11, 0 }, { 0, 0 }, "Subscription", sch_subscription },
    { { 3, 5 }, { 0, 0 }, "Substance", sch_substance },
    { { 2, 0 }, { 0, 0 }, "Subwaystation", sch_subwaystation },
    { { 3, 1 }, { 0, 0 }, "Suite", sch_suite },
    { { 3, 5 }, { 0, 0 }, "Sunday", sch_sunday },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "SuperficialAnatomy", sch_superficialanatomy },
    { { 3, 6 }, { 6, 0 }, "Surgical", sch_surgical },
    { { 3, 5 }, { 0, 0 }, "Surgicalprocedure", sch_surgicalprocedure },
    { { 2, 0 }, { 0, 0 }, "Suspendaction", sch_suspendaction },
    { { 3, 6 }, { 6, 0 }, "Suspended", sch_suspended },
    { { 5, 0 }, { 0, 0 }, "Symptomshealthaspect", sch_symptomshealthaspect },
    { { 2, 0 }, { 0, 0 }, "Synagogue", sch_synagogue },
    { { 2, 0 }, { 0, 0 }, "Table", sch_table },
    { { 2, 0 }, { 0, 0 }, "TakeAction", sch_takeaction },
    { { 2, 0 }, { 0, 0 }, "TattooParlor", sch_tattooparlour },
    { { 2, 0 }, { 3, 4 }, "Taxi", sch_taxi },
    { { 2, 0 }, { 0, 0 }, "Taxireservation", sch_taxireservation },
    { { 2, 0 }, { 0, 0 }, "Taxiservice", sch_taxiservice },
    { { 2, 0 }, { 0, 0 }, "Taxistand", sch_taxistand },
    { { 3, 6 }, { 6, 0 }, "Taxivehicleusage", sch_taxivehicleusage },
    { { 13, 0 }, { 0, 0 }, "Taxon", sch_taxon },
    { { 2, 0 }, { 0, 0 }, "Techarticle", sch_techarticle },
    { { 2, 0 }, { 0, 0 }, "Televisionchannel", sch_televisionchannel },
    { { 2, 0 }, { 0, 0 }, "Televisionstation", sch_televisionstation },
    { { 2, 0 }, { 0, 0 }, "Tenniscomplex", sch_tenniscomplex },
    { { 3, 6 }, { 6, 0 }, "Terminated", sch_terminated },
    { { 2, 0 }, { 0, 0 }, "Text", sch_text, s_schema, MAKE_SIMPLE_TYPE (t_text) },
    { { 3, 0 }, { 0, 0 }, "Textdigitaldocument", sch_textdigitaldocument },
    { { 2, 0 }, { 0, 0 }, "Theaterevent", sch_theatreevent },
    { { 2, 0 }, { 0, 0 }, "Theatergroup", sch_theatregroup },
    { { 3, 6 }, { 6, 0 }, "Therapeutic", sch_therapeutic },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Therapeuticprocedure", sch_therapeuticprocedure },
    { { 3, 5 }, { 0, 0 }, "Thesis", sch_thesis },
    { { 2, 0 }, { 0, 0 }, "Thing", sch_thing },
    { { 3, 6 }, { 6, 0 }, "Throat", sch_throat },
    { { 3, 5 }, { 0, 0 }, "Thursday", sch_thursday },
    { { 2, 0 }, { 0, 0 }, "Ticket", sch_ticket },
    { { 2, 0 }, { 0, 0 }, "Tieaction", sch_tieaction },
    { { 2, 0 }, { 0, 0 }, "Time", sch_time, s_schema, MAKE_SIMPLE_TYPE (t_just_time) },
    { { 2, 0 }, { 0, 0 }, "Tipaction", sch_tipaction },
    { { 2, 0 }, { 0, 0 }, "Tireshop", sch_tyreshop },
    { { 3, 5 }, { 0, 0 }, "Tollfree", sch_tollfree },
    { { 2, 0 }, { 0, 0 }, "Touristattraction", sch_touristattraction },
    { { 3, 5 }, { 0, 0 }, "Touristdestination", sch_touristdestination },
    { { 2, 0 }, { 0, 0 }, "Touristinformationcenter", sch_touristinformationcentre },
    { { 3, 5 }, { 0, 0 }, "Touristtrip", sch_touristtrip },
    { { 3, 6 }, { 6, 0 }, "Toxicologic", sch_toxicologic },
    { { 2, 0 }, { 0, 0 }, "ToyStore", sch_toystore },
    { { 2, 0 }, { 0, 0 }, "TrackAction", sch_trackaction },
    { { 2, 0 }, { 0, 0 }, "TradeAction", sch_tradeaction },
    { { 3, 6 }, { 6, 0 }, "TraditionalChinese", sch_environmentharm },
    { { 2, 0 }, { 0, 0 }, "TrainReservation", sch_trainreservation },
    { { 2, 0 }, { 0, 0 }, "TrainStation", sch_trainstation },
    { { 2, 0 }, { 0, 0 }, "TrainTrip", sch_traintrip },
    { { 2, 0 }, { 0, 0 }, "TransferAction", sch_transferaction },
    { { 12, 0 }, { 0, 0 }, "TransformedContent", sch_transformedcontent },
    { { 3, 5 }, { 0, 0 }, "TransitMap", sch_transitmap },
    { { 2, 0 }, { 0, 0 }, "TravelAction", sch_travelaction },
    { { 2, 0 }, { 0, 0 }, "TravelAgency", sch_travelagency },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "TreatmentIndication", sch_treatmentindication },
    { { 5, 0 }, { 0, 0 }, "TreatmentsHealthAspect", sch_treatmentshealthaspect },
    { { 3, 4 }, { 0, 0 }, "Trip", sch_trip },
    { { 3, 5 }, { 0, 0 }, "TripleBlindedTrial", sch_tripleblindedtrial },
    { { 2, 0 }, { 0, 0 }, "True", sch_true },
    { { 3, 5 }, { 0, 0 }, "Tuesday", sch_tuesday },
    { { 2, 0 }, { 0, 0 }, "TvClip", sch_tvclip },
    { { 2, 0 }, { 0, 0 }, "TvEpisode", sch_tvepisode },
    { { 2, 0 }, { 0, 0 }, "TvSeason", sch_tvseason },
    { { 2, 0 }, { 0, 0 }, "TvSeries", sch_tvseries },
    { { 2, 0 }, { 0, 0 }, "TypeAndQuantityNode", sch_typeandquantitynode },
    { { 5, 0 }, { 0, 0 }, "TypesHealthAspect", sch_typeshealthaspect },
    { { 8, 0 }, { 0, 0 }, "UkNonProfitType", sch_uknonprofittype, s_schema, SF_ENUMERATION },
    { { 9, 0 }, { 0, 0 }, "UkTrust", sch_uktrust },
    { { 3, 6 }, { 6, 0 }, "Ultrasound", sch_ultrasound },
    { { 7, 4 }, { 0, 0 }, "UnemploymentSupport", sch_unemploymentsupport },
    { { 9, 0 }, { 0, 0 }, "UnincorporatedAssociationCharity", sch_unincorporatedassociationcharity },
    { { 2, 0 }, { 0, 0 }, "UnitPriceSpecification", sch_unitpricespecification },
    { { 3, 5 }, { 0, 0 }, "UnofficialLegalValue", sch_unofficiallegalvalue },
    { { 2, 0 }, { 0, 0 }, "UnregisterAction", sch_unregisteraction },
    { { 2, 0 }, { 0, 0 }, "UpdateAction", sch_updateaction },
    { { 2, 0 }, { 0, 0 }, "Url", sch_url, s_schema, MAKE_SIMPLE_TYPE (t_url) },
    { { 3, 6 }, { 6, 0 }, "Urologic", sch_urologic },
    { { 5, 0 }, { 0, 0 }, "UsageOrScheduleHealthAspect", sch_usageorschedulehealthaspect },
    { { 2, 0, SV_DEP_3034 }, { 3, 4 }, "UseAction", sch_useaction },
    { { 3, 5 }, { 0, 0 }, "UsedCondition", sch_usedcondition },
    { { 2, 0, SV_DEP_3034 }, { 3, 4 }, "UserBlocks", sch_userblocks },
    { { 2, 0, SV_DEP_3034 }, { 3, 4 }, "UserCheckins", sch_usercheckins },
    { { 2, 0, SV_DEP_3034 }, { 3, 4 }, "UserComments", sch_usercomments },
    { { 2, 0, SV_DEP_3034 }, { 3, 4 }, "UserDownloads", sch_userdownloads },
    { { 2, 0, SV_DEP_3034 }, { 3, 4 }, "UserInteraction", sch_userinteraction },
    { { 2, 0, SV_DEP_3034 }, { 3, 4 }, "UserLikes", sch_userlikes },
    { { 2, 0, SV_DEP_3034 }, { 3, 4 }, "UserPageVisits", sch_userpagevisits },
    { { 2, 0, SV_DEP_3034 }, { 3, 4 }, "UserPlays", sch_userplays },
    { { 2, 0, SV_DEP_3034 }, { 3, 4 }, "UserPlusOnes", sch_userplusones },
    { { 3, 5 }, { 0, 0 }, "UserReview", sch_userreview },
    { { 2, 0, SV_DEP_3034 }, { 3, 4 }, "UserTweets", sch_usertweets },
    { { 8, 0 }, { 0, 0 }, "USNonprofitType", sch_usnonprofittype, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "VeganDiet", sch_vegandiet },
    { { 3, 5 }, { 0, 0 }, "VegetarianDiet", sch_vegetariandiet },
    { { 2, 0 }, { 0, 0 }, "Vehicle", sch_vehicle },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Vein", sch_vein },
    { { 3, 5 }, { 0, 0 }, "VenueMap", sch_venuemap },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "Vessel", sch_vessel },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, "VeterinaryCare", sch_veterinarycare },
    { { 2, 0 }, { 0, 0 }, "VideoGallery", sch_videogallery },
    { { 2, 0 }, { 0, 0 }, "VideoGame", sch_videogame },
    { { 2, 0 }, { 0, 0 }, "VideoGameClip", sch_videogameclip },
    { { 2, 0 }, { 0, 0 }, "VideoGameSeries", sch_videogameseries },
    { { 2, 0 }, { 0, 0 }, "VideoObject", sch_videoobject },
    { { 13, 0 }, { 0, 0 }, "VideoObjectSnapshot", sch_videoobjectsnapshot },
    { { 2, 0 }, { 0, 0 }, "ViewAction", sch_viewaction },
    { { 3, 5 }, { 0, 0 }, "VinylFormat", sch_vinylformat },
    { { 7, 0 }, { 0, 0 }, "VirtualLocation", sch_virtuallocation },
    { { 3, 6 }, { 6, 0 }, "Virus", sch_virus },
    { { 2, 0 }, { 0, 0 }, "VisualArtsEvent", sch_visualartsevent },
    { { 2, 0 }, { 0, 0 }, "VisualArtwork", sch_visualartwork },
    { { 3, 5 }, { 0, 0 }, "VitalSign", sch_vitalsign },
    { { 2, 0 }, { 0, 0 }, "Volcano", sch_volcano },
    { { 2, 0 }, { 0, 0 }, "VoteAction", sch_voteaction },
    { { 2, 0 }, { 0, 0 }, "WantAction", sch_wantaction },
    { { 2, 0 }, { 0, 0 }, "WarrantyPromise", sch_warrantypromise },
    { { 2, 0 }, { 0, 0 }, "WarrantyScope", sch_warrantyscope, s_schema, SF_EXTERNAL_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "WatchAction", sch_watchaction },
    { { 2, 0 }, { 0, 0 }, "Waterfall", sch_waterfall },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementTypeEnumeration", sch_wearablemeasurementtypeenumeration, s_schema, SF_ENUMERATION },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementBack", sch_wearablemeasurementback },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementChestOrBust", sch_wearablemeasurementchestorbust },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementCollar", sch_wearablemeasurementcollar },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementCup", sch_wearablemeasurementcup },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementHeight", sch_wearablemeasurementheight },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementHips", sch_wearablemeasurementhips },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementInseam", sch_wearablemeasurementinseam },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementLength", sch_wearablemeasurementlength },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementOutsideLeg", sch_wearablemeasurementoutsideleg },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementSleeve", sch_wearablemeasurementsleeve },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementWaist", sch_wearablemeasurementwaist },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementWidth", sch_wearablemeasurementwidth },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupBig", sch_wearablesizegroupbig },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupBoys", sch_wearablesizegroupboys },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupEnumeration", sch_wearablesizegroupenumeration, s_schema, SF_ENUMERATION },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupExtraShort", sch_wearablesizegroupextrashort },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupExtraTall", sch_wearablesizegroupextratall },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupGirls", sch_wearablesizegroupgirls },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupHusky", sch_wearablesizegrouphusky },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupInfants", sch_wearablesizegroupinfants },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupJuniors", sch_wearablesizegroupjuniors },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupMaternity", sch_wearablesizegroupmaternity },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupMens", sch_wearablesizegroupmens },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupMisses", sch_wearablesizegroupmisses },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupPetite", sch_wearablesizegrouppetite },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupPlus", sch_wearablesizegroupplus },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupRegular", sch_wearablesizegroupregular },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupShort", sch_wearablesizegroupshort },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupTall", sch_wearablesizegrouptall },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupWomens", sch_wearablesizegroupwomens },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemAU", sch_wearablesizesystemau },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemBR", sch_wearablesizesystembr },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemCN", sch_wearablesizesystemcn },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemContinental", sch_wearablesizesystemcontinental },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemDE", sch_wearablesizesystemde },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemEN13402", sch_wearablesizesystemen13402 },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemEnumeration", sch_wearablesizesystemenumeration, s_schema, SF_ENUMERATION },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemEurope", sch_wearablesizesystemeurope },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemFR", sch_wearablesizesystemfr },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemGS1", sch_wearablesizesystemgs1 },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemIT", sch_wearablesizesystemit },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemJP", sch_wearablesizesystemjp },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemMX", sch_wearablesizesystemmx },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemUK", sch_wearablesizesystemuk },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemUS", sch_wearablesizesystemus },
    { { 2, 0 }, { 0, 0 }, "Wearaction", sch_wearaction },
    { { 3, 5 }, { 0, 0 }, "Webapi", sch_webapi },
    { { 2, 0 }, { 0, 0 }, "Webapplication", sch_webapplication },
    { { 5, 0 }, { 0, 0 }, "Webcontent", sch_webcontent },
    { { 2, 0 }, { 0, 0 }, "Webpage", sch_webpage },
    { { 2, 0 }, { 0, 0 }, "Webpageelement", sch_webpageelement },
    { { 2, 0 }, { 0, 0 }, "Website", sch_website },
    { { 3, 5 }, { 0, 0 }, "Wednesday", sch_wednesday },
    { { 3, 6 }, { 6, 0 }, "WesternConventional", sch_realitybased },
    { { 3, 6 }, { 6, 0 }, "Wholesale", sch_wholesale },
    { { 2, 0 }, { 0, 0 }, "Wholesalestore", sch_wholesalestore },
    { { 2, 0 }, { 0, 0 }, "Winaction", sch_winaction },
    { { 2, 0 }, { 0, 0 }, "Winery", sch_winery },
    { { 3, 6 }, { 6, 0 }, "Withdrawn", sch_withdrawn },
    { { 3, 9 }, { 0, 0 }, "Workbasedprogram", sch_workbasedprogramme },
    { { 3, 5 }, { 0, 0 }, "WorkersUnion", sch_workersunion },
    { { 2, 0 }, { 0, 0 }, "Wpadblock", sch_wpadblock },
    { { 2, 0 }, { 0, 0 }, "Wpfooter", sch_wpfooter },
    { { 2, 0 }, { 0, 0 }, "Wpheader", sch_wpheader },
    { { 2, 0 }, { 0, 0 }, "Wpsidebar", sch_wpsidebar },
    { { 2, 0 }, { 0, 0 }, "WriteAction", sch_writeaction },
    { { 3, 5 }, { 0, 0 }, "WritePermission", sch_writepermission },
    { { 3, 5 }, { 0, 0 }, "Xpathtype", sch_xpathtype },
    { { 3, 6 }, { 6, 0 }, "Xray", sch_xray },
    { { 3, 5 }, { 0, 0 }, "Zoneboardingpolicy", sch_zoneboardingpolicy },

    // living standard
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/hcalendar#vevent", mf_vevent, s_microformats, SF_NO_ITEMID },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/hcard", mft_hcard, s_microformats, SF_NO_ITEMID },
    { { s_whatwg, 1, 0 }, { 0, 0 }, "work", wwg_work, s_whatwg },

    // XML schema types
    { { s_xsd, 1, 1 }, { 0, 0 }, "anyAtomicType", xsd_anyatomictype, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "anySimpleType", xsd_anysimpletype, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "anyType", xsd_anytype, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "anyURI", xsd_anyuri, s_xsd, MAKE_SIMPLE_TYPE (t_url) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "base64Binary", xsd_base64binary, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "boolean", xsd_boolean, s_xsd, MAKE_SIMPLE_TYPE (t_xsdbool) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "byte", xsd_byte, s_xsd, MAKE_SIMPLE_TYPE (t_integer) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "date", xsd_date, s_xsd, MAKE_SIMPLE_TYPE (t_just_date) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "dateTime", xsd_datetime, s_xsd, MAKE_SIMPLE_TYPE (t_datetime) },
    { { s_xsd, 1, 1 }, { 0, 0 }, "dateTimeDuration", xsd_datetimeduration, s_xsd, MAKE_SIMPLE_TYPE (t_duration) },
    { { s_xsd, 1, 1 }, { 0, 0 }, "dateTimeStamp", xsd_datetimestamp, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "decimal", xsd_decimal, s_xsd, MAKE_SIMPLE_TYPE (t_real) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "double", xsd_double, s_xsd, MAKE_SIMPLE_TYPE (t_real) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "duration", xsd_duration, s_xsd, MAKE_SIMPLE_TYPE (t_duration) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "ENTITIES", xsd_entities, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "ENTITY", xsd_entity, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "float", xsd_float, s_xsd, MAKE_SIMPLE_TYPE (t_real) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "gDay", xsd_gday, s_xsd, MAKE_SIMPLE_TYPE (t_day) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "gMonth", xsd_gmonth, s_xsd, MAKE_SIMPLE_TYPE (t_month) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "gMonthDay", xsd_gmonthday, s_xsd, MAKE_SIMPLE_TYPE (t_monthday) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "gYear", xsd_gyear, s_xsd, MAKE_SIMPLE_TYPE (t_year) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "gYearMonth", xsd_gyearmonth, s_xsd, MAKE_SIMPLE_TYPE (t_yearmonth) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "hexBinary", xsd_hexbinary, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "ID", xsd_id, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "IDREF", xsd_idref, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "IDREFS", xsd_idrefs, s_xsd, MAKE_SIMPLE_TYPE (t_idrefs) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "int", xsd_int, s_xsd, MAKE_SIMPLE_TYPE (t_integer) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "integer", xsd_integer, s_xsd, MAKE_SIMPLE_TYPE (t_integer) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "language", xsd_language, s_xsd, MAKE_SIMPLE_TYPE (t_lang) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "long", xsd_long, s_xsd, MAKE_SIMPLE_TYPE (t_integer) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "Name", xsd_name, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "NCName", xsd_ncname, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "negativeInteger", xsd_negativeinteger, s_xsd, MAKE_SIMPLE_TYPE (t_negative) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "NMTOKEN", xsd_nmtoken, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "NMTOKENS", xsd_nmtokens, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "nonNegativeInteger", xsd_nonnegativeinteger, s_xsd, MAKE_SIMPLE_TYPE (t_not_neg) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "nonPositiveInteger", xsd_nonpositiveinteger, s_xsd, MAKE_SIMPLE_TYPE (t_not_pos) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "normalizedString", xsd_normalisedstring, s_xsd, MAKE_SIMPLE_TYPE (t_text) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "NOTATION", xsd_notation, s_xsd, MAKE_SIMPLE_TYPE (t_namespace) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "positiveInteger", xsd_positiveinteger, s_xsd, MAKE_SIMPLE_TYPE (t_positive) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "QName", xsd_qname, s_xsd, MAKE_SIMPLE_TYPE (t_namespace) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "short", xsd_short, s_xsd, MAKE_SIMPLE_TYPE (t_short) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "string", xsd_string, s_xsd, MAKE_SIMPLE_TYPE (t_text) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "time", xsd_time, s_xsd, MAKE_SIMPLE_TYPE (t_just_time) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "token", xsd_token, s_xsd, MAKE_SIMPLE_TYPE (t_text) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "unsignedByte", xsd_unsignedbyte, s_xsd, MAKE_SIMPLE_TYPE (t_unsigned_byte) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "unsignedInt", xsd_unsignedint, s_xsd, MAKE_SIMPLE_TYPE (t_unsigned) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "unsignedLong", xsd_unsignedlong, s_xsd, MAKE_SIMPLE_TYPE (t_unsigned) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "unsignedShort", xsd_unsignedshort, s_xsd, MAKE_SIMPLE_TYPE (t_unsigned_short) },
    { { s_xsd, 1, 1 }, { 0, 0 }, "yearMonthDuration", xsd_yearmonthduration, s_xsd },

    { { 0, 0 }, { 0, 0 }, "", sty_illegal } };

sch::sch (nitpick& nits, const html_version& v, const ::std::string& x, const e_schema root)
{   e_schema_type es = parse (nits, v, x, root);
    if (es != sty_illegal) set (v, es); }

void sch::init (nitpick& nits)
{   symbol < schema_version, e_schema_type, e_schema, s_schema > ::
        init (nits, schema_type_symbol_table, sizeof (schema_type_symbol_table) / sizeof (symbol_entry < schema_version, e_schema_type, e_schema, s_schema >), true); }

e_schema_type sch::parse (nitpick& nits, const html_version& v, const ::std::string& x, const e_schema root)
{   ::std::string lc (trim_the_lot_off (x));
    if (lc.empty ())
    {   nits.pick (nit_empty, ed_jul20, "5.2.2 Items", es_error, ec_microdata, "An ITEMTYPE cannot be empty");
        return sty_illegal; }
    if (lc.at (0) == '/') lc = lc.substr (1);
    if (! v.xhtml ()) lc = ::boost::algorithm::to_lower_copy (lc);
    symbol < schema_version, e_schema_type, e_schema, s_schema > s (default_schema, lc, root);
    for (int i = s_none; (s.unknown () || (s.get () == sty_context)) && (i < s_error); ++i)
        if (i != root)
            s.reset (symbol < schema_version, e_schema_type, e_schema, s_schema > (default_schema, lc, static_cast < e_schema > (i)));
    if (s.unknown () || (s.get () == sty_context))
    {   if (! check_spelling (nits, v, lc))
            nits.pick (nit_unrecognised_schema, es_error, ec_microdata, "Unrecognised type ", quote (x)); }
    else
    {   if (s.first ().deprecated ())
            nits.pick (nit_deprecated_schema, es_warning, ec_schema, quote (x), " is deprecated");
        schema_version sv;
        switch (s.ns ())
        {   case s_article :
            case s_book :
            case s_cc :
            case s_ctag :
            case s_dcam :
            case s_dcmi :
            case s_dct :
            case s_gr :
            case s_ical :
            case s_icaltzd :
            case s_music :
            case s_none :
            case s_og :
            case s_poetry :
            case s_profile :
            case s_rdf :
            case s_rdfs :
            case s_video :
            case s_wdrs :
            case s_website :
                return s.get ();
            case s_dc :
                if (context.dc () == 0) sv = dc_1_0; else sv = dc_1_1;
                if (may_apply (sv, s.first (), s.last ())) return s.get ();
                nits.pick (nit_unrecognised_schema, es_error, ec_schema, quote (x), " is invalid in ", sv.report ());
                break;
            case s_foaf :
                {   int n = context.foaf ();
                    if ((n > 0) && (n < 99)) sv = schema_version (s_foaf, 0, static_cast < char > (n));
                    else sv = schema_version (s_foaf, 0, 99); }
                if (may_apply (sv, s.first (), s.last ())) return s.get ();
                nits.pick (nit_unrecognised_schema, es_error, ec_schema, quote (x), " is invalid in ", sv.report ());
                break;
            case s_microformats :
                if (may_apply (context.mf_ver (), s.first (), s.last ())) return s.get ();
                nits.pick (nit_unrecognised_schema, es_error, ec_schema, quote (x), " is invalid here");
                break;
            case s_rdfa :
                {   int n = context.rdfa ();
                    if ((n >= 0) && (n < 4)) sv = schema_version (s_rdfa, 1, static_cast < char > (n));
                    else sv = schema_version (s_rdfa, 1, 3); }
                if (may_apply (sv, s.first (), s.last ())) return s.get ();
                nits.pick (nit_unrecognised_schema, es_error, ec_schema, quote (x), " is invalid in ", sv.report ());
                break;
            case s_schema :
                sv = context.schema_ver ();
                if (may_apply (sv, s.first (), s.last ())) return s.get ();
                nits.pick (nit_unrecognised_schema, es_error, ec_schema, quote (x), " is invalid in ", sv.report ());
                break;
            case s_whatwg :
                if (v >= html_jul09) return s.get ();
                nits.pick (nit_unrecognised_schema, es_error, ec_schema, quote (x), " is invalid in ", v.report ()); // yes, html_version
                break;
            case s_xsd :
                if (context.xsd () == 0) sv = xsd_1_0; else sv = xsd_1_1;
                if (may_apply (sv, s.first (), s.last ())) return s.get ();
                nits.pick (nit_unrecognised_schema, es_error, ec_schema, quote (x), " is invalid in ", sv.report ());
                break;
            default :
                nits.pick (nit_unrecognised_schema, es_error, ec_schema, quote (x), " requires a schema");
                break; } }
    return sty_illegal; }

e_schema sch::root () const
{ return first ().root (); }

e_schema sch::root (const e_schema_type st)
{ return first_version (st).root (); }

bool sch::enumerated () const
{ return enumerated_schema_type (flags ()); }

bool sch::has_simple_type () const
{ return has_simple_schema_type (flags ()); }

bool sch::external_enumerated () const
{ return external_enumerated_schema_type (flags ()); }

e_type sch::get_simple_type () const
{ return get_simple_schema_type (flags ()); }

bool enumerated_schema_type (const flags_t flags)
{ return (flags & SF_ENUMERATION) == SF_ENUMERATION; }

bool has_simple_schema_type (const flags_t flags)
{ return (flags & SF_SIMPLE_MASK) != 0; }

bool external_enumerated_schema_type (const flags_t flags)
{ return (flags & SF_EXTERNAL_ENUMERATION) == SF_EXTERNAL_ENUMERATION; }

e_type get_simple_schema_type (const flags_t flags)
{ return static_cast < e_type > (flags & SF_SIMPLE_MASK); }

bool is_itemid_ok (const flags_t flags)
{ return (flags & SF_NO_ITEMID) == 0; }
