/*
ssc (static site checker)
Copyright (c) 2020,2021 Dylan Harris
https://dylanharris.org/

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public Licence as published by
the Free Software Foundation, either version 3 of the Licence, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public Licence for more details.

You should have received a copy of the GNU General Public
Licence along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "main/standard.h"
#include "main/context.h"
#include "webpage/crosslink.h"
#include "schema/schema_type.h"
#include "schema/schema_name.h"
#include "schema/schema_structure.h"
#include "schema/schema_property.h"
#include "schema/schema_hierarchy.h"
#include "type/type.h"

struct property_gen
{   schema_version from_;
    schema_version to_;
    e_schema_property prop_;
    e_type field_type_ = t_generic;
    e_schema_type field_schema_ = sty_context;
    uint32_t flags_ = 0; };

#define SP_VALUE_TYPENAME   0x00000001

property_gen gentab [] =
{   { { 0, 0 }, { 0, 0 }, sp_context, t_unknown },

    // creative commons
    { { s_cc, 1, 0 }, { 0, 0 }, cc_attributionname, t_text },
    { { s_cc, 1, 0 }, { 0, 0 }, cc_attributionurl, t_url },
    { { s_cc, 1, 0 }, { 0, 0 }, cc_deprecatedon, t_datetime },
    { { s_cc, 1, 0 }, { 0, 0 }, cc_jurisdiction, t_url },
    { { s_cc, 1, 0 }, { 0, 0 }, cc_legalcode, t_url },
    { { s_cc, 1, 0 }, { 0, 0 }, cc_licence, t_url },
    { { s_cc, 1, 0 }, { 0, 0 }, cc_morepermissions, t_url },
    { { s_cc, 1, 0 }, { 0, 0 }, cc_permits, t_cc_permits },
    { { s_cc, 1, 0 }, { 0, 0 }, cc_prohibits, t_cc_prohibits },
    { { s_cc, 1, 0 }, { 0, 0 }, cc_requires, t_cc_requires },

    // open tag
    { { s_article, 1, 0 }, { 0, 0 }, og_article_author, t_text },
    { { s_article, 1, 0 }, { 0, 0 }, og_article_expiration_date, t_datetime },
    { { s_article, 1, 0 }, { 0, 0 }, og_article_modified_time, t_datetime },
    { { s_article, 1, 0 }, { 0, 0 }, og_article_published_time, t_datetime },
    { { s_article, 1, 0 }, { 0, 0 }, og_article_section, t_text },
    { { s_article, 1, 0 }, { 0, 0 }, og_article_tag, t_text },

    // open tag
    { { s_book, 1, 0 }, { 0, 0 }, og_book_author, t_text },
    { { s_book, 1, 0 }, { 0, 0 }, og_book_isbn, t_text },
    { { s_book, 1, 0 }, { 0, 0 }, og_book_release_date, t_datetime },
    { { s_book, 1, 0 }, { 0, 0 }, og_book_tag, t_url },

    // common tag
    { { s_ctag, 1, 0 }, { 0, 0 }, ctag_label, t_text },
    { { s_ctag, 1, 0 }, { 0, 0 }, ctag_means, t_url },
    { { s_ctag, 1, 0 }, { 0, 0 }, ctag_tagged, t_url },
    { { s_ctag, 1, 0 }, { 0, 0 }, ctag_taggingdate, t_datetime_4 },

    // dublin core elements
    { { s_dc, 1, 0 }, { 0, 0 }, dc_contributor, t_text },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_coverage, t_datetime },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_coverage, t_text },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_creator, t_text },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_date, t_datetime },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_description, t_text },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_format, t_mime },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_format, t_text },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_identifier, t_text },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_language, t_lang },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_publisher, t_text },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_relation, t_url },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_rights, t_text },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_source, t_text },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_subject, t_text },
    { { s_dc, 1, 0 }, { 0, 0 }, dc_title, t_text },
    { { s_dc, 1, 0 }, { s_dc, 1, 0 }, dc_type, t_text },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_type, t_schema, dcmi_collection },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_type, t_schema, dcmi_dataset },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_type, t_schema, dcmi_event },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_type, t_schema, dcmi_image },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_type, t_schema, dcmi_interactiveresource },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_type, t_schema, dcmi_movingimage },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_type, t_schema, dcmi_physicalobject },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_type, t_schema, dcmi_service },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_type, t_schema, dcmi_software },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_type, t_schema, dcmi_sound },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_type, t_schema, dcmi_stillimage },
    { { s_dc, 1, 1 }, { 0, 0 }, dc_type, t_schema, dcmi_text },

    // dublin core vocabulary description
    { { s_dcam, 1, 0 }, { 0, 0 }, dcam_domainincludes, t_text },
    { { s_dcam, 1, 0 }, { 0, 0 }, dcam_memberof, t_schema, dcam_vocabularyencodingscheme },
    { { s_dcam, 1, 0 }, { 0, 0 }, dcam_rangeincludes, t_text },

    // dublin core terms
    { { s_dct, 1, 0 }, { 0, 0 }, dct_abstract, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_accessrights, t_schema, dct_rightsstatement },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_accessrights, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_accrualmethod, t_schema, dct_methodofaccrual },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_accrualmethod, t_accrual_method },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_accrualperiodicity, t_schema, dct_frequency },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_accrualperiodicity, t_accrual_periodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_accrualpolicy, t_schema, dct_policy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_accrualpolicy, t_accrual_policy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_alternative, t_schema, rdfs_literal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_alternative, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_audience, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_available, t_schema, rdfs_literal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_available, t_datetime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_available, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographiccitation, t_schema, rdfs_literal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographiccitation, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_conformsto, t_schema, dct_standard },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_conformsto, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_contributor, t_schema, dct_agent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_contributor, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_coverage, t_schema, dct_jurisdiction },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_coverage, t_schema, dct_location },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_coverage, t_schema, dct_periodoftime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_coverage, t_datetime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_coverage, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_created, t_schema, rdfs_literal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_created, t_datetime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_created, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_creator, t_schema, dct_agent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_creator, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_date, t_schema, rdf_xmlliteral },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_date, t_datetime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_date, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_dateaccepted, t_schema, rdfs_literal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_dateaccepted, t_datetime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_dateaccepted, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_datecopyrighted, t_schema, rdfs_literal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_datecopyrighted, t_datetime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_datecopyrighted, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_datesubmitted, t_schema, rdfs_literal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_datesubmitted, t_datetime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_datesubmitted, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_description, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_educationlevel, t_schema, dct_agentclass },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_educationlevel, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_extent, t_schema, dct_sizeorduration },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_extent, t_mb },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_extent, t_datetime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_extent, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_format, t_schema, dct_mediatype },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_format, t_duration },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_format, t_mb },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_format, t_mime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_format, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_hasformat, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_haspart, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_hasversion, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_identifier, t_schema, rdfs_literal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_identifier, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_identifier, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_instructionalmethod, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_instructionalmethod, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_isformatof, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_ispartof, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_isreferencedby, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_isreplacedby, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_isrequiredby, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_issued, t_datetime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_isversionof, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_language, t_lang },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licence, t_schema, dct_licencedocument },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licence, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licence, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediator, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediator, t_schema, dct_agent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_medium, t_schema, dct_physicalmedium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_medium, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_modified, t_schema, rdfs_literal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_modified, t_datetime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_modified, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenance, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_publisher, t_schema, dct_agent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_publisher, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_references, t_urls },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_references, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_relation, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_relation, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_replaces, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_replaces, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_requires, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_requires, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rights, t_schema, dct_rightsstatement },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rights, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rights, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsholder, t_schema, dct_agent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsholder, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsholder, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_source, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_source, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_spatial, t_schema, dct_location },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_subject, t_url },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_subject, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_tableofcontents, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_temporal, t_schema, dct_periodoftime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_title, t_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_type, t_schema, dcmi_collection },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_type, t_schema, dcmi_dataset },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_type, t_schema, dcmi_event },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_type, t_schema, dcmi_image },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_type, t_schema, dcmi_interactiveresource },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_type, t_schema, dcmi_movingimage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_type, t_schema, dcmi_physicalobject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_type, t_schema, dcmi_service },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_type, t_schema, dcmi_software },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_type, t_schema, dcmi_sound },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_type, t_schema, dcmi_stillimage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_type, t_schema, dcmi_text },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_valid, t_schema, rdfs_literal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_valid, t_datetime },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_valid, t_text },

    // foaf
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_account, t_schema, foaf_onlineaccount },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_account, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_accountname, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_accountservicehomepage, t_schema, foaf_document },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_accountservicehomepage, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_accountservicehomepage, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_age, t_unsigned },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_aimchatid, t_text },
//    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_basednear, t_schema, owl_spatialthing },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_basednear, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_birthday, t_datetime },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_birthday, t_text },
//    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_currentproject, t_schema, owl_thing },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_currentproject, t_text },
//    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_depiction, t_schema, owl_thing },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_depicts, t_schema, foaf_image },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_depiction, t_url },
//    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_depicts, t_schema, owl_thing },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_depicts, t_text },
    { { s_foaf, 0, 1 }, { s_foaf, 0, 97 }, foaf_dnachecksum, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_familyname, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_firstname, t_text },
//    { { s_foaf, 0, 98 }, { 0, 0 }, foaf_focus, t_schema, owl_thing },
    { { s_foaf, 0, 98 }, { 0, 0 }, foaf_focus, t_text },
    { { s_foaf, 0, 1 }, { s_foaf, 0, 97 }, foaf_fundedby, t_text },
    { { s_foaf, 0, 1 }, { s_foaf, 0, 95 }, foaf_geekcode, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_gender, t_gender },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_gender, t_text },
    { { s_foaf, 0, 1 }, { s_foaf, 0, 96 }, foaf_givenname, t_text },
    { { s_foaf, 0, 1 }, { s_foaf, 0, 97 }, foaf_holdsaccount, t_schema, foaf_onlineaccount },
    { { s_foaf, 0, 1 }, { s_foaf, 0, 97 }, foaf_holdsaccount, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_homepage, t_schema, foaf_document },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_homepage, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_homepage, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_icqchatid, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_img, t_schema, foaf_image },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_img, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_interest, t_schema, foaf_document },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_interest, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_interest, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_isprimarytopicof, t_schema, foaf_document },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_isprimarytopicof, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_isprimarytopicof, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_jabberid, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_knows, t_schema, foaf_person },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_knows, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_knows, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_knows, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_lastname, t_text },
//    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_logo, t_schema, owl_thing },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_logo, t_text },
//    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_made, t_schema, owl_thing },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_made, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_maker, t_schema, foaf_agent },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_maker, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_maker, t_text },
//    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_mbox, t_schema, owl_thing },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_mbox, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_mboxsha1sum, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_member, t_schema, foaf_agent },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_member, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_member, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_membershipclass, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_msnchatid, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_myersbriggs, t_myersbriggs },
    { { s_foaf, 0, 96 }, { 0, 0 }, foaf_openid, t_schema, foaf_document },
    { { s_foaf, 0, 96 }, { 0, 0 }, foaf_openid, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_openid, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_page, t_schema, foaf_document },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_page, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_page, t_text },
//    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_pastproject, t_schema, owl_thing },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_pastproject, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_phone, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_plan, t_text },
//    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_primarytopic, t_schema, owl_thing },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_primarytopic, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_publications, t_schema, foaf_document },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_publications, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_publications, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_schoolhomepage, t_schema, foaf_document },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_schoolhomepage, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_schoolhomepage, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_sha1, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_skypeid, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_status, t_text },
    { { s_foaf, 0, 1 }, { s_foaf, 0, 97 }, foaf_surname, t_text },
//    { { s_foaf, 0, 1 }, { s_foaf, 0, 97 }, foaf_theme, t_schema, owl_thing },
    { { s_foaf, 0, 1 }, { s_foaf, 0, 97 }, foaf_theme, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_thumbnail, t_schema, foaf_image },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_thumbnail, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_tipjar, t_schema, foaf_document },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_tipjar, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_tipjar, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_title, t_text },
//    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_topic, t_schema, owl_thing },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_topic, t_text },
//    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_topic_interest, t_schema, owl_thing },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_topic_interest, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_weblog, t_schema, foaf_document },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_weblog, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_weblog, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_workinfohomepage, t_schema, foaf_document },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_workinfohomepage, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_workinfohomepage, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_workplacehomepage, t_schema, foaf_document },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_workplacehomepage, t_url },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_workplacehomepage, t_text },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_yahoochatid, t_text },

    // good relations
    { { s_gr, 1, 0 }, { 0, 0 }, gr_acceptedpaymentmethods, t_schema, gr_paymentmethodcreditcard },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_acceptedpaymentmethods, t_schema, gr_paymentmethod },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_addon, t_schema, gr_offering },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_addon, t_schema, sch_offer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_advancebookingrequirement, t_schema, gr_quantitativevalueinteger },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_amountofgoods, t_schema, xsd_float },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_amountofgoods, t_real },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_appliestodeliverymethod, t_schema, gr_deliverymodeparcelservice },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_appliestodeliverymethod, t_schema, gr_deliverymethod },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_appliestopaymentmethod, t_schema, gr_paymentmethodcreditcard },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_appliestopaymentmethod, t_schema, gr_paymentmethod },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_availabilityends, t_schema, xsd_datetime },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_availabilityends, t_datetime },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_availabilitystarts, t_schema, xsd_datetime },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_availabilitystarts, t_datetime },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_availableatorfrom, t_schema, gr_location },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_availabledeliverymethods, t_schema, gr_deliverymodeparcelservice },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_availabledeliverymethods, t_schema, gr_deliverymethod },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_billingincrement, t_schema, xsd_float },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_billingincrement, t_real },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_category, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_closes, t_schema, xsd_time },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_closes, t_just_time },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_colour, t_schema, rdfs_literal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_colour, t_colour },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_colour, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_condition, t_schema, rdfs_literal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_condition, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_datatypeproductorserviceproperty, t_schema, rdfs_literal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_datatypeproductorserviceproperty, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliveryleadtime, t_schema, gr_quantitativevalueinteger },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_depth, t_schema, gr_quantitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_depth, t_schema, gr_quantitativevaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_depth, t_schema, gr_quantitativevalueinteger },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_description, t_schema, rdfs_literal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_description, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_displayposition, t_real },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_durationofwarrantyinmonths, t_schema, xsd_int },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_durationofwarrantyinmonths, t_integer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_eligiblecustomertypes, t_schema, gr_businessentitytype },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_eligibleduration, t_schema, gr_quantitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_eligibleduration, t_schema, gr_quantitativevaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_eligibleduration, t_schema, gr_quantitativevalueinteger },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_eligibleregions, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_eligibleregions, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_eligibletransactionvolume, t_schema, gr_deliverychargespecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_eligibletransactionvolume, t_schema, gr_paymentchargespecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_eligibletransactionvolume, t_schema, gr_pricespecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_eligibletransactionvolume, t_schema, gr_unitpricespecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_equal, t_schema, gr_qualitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_greater, t_schema, gr_qualitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_greaterorequal, t_schema, gr_qualitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasbrand, t_schema, gr_brand },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasbusinessfunction, t_schema, gr_businessentity },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasbusinessfunction, t_schema, gr_businessfunction },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasbusinessfunction, t_schema, gr_licence },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hascurrency, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hascurrency, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hascurrencyvalue, t_schema, xsd_float },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hascurrencyvalue, t_real },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasduns, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasduns, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasean_ucc_13, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasean_ucc_13, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_haseligiblequantity, t_schema, gr_quantitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_haseligiblequantity, t_schema, gr_quantitativevaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_haseligiblequantity, t_schema, gr_quantitativevalueinteger },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasgtin_14, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasgtin_14, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasgtin_8, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasgtin_8, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasgloballocationnumber, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasgloballocationnumber, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasisicv4, t_schema, xsd_int },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasisicv4, t_integer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasinventorylevel, t_schema, gr_quantitativevaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmakeandmodel, t_schema, gr_productorservicemodel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmanufacturer, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmaxcurrencyvalue, t_schema, xsd_float },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmaxcurrencyvalue, t_real },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmaxvalue, t_schema, rdfs_literal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmaxvalue, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmaxvaluefloat, t_schema, xsd_float },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmaxvaluefloat, t_real },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmaxvalueinteger, t_schema, xsd_int },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmaxvalueinteger, t_integer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmincurrencyvalue, t_schema, xsd_float },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmincurrencyvalue, t_real },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasminvalue, t_schema, rdfs_literal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasminvalue, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasminvaluefloat, t_schema, xsd_float },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasminvaluefloat, t_real },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasminvalueinteger, t_schema, xsd_int },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasminvalueinteger, t_integer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmpn, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasmpn, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasnaics, t_schema, xsd_int },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasnaics, t_integer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasnext, t_schema, gr_dayofweek },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasopeninghoursdayofweek, t_schema, gr_dayofweek },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasopeninghoursspecification, t_schema, gr_openinghoursspecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_haspos, t_schema, gr_location },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasprevious, t_schema, gr_dayofweek },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_haspricespecification, t_schema, gr_deliverychargespecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_haspricespecification, t_schema, gr_paymentchargespecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_haspricespecification, t_schema, gr_pricespecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_haspricespecification, t_schema, gr_unitpricespecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasstockkeepingunit, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasstockkeepingunit, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasunitofmeasurement, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasunitofmeasurement, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasvalue, t_schema, rdfs_literal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasvalue, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasvaluefloat, t_schema, xsd_float },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasvaluefloat, t_real },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasvalueinteger, t_schema, xsd_int },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_hasvalueinteger, t_integer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_haswarrantypromise, t_schema, gr_warrantypromise },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_haswarrantyscope, t_schema, gr_warrantyscope },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_height, t_schema, gr_quantitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_height, t_schema, gr_quantitativevaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_height, t_schema, gr_quantitativevalueinteger },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_includesobject, t_schema, gr_typeandquantitynode },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_includes, t_schema, gr_actualproductorserviceinstance },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_includes, t_schema, gr_individual },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_includes, t_schema, gr_productorservice },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_includes, t_schema, gr_productorservicemodel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_includes, t_schema, gr_locationofsalesorserviceprovisioning },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_includes, t_schema, gr_someitems },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isaccessoryorsparepartfor, t_schema, gr_actualproductorserviceinstance },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isaccessoryorsparepartfor, t_schema, gr_individual },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isaccessoryorsparepartfor, t_schema, gr_productorservice },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isaccessoryorsparepartfor, t_schema, gr_productorservicemodel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isaccessoryorsparepartfor, t_schema, gr_locationofsalesorserviceprovisioning },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isaccessoryorsparepartfor, t_schema, gr_someitems },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isconsumablefor, t_schema, gr_actualproductorserviceinstance },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isconsumablefor, t_schema, gr_individual },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isconsumablefor, t_schema, gr_productorservice },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isconsumablefor, t_schema, gr_productorservicemodel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isconsumablefor, t_schema, gr_locationofsalesorserviceprovisioning },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isconsumablefor, t_schema, gr_someitems },
    { { s_gr, 1, 0, SV_DEPRECATED }, { 0, 0 }, gr_islistprice, t_schema, xsd_boolean },
    { { s_gr, 1, 0, SV_DEPRECATED }, { 0, 0 }, gr_islistprice, t_xsdbool },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_issimilarto, t_schema, gr_actualproductorserviceinstance },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_issimilarto, t_schema, gr_individual },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_issimilarto, t_schema, gr_productorservice },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_issimilarto, t_schema, gr_productorservicemodel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_issimilarto, t_schema, gr_locationofsalesorserviceprovisioning },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_issimilarto, t_schema, gr_someitems },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_isvariantof, t_schema, gr_productorservicemodel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_legalname, t_schema, rdfs_literal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_legalname, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_lesser, t_schema, gr_qualitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_lesserorequal, t_schema, gr_qualitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_name, t_schema, rdfs_literal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_name, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_nonequal, t_schema, gr_qualitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offers, t_schema, gr_offering },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_opens, t_schema, xsd_time },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_opens, t_just_time },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_owns, t_schema, gr_individual },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_predecessorof, t_schema, gr_productorservicemodel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricetype, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricetype, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_qualitativeproductorserviceproperty, t_schema, gr_qualitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativeproductorserviceproperty, t_schema, gr_quantitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativeproductorserviceproperty, t_schema, gr_quantitativevaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativeproductorserviceproperty, t_schema, gr_quantitativevalueinteger },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_relatedwebservice, t_url },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_seeks, t_schema, gr_offering },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_serialnumber, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_serialnumber, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_successorof, t_schema, gr_productorservicemodel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_taxid, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_taxid, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_typeofgoods, t_schema, gr_individual },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_typeofgoods, t_schema, gr_someitems },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_validfrom, t_schema, xsd_datetime },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_validfrom, t_datetime },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_validto, t_schema, xsd_datetime },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_validto, t_datetime },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_valueaddedtaxincluded, t_schema, xsd_boolean },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_valueaddedtaxincluded, t_xsdbool },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_valuereference, t_schema, gr_qualitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_valuereference, t_schema, gr_quantitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_valuereference, t_schema, gr_quantitativevaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_valuereference, t_schema, gr_quantitativevalueinteger },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_vatid, t_schema, xsd_string },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_vatid, t_text },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_weight, t_schema, gr_quantitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_weight, t_schema, gr_quantitativevaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_weight, t_schema, gr_quantitativevalueinteger },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_width, t_schema, gr_quantitativevalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_width, t_schema, gr_quantitativevaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_width, t_schema, gr_quantitativevalueinteger },

    // ical
    { { s_ical, 1, 0 }, { 0, 0 }, ical_action, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_attendee, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_byday, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_bymonth, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_caladdress, t_url },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_calscale, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_categories, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_class, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_cn, t_existential },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_comment, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_component, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_cutype, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_date, t_just_date },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_datetime, t_datetime },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_daylight, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_description, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_dtend, t_datetime },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_dstamp, t_datetime },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_dtstart, t_datetime },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_duration, t_duration },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_freq, t_icalfreq },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_interval, t_icalfreq },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_language, t_lang },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_location, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_organiser, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_partstat, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_prodid, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_related, t_startend },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_role, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_rrule, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_rsvp, t_bool },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_sequence, t_integer },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_standard, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_summary, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_transp, t_transp },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_trigger, t_duration },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_tzid, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_tzname, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_tzoffsetfrom, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_tzoffsetto, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_uid, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_valarm, t_text },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_version, t_text },

    // living standard / microformats v1
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_additional_name, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_adr, t_schema_type, mft_adr },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_adr, t_schema_type, sty_context },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_adr, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_affliation, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_agent, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_agent, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_album, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_anniversary, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_amount, t_fixedpoint },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_attach, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_attach, t_generic },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_attendee, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_attendee, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_author, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_author, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_availability, t_mf_availability },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_bday, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_best, t_unsigned },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_bookmark, t_generic },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_brand, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_brand, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_categories, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_category, t_schema_type, sty_context },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_category, t_mf_category },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_category, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_class, t_mf_class },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_comment, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_contact, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_contributor, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_contributor, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_count, t_unsigned },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_country_name, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_created, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_currency, t_currency },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_dateline, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_dateline, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_description, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_dtend, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_dtexpired, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_dtlisted, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_dtreviewed, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_dtstart, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_duration, t_schema_type, sty_context },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_duration, t_duration },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_education, t_schema_type, mft_hcalendar },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_email, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_enclosure, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_entry, t_schema_type, mft_hentry },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_entry_content, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_entry_summary, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_entry_title, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_exdate, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_experience, t_schema_type, mft_hcalendar },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_extended_address, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_family_name, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_feed_category, t_generic },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_fn, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_gender_identity, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_geo, t_schema_type, mft_geo },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_geo, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_given_name, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_h, t_hour },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_hentry, t_schema_type, mft_hentry },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_hentry, t_schema_type, sty_context },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_hentry, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_honourific_prefix, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_honourific_suffix, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_identifier, t_schema_type, sty_context },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_impp, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_ingredient, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_instructions, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_item, t_schema_type, mft_hcalendar },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_item, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_item, t_schema_type, sty_context },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_item_licence, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_include, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_info, t_schema_type, mft_adr },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_info, t_schema_type, mft_geo },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_info, t_schema_type, mft_hcalendar },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_info, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_info, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_info, t_generic },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_key, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_kind, t_kind },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_label, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_lang, t_lang },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_last_modified, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_latitude, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_licence, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_lister, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_lister, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_listing_action, t_mf_listing_actions },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_listing, t_schema_type, mft_hlisting },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_locality, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_location, t_schema_type, mft_adr },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_location, t_schema_type, mft_geo },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_location, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_location, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_logo, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_longitude, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_mailer, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_member, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_member, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_method, t_mf_method },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_min, t_minute },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_n, t_schema_type, sty_context },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_n, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_nickname, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_note, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_nutrition, t_schema_type, sty_context },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_org, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_org, t_schema_type, sty_context },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_org, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_organiser, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_organiser, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_organisation_name, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_organisation_unit, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_partstat, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_payment, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_permalink, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_photo, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_photo, t_generic },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_player, t_generic },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_position, t_fixedpoint },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_post_office_box, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_postal_code, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_price, t_schema_type, sty_context },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_price, t_dosh },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_price, t_fixedpoint },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_price, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_principles, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_publications, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_published, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_rating, t_unsigned },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_rdate, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_region, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_rel, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_related, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_related, t_schema_type, sty_context },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_related, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_resources, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_rev, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_review, t_schema_type, mft_hreview },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_review, t_schema_type, mft_haggregate },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_reviewer, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_role, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_rrule, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_s, t_second },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_sample, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_sequence, t_unsigned },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_self, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_sex, t_sex },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_skill, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_sort_string, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_source_org, t_schema_type, mft_hcard },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_sound, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_status, t_mf_status },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_street_address, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_summary, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_tag, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_tags, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_tel, t_schema_type, sty_context },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_tel, t_tel_format },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_term, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_title, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_transp, t_transp },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_type, t_mf_identifier },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_type, t_mf_itemtype },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_type, t_mf_reviewtype },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_type, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_tz, t_tz },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_uid, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_updated, t_datetime },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_url, t_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_value, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_value_title, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_version, t_text },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_votes, t_unsigned },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_worst, t_unsigned },
    { { s_microformats, 1, 0 }, { 0, 0 }, mp_yield, t_text },

    // microformats v2
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_dt_accessed, t_datetime },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_dt_anniversary, t_datetime },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_dt_bday, t_datetime },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_dt_duration, t_datetime },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_dt_end, t_datetime },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_dt_expired, t_datetime },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_dt_listed, t_datetime },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_dt_published, t_datetime },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_dt_start, t_datetime },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_dt_rev, t_datetime },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_dt_updated, t_datetime },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_e_content, t_generic },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_e_description, t_generic },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_e_instructions, t_generic },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_h_breadcrumb, t_schema_type, mft2_hbreadcrumb },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_h_entry, t_schema_type, mft2_hentry },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_action, t_action },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_additional_name, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_adr, t_schema_type, mft2_hadr },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_adr, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_affiliation, t_schema_type, mft2_hcard },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_altitude, t_fixedpoint },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_attendee, t_schema_type, mft2_hcard },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_attendee, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_author, t_schema_type, mft2_hcard },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_author, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_average, t_fixedpoint },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_best, t_unsigned },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_brand, t_schema_type, mft2_hcard },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_brand, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_category, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_comment, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_contact, t_schema_type, mft2_hcard },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_content, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_count, t_unsigned },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_country_name, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_description, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_duration, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_education, t_schema_type, mft2_hevent },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_entry, t_schema_type, mft2_hentry },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_experience, t_schema_type, mft2_hevent },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_extended_address, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_family_name, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_region, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_gender_identity, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_geo, t_schema_type, mft2_hgeo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_geo, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_given_name, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_honourific_prefix, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_honourific_suffix, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_item, t_schema_type, mft2_hadr },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_item, t_schema_type, mft2_hcard },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_item, t_schema_type, mft2_hevent },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_item, t_schema_type, mft2_hgeo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_item, t_schema_type, mft2_hitem },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_item, t_schema_type, mft2_hproduct },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_item, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_job_title, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_ingredient, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_label, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_latitude, t_fixedpoint },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_lister, t_schema_type, mft2_hcard },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_locality, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_location, t_schema_type, mft2_hadr },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_location, t_schema_type, mft2_hgeo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_location, t_schema_type, mft2_hcard },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_location, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_longitude, t_fixedpoint },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_note, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_name, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_nickname, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_nutrition, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_org, t_schema_type, mft2_hcard },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_org, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_organisation_name, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_organisation_unit, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_price, t_dosh },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_post_office_box, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_postal_code, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_publication, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_rating, t_unsigned },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_read_of, t_schema_type, mft2_hcite },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_read_of, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_review, t_schema_type, mft2_hreview },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_review, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_role, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_rsvp, t_rsvp },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_sex, t_sex },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_size, t_unsigned },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_skill, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_sort_string, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_street_address, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_summary, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_tel, t_tel_format },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_tz, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_votes, t_unsigned },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_worst, t_unsigned },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_p_yield, t_text },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_audio, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_bookmark_of, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_email, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_featured, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_geo, t_schema_type, mft2_hgeo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_geo, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_identifier, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_impp, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_in_reply_to, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_like, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_like_of, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_listen_of, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_logo, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_key, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_photo, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_read_of, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_repost, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_repost_of, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_sound, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_syndication, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_translation_of, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_uid, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_url, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_video, t_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mp2_u_watch_of, t_url },

    // open graph
    { { s_music, 1, 0 }, { 0, 0 }, og_music_album, t_text },
    { { s_music, 1, 0 }, { 0, 0 }, og_music_album_disc, t_1_more },
    { { s_music, 1, 0 }, { 0, 0 }, og_music_album_track, t_1_more },
    { { s_music, 1, 0 }, { 0, 0 }, og_music_creator, t_text },
    { { s_music, 1, 0 }, { 0, 0 }, og_music_duration, t_1_more },
    { { s_music, 1, 0 }, { 0, 0 }, og_music_musician, t_text },
    { { s_music, 1, 0 }, { 0, 0 }, og_music_release_date, t_datetime },
    { { s_music, 1, 0 }, { 0, 0 }, og_music_song, t_text },
    { { s_music, 1, 0 }, { 0, 0 }, og_music_song_disc, t_1_more },
    { { s_music, 1, 0 }, { 0, 0 }, og_music_song_track, t_1_more },

    // open tag
    { { s_og, 1, 0 }, { 0, 0 }, og_audio, t_url },
    { { s_og, 1, 0 }, { 0, 0 }, og_audio_secure_url, t_url },
    { { s_og, 1, 0 }, { 0, 0 }, og_audio_type, t_mime },
    { { s_og, 1, 0 }, { 0, 0 }, og_audio_url, t_url },
    { { s_og, 1, 0 }, { 0, 0 }, od_description, t_text },
    { { s_og, 1, 0 }, { 0, 0 }, og_determiner, t_text },
    { { s_og, 1, 0 }, { 0, 0 }, og_image, t_url },
    { { s_og, 1, 0 }, { 0, 0 }, og_image_alt, t_text },
    { { s_og, 1, 0 }, { 0, 0 }, og_image_height, t_unsigned },
    { { s_og, 1, 0 }, { 0, 0 }, og_image_secure_url, t_url },
    { { s_og, 1, 0 }, { 0, 0 }, og_image_type, t_mime },
    { { s_og, 1, 0 }, { 0, 0 }, og_image_url, t_url },
    { { s_og, 1, 0 }, { 0, 0 }, og_image_width, t_unsigned },
    { { s_og, 1, 0 }, { 0, 0 }, og_locale, t_lang },
    { { s_og, 1, 0 }, { 0, 0 }, og_locale_alternative, t_langs },
    { { s_og, 1, 0 }, { 0, 0 }, og_site_name, t_text },
    { { s_og, 1, 0 }, { 0, 0 }, og_title, t_text },
    { { s_og, 1, 0 }, { 0, 0 }, og_type, t_ogtype },
    { { s_og, 1, 0 }, { 0, 0 }, og_url, t_url },
    { { s_og, 1, 0 }, { 0, 0 }, og_video, t_url },
    { { s_og, 1, 0 }, { 0, 0 }, og_video_alt, t_text },
    { { s_og, 1, 0 }, { 0, 0 }, og_video_height, t_1_more },
    { { s_og, 1, 0 }, { 0, 0 }, og_video_secure_url, t_url },
    { { s_og, 1, 0 }, { 0, 0 }, og_video_type, t_mime },
    { { s_og, 1, 0 }, { 0, 0 }, og_video_url, t_url },
    { { s_og, 1, 0 }, { 0, 0 }, og_video_width, t_1_more },

    // poetry
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_age, t_text },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_containedinpoem, t_schema, poetry_poem },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_containedinpoem, t_url },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_containspoem, t_schema, poetry_poem },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_containspoem, t_url },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_hasform, t_text },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_hasstanza, t_schema, poetry_stanza },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_hasstanza, t_url },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_indialect, t_schema, poetry_dialect },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_indialect, t_url },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_ischorus, t_text },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_islaidout, t_bool },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_keepsstrictform, t_bool },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_lines, t_integer },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_medium, t_text },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_metre, t_text },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_poet, t_schema, sch_person },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_poet, t_text },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_region, t_schema, sch_place },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_region, t_text },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_rhymingscheme, t_text },
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_subform, t_text },

    // open tag
    { { s_profile, 1, 0 }, { 0, 0 }, og_profile_first_name, t_text },
    { { s_profile, 1, 0 }, { 0, 0 }, og_profile_gender, t_text },
    { { s_profile, 1, 0 }, { 0, 0 }, og_profile_last_name, t_text },
    { { s_profile, 1, 0 }, { 0, 0 }, og_profile_username, t_gender },

    // RDF
    { { s_rdf, 1, 0 }, { 0, 0 }, rdf_direction, t_direction },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdf_first, t_text },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdf_language, t_lang },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdf_object, t_text },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdf_predicate, t_text },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdf_rest, t_schema, rdf_list },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdf_subject, t_text },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdf_type, t_text },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdf_value, t_text },

    // RDFs
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_comment, t_schema, rdfs_literal },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_comment, t_text },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_domain, t_schema, rdfs_class },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_isdefinedby, t_schema, rdfs_resource },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_label, t_schema, rdfs_literal },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_label, t_text },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_member, t_schema, rdfs_resource },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_range, t_schema, rdfs_class },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_seealso, t_schema, rdfs_resource },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_subclassof, t_schema, rdfs_class },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_subpropertyof, t_schema, rdf_property },

    // schema.org
    { { 2, 0 }, { 0, 0 }, sp_about, t_schema_type, sch_thing },
    { { 3, 5 }, { 0, 0 }, sp_abridged, t_schema_type, sch_boolean },
    { { 3, 5 }, { 0, 0 }, sp_abridged, t_bool },
    { { 4, 0 }, { 0, 0 }, sp_abstract, t_schema_type, sch_text },
    { { 4, 0 }, { 0, 0 }, sp_abstract, t_text },
    { { 2, 0 }, { 0, 0 }, sp_accelerationtime, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_acceptedanswer, t_schema_type, sch_answer },
    { { 3, 4 }, { 0, 0 }, sp_acceptedanswer, t_schema_type, sch_itemlist },
    { { 2, 0 }, { 0, 0 }, sp_acceptedoffer, t_schema_type, sch_offer },
    { { 3, 0 }, { 0, 0 }, sp_acceptedpaymentmethod, t_schema_type, sch_loanorcredit },
    { { 2, 0 }, { 0, 0 }, sp_acceptedpaymentmethod, t_schema_type, sch_paymentmethod },
    { { 2, 0 }, { 0, 0 }, sp_acceptsreservations, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_acceptsreservations, t_text },
    { { 2, 0 }, { 0, 0 }, sp_acceptsreservations, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_acceptsreservations, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_acceptsreservations, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_acceptsreservations, t_url },
    { { 2, 0 }, { 0, 0 }, sp_accesscode, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_accesscode, t_text },
    { { 3, 0 }, { 0, 0 }, sp_accessmode, t_schema_type, sch_text },
    { { 3, 0 }, { 0, 0 }, sp_accessmode, t_text },
    { { 3, 8 }, { 0, 0 }, sp_accessmodesufficient, t_schema_type, sch_itemlist },
    { { 3, 0 }, { 3, 7 }, sp_accessmodesufficient, t_schema_type, sch_text },
    { { 3, 0 }, { 3, 7 }, sp_accessmodesufficient, t_text },
    { { 2, 0 }, { 0, 0 }, sp_accessibilityapi, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_accessibilityapi, t_text },
    { { 2, 0 }, { 0, 0 }, sp_accessibilitycontrol, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_accessibilitycontrol, t_text },
    { { 2, 0 }, { 0, 0 }, sp_accessibilityfeature, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_accessibilityfeature, t_text },
    { { 2, 0 }, { 0, 0 }, sp_accessibilityhazard, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_accessibilityhazard, t_text },
    { { 2, 0 }, { 0, 0 }, sp_accessibilitysummary, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_accessibilitysummary, t_text },
    { { 5, 0 }, { 0, 0 }, sp_accommodationcategory, t_schema_type, sch_text },
    { { 5, 0 }, { 0, 0 }, sp_accommodationcategory, t_text },
    { { 6, 0 }, { 0, 0 }, sp_accommodationfloorplan, t_schema_type, sch_floorplan },
    { { 2, 0 }, { 0, 0 }, sp_accountid, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_accountid, t_text },
    { { 3, 5 }, { 0, 0 }, sp_accountminimuminflow, t_schema_type, sch_monetaryamount },
    { { 3, 5 }, { 0, 0 }, sp_accountoverdraftlimit, t_schema_type, sch_monetaryamount },
    { { 2, 0 }, { 0, 0 }, sp_accountableperson, t_schema_type, sch_person },
    { { 7, 0 }, { 0, 0 }, sp_acquirelicensepage, t_schema_type, sch_creativework },
    { { 7, 0 }, { 0, 0 }, sp_acquirelicensepage, t_schema_type, sch_url },
    { { 7, 0 }, { 0, 0 }, sp_acquirelicensepage, t_url },
    { { 2, 0 }, { 0, 0 }, sp_acquiredfrom, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_acquiredfrom, t_schema_type, sch_person },
    { { 3, 5 }, { 0, 0 }, sp_acrisscode, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_acrisscode, t_text },
    { { 3, 5 }, { 6, 0 }, sp_action, t_schema_type, sch_text },
    { { 3, 5 }, { 6, 0 }, sp_action, t_text },
    { { 3, 5 }, { 0, 0 }, sp_actionaccessibilityrequirement, t_schema_type, sch_actionaccessspecification },
    { { 2, 0 }, { 0, 0 }, sp_actionapplication, t_schema_type, sch_softwareapplication },
    { { 2, 0 }, { 0, 0 }, sp_actionoption, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_actionoption, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_actionoption, t_text },
    { { 2, 2 }, { 0, 0 }, sp_actionplatform, t_schema_type, sch_text },
    { { 2, 2 }, { 0, 0 }, sp_actionplatform, t_text },
    { { 2, 2 }, { 0, 0 }, sp_actionplatform, t_schema_type, sch_url },
    { { 2, 2 }, { 0, 0 }, sp_actionplatform, t_url },
    { { 2, 0 }, { 0, 0 }, sp_actionstatus, t_schema_type, sch_actionstatustype },
    { { 3, 5 }, { 0, 0 }, sp_actionablefeedbackpolicy, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_actionablefeedbackpolicy, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_actionablefeedbackpolicy, t_url },
    { { 2, 0 }, { 0, 0 }, sp_activeingredient, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_activeingredient, t_text },
    { { 2, 0 }, { 0, 0 }, sp_activityduration, t_schema_type, sch_duration },
    { { 2, 0 }, { 0, 0 }, sp_activityduration, t_duration },
    { { 3, 5 }, { 0, 0 }, sp_activityduration, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_activityfrequency, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_activityfrequency, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_activityfrequency, t_text },
    { { 2, 0 }, { 0, 0 }, sp_actor, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_addon, t_schema_type, sch_offer },
    { { 2, 0 }, { 0, 0 }, sp_additionalname, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_additionalname, t_text },
    { { 2, 0 }, { 0, 0 }, sp_additionalnumberofguests, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_additionalproperty, t_schema_type, sch_propertyvalue },
    { { 2, 0 }, { 0, 0 }, sp_additionaltype, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_additionaltype, t_url },
    { { 2, 0 }, { 0, 0 }, sp_additionalvariable, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_additionalvariable, t_text },
    { { 2, 0 }, { 0, 0 }, sp_address, t_schema_type, sch_postaladdress },
    { { 2, 0 }, { 0, 0 }, sp_address, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_address, t_text },
    { { 2, 0 }, { 0, 0 }, sp_addresscountry, t_schema_type, sch_country },
    { { 2, 2 }, { 0, 0 }, sp_addresscountry, t_schema_type, sch_text },
    { { 2, 2 }, { 0, 0 }, sp_addresscountry, t_text },
    { { 2, 0 }, { 0, 0 }, sp_addresslocality, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_addresslocality, t_text },
    { { 2, 0 }, { 0, 0 }, sp_addressregion, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_addressregion, t_text },
    { { 2, 0 }, { 0, 0 }, sp_administrationroute, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_administrationroute, t_text },
    { { 2, 0 }, { 0, 0 }, sp_advancebookingrequirement, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_adverseoutcome, t_schema_type, sch_medicalentity },
    { { 2, 0 }, { 0, 0 }, sp_affectedby, t_schema_type, sch_drug },
    { { 2, 0 }, { 0, 0 }, sp_affiliation, t_schema_type, sch_organisation },
    { { 3, 3 }, { 0, 0 }, sp_aftermedia, t_schema_type, sch_mediaobject },
    { { 3, 4 }, { 0, 0 }, sp_aftermedia, t_schema_type, sch_url },
    { { 3, 4 }, { 0, 0 }, sp_aftermedia, t_url },
    { { 2, 0 }, { 0, 0 }, sp_agent, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_agent, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_aggregaterating, t_schema_type, sch_aggregaterating },
    { { 2, 0 }, { 0, 0 }, sp_aircraft, t_schema_type, sch_vehicle },
    { { 2, 0 }, { 0, 0 }, sp_aircraft, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_aircraft, t_text },
    { { 2, 0 }, { 0, 0 }, sp_album, t_schema_type, sch_musicalbum },
    { { 2, 0 }, { 0, 0 }, sp_albumproductiontype, t_schema_type, sch_musicalbumproductiontype },
    { { 2, 0 }, { 0, 0 }, sp_albumrelease, t_schema_type, sch_musicrelease },
    { { 2, 0 }, { 0, 0 }, sp_albumreleasetype, t_schema_type, sch_musicalbumreleasetype },
    { { 2, 0 }, { 0, 0 }, sp_alcoholwarning, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_alcoholwarning, t_text },
    { { 2, 0 }, { 0, 0 }, sp_algorithm, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_algorithm, t_text },
    { { 2, 0 }, { 0, 0 }, sp_alignmenttype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_alignmenttype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_alternativename, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_alternativename, t_text },
    { { 2, 0 }, { 0, 0 }, sp_alternativeheadline, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_alternativeheadline, t_text },
    { { 13, 0 }, { 0, 0 }, sp_alternativeof, t_schema_type, sch_gene },
    { { 2, 0 }, { 0, 0 }, sp_alumni, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_alumniof, t_schema_type, sch_educationalorganisation },
    { { 2, 2 }, { 0, 0 }, sp_alumniof, t_schema_type, sch_organisation },
    { { 3, 1 }, { 0, 0 }, sp_amenityfeature, t_schema_type, sch_locationfeaturespecification },
    { { 2, 0 }, { 0, 0 }, sp_amount, t_schema_type, sch_float },
    { { 2, 0 }, { 0, 0 }, sp_amount, t_fixedpoint },
    { { 3, 0 }, { 0, 0 }, sp_amount, t_schema_type, sch_monetaryamount },
    { { 2, 0 }, { 0, 0 }, sp_amountofthisgood, t_schema_type, sch_float },
    { { 2, 0 }, { 0, 0 }, sp_amountofthisgood, t_fixedpoint },
    { { 7, 2 }, { 0, 0 }, sp_announcementlocation, t_schema_type, sch_civicstructure },
    { { 7, 2 }, { 0, 0 }, sp_announcementlocation, t_schema_type, sch_localbusiness },
    { { 3, 0 }, { 0, 0 }, sp_annualpercentagerate, t_schema_type, sch_quantitativevalue },
    { { 3, 0 }, { 0, 0 }, sp_annualpercentagerate, t_schema_type, sch_float },
    { { 3, 0 }, { 0, 0 }, sp_annualpercentagerate, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_answercount, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_answercount, t_integer },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_antagonist, t_schema_type, sch_muscle },
    { { 3, 5 }, { 0, 0 }, sp_appearance, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_applicablelocation, t_schema_type, sch_administrativearea },
    { { 6, 0 }, { 0, 0 }, sp_applicantcontact, t_schema_type, sch_contactpoint },
    { { 3, 5 }, { 0, 0 }, sp_applicantlocationrequirements, t_schema_type, sch_administrativearea },
    { { 2, 0 }, { 0, 0 }, sp_applicationcategory, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_applicationcategory, t_text },
    { { 2, 0 }, { 0, 0 }, sp_applicationcategory, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_applicationcategory, t_url },
    { { 6, 0 }, { 0, 0 }, sp_applicationdeadline, t_schema_type, sch_date },
    { { 6, 0 }, { 0, 0 }, sp_applicationdeadline, t_just_date },
    { { 6, 0 }, { 0, 0 }, sp_applicationstartdate, t_schema_type, sch_date },
    { { 6, 0 }, { 0, 0 }, sp_applicationstartdate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_applicationsubcategory, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_applicationsubcategory, t_text },
    { { 2, 0 }, { 0, 0 }, sp_applicationsubcategory, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_applicationsubcategory, t_url },
    { { 2, 0 }, { 0, 0 }, sp_applicationsuite, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_applicationsuite, t_text },
    { { 2, 0 }, { 0, 0 }, sp_appliestodeliverymethod, t_schema_type, sch_deliverymethod },
    { { 2, 0 }, { 0, 0 }, sp_appliestopaymentmethod, t_schema_type, sch_paymentmethod },
    { { 13, 0 }, { 0, 0 }, sp_archivedat, t_schema_type, sch_webpage },
    { { 13, 0 }, { 0, 0 }, sp_archivedat, t_schema_type, sch_url },
    { { 13, 0 }, { 0, 0 }, sp_archivedat, t_url },
    { { 3, 5 }, { 0, 0 }, sp_archiveheld, t_schema_type, sch_archivecomponent },
    { { 2, 0 }, { 0, 0 }, sp_areaserved, t_schema_type, sch_administrativearea },
    { { 3, 0 }, { 0, 0 }, sp_areaserved, t_schema_type, sch_geoshape },
    { { 3, 0 }, { 0, 0 }, sp_areaserved, t_schema_type, sch_place },
    { { 3, 0 }, { 0, 0 }, sp_areaserved, t_schema_type, sch_text },
    { { 3, 0 }, { 0, 0 }, sp_areaserved, t_text },
    { { 2, 0 }, { 0, 0 }, sp_arrivalairport, t_schema_type, sch_airport },
    { { 9, 0 }, { 0, 0 }, sp_arrivalboatterminal, t_schema_type, sch_boatterminal },
    { { 2, 0 }, { 0, 0 }, sp_arrivalbusstop, t_schema_type, sch_busstation },
    { { 2, 0 }, { 0, 0 }, sp_arrivalbusstop, t_schema_type, sch_busstop },
    { { 2, 0 }, { 0, 0 }, sp_arrivalgate, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_arrivalgate, t_text },
    { { 2, 0 }, { 0, 0 }, sp_arrivalplatform, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_arrivalplatform, t_text },
    { { 2, 0 }, { 0, 0 }, sp_arrivalstation, t_schema_type, sch_trainstation },
    { { 2, 0 }, { 0, 0 }, sp_arrivalterminal, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_arrivalterminal, t_text },
    { { 2, 0 }, { 0, 0 }, sp_arrivaltime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_arrivaltime, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_arrivaltime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_arrivaltime, t_datetime_absolute },
    { { 2, 0 }, { 0, 0 }, sp_artedition, t_schema_type, sch_float },
    { { 2, 0 }, { 0, 0 }, sp_artedition, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_artedition, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_artedition, t_text },
    { { 2, 0 }, { 0, 0 }, sp_artmedium, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_artmedium, t_text },
    { { 2, 0 }, { 0, 0 }, sp_artmedium, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_artmedium, t_url },
    { { 2, 0 }, { 0, 0 }, sp_arterialbranch, t_schema_type, sch_anatomicalstructure },
    { { 2, 0 }, { 0, 0 }, sp_artform, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_artform, t_text },
    { { 2, 0 }, { 0, 0 }, sp_artform, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_artform, t_url },
    { { 2, 0 }, { 0, 0 }, sp_articlebody, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_articlebody, t_text },
    { { 2, 0 }, { 0, 0 }, sp_articlesection, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_articlesection, t_text },
    { { 3, 5 }, { 0, 0 }, sp_artist, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_artworksurface, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_artworksurface, t_text },
    { { 2, 0 }, { 0, 0 }, sp_artworksurface, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_artworksurface, t_url },
    { { 2, 0 }, { 0, 0 }, sp_aspect, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_aspect, t_text },
    { { 2, 0 }, { 0, 0 }, sp_assemblyversion, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_assemblyversion, t_text },
    { { 2, 0 }, { 0, 0 }, sp_assess, t_schema_type, sch_definedterm },
    { { 8, 0 }, { 0, 0 }, sp_assess, t_schema_type, sch_text },
    { { 8, 0 }, { 0, 0 }, sp_assess, t_text },
    { { 9, 0 }, { 0, 0 }, sp_assesses, t_schema_type, sch_definedterm },
    { { 9, 0 }, { 0, 0 }, sp_assesses, t_schema_type, sch_text },
    { { 9, 0 }, { 0, 0 }, sp_assesses, t_text },
    { { 2, 0 }, { 0, 0 }, sp_associatedanatomy, t_schema_type, sch_anatomicalstructure },
    { { 2, 0 }, { 0, 0 }, sp_associatedanatomy, t_schema_type, sch_anatomicalsystem },
    { { 2, 0 }, { 0, 0 }, sp_associatedanatomy, t_schema_type, sch_superficialanatomy },
    { { 2, 0 }, { 0, 0 }, sp_associatedarticle, t_schema_type, sch_newsarticle },
    { { 13, 0 }, { 0, 0 }, sp_associatedclaimreview, t_schema_type, sch_review },
    { { 13, 0 }, { 0, 0 }, sp_associateddisease, t_schema_type, sch_medicalcondition },
    { { 13, 0 }, { 0, 0 }, sp_associateddisease, t_schema_type, sch_propertyvalue },
    { { 13, 0 }, { 0, 0 }, sp_associateddisease, t_schema_type, sch_url },
    { { 13, 0 }, { 0, 0 }, sp_associateddisease, t_url },
    { { 2, 0 }, { 0, 0 }, sp_associatedmedia, t_schema_type, sch_mediaobject },
    { { 13, 0 }, { 0, 0 }, sp_associatedmediareview, t_schema_type, sch_review },
    { { 2, 0 }, { 0, 0 }, sp_associatedpathophysiology, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_associatedpathophysiology, t_text },
    { { 13, 0 }, { 0, 0 }, sp_associatedreview, t_schema_type, sch_review },
    { { 2, 0 }, { 0, 0 }, sp_athlete, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_attendee, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_attendee, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_audience, t_schema_type, sch_audience },
    { { 2, 0 }, { 0, 0 }, sp_audiencetype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_audiencetype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_audio, t_schema_type, sch_audioobject },
    { { 3, 5 }, { 0, 0 }, sp_audio, t_schema_type, sch_clip },
    { { 5, 0 }, { 0, 0 }, sp_audio, t_schema_type, sch_musicrecording },
    { { 3, 5 }, { 0, 0 }, sp_authenticator, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_author, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_author, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_availabedeliverymethod, t_schema_type, sch_deliverymethod },
    { { 2, 0 }, { 0, 0 }, sp_availability, t_schema_type, sch_itemavailability },
    { { 2, 0 }, { 0, 0 }, sp_availabilityends, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_availabilityends, t_datetime },
    { { 3, 5 }, { 0, 0 }, sp_availabilityends, t_datetime_absolute },
    { { 3, 8 }, { 0, 0 }, sp_availabilityends, t_schema_type, sch_date },
    { { 3, 8 }, { 0, 0 }, sp_availabilityends, t_just_date },
    { { 3, 8 }, { 0, 0 }, sp_availabilityends, t_schema_type, sch_time },
    { { 3, 8 }, { 0, 0 }, sp_availabilityends, t_just_time },
    { { 2, 0 }, { 0, 0 }, sp_availabilitystarts, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_availabilitystarts, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_availabilitystarts, t_datetime_absolute },
    { { 3, 5 }, { 0, 0 }, sp_availabilitystarts, t_schema_type, sch_date },
    { { 3, 5 }, { 0, 0 }, sp_availabilitystarts, t_just_date },
    { { 3, 8 }, { 0, 0 }, sp_availabilitystarts, t_schema_type, sch_time },
    { { 3, 8 }, { 0, 0 }, sp_availabilitystarts, t_just_time },
    { { 2, 0 }, { 0, 0 }, sp_availableatorfrom, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_availablechannel, t_schema_type, sch_servicechannel },
    { { 2, 0 }, { 0, 0 }, sp_availabledeliverymethod, t_schema_type, sch_deliverymethod },
    { { 2, 0 }, { 0, 0 }, sp_availablefrom, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_availablefrom, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_availablein, t_schema_type, sch_administrativearea },
    { { 2, 0 }, { 0, 0 }, sp_availablelanguage, t_schema_type, sch_language },
    { { 3, 0 }, { 0, 0 }, sp_availablelanguage, t_schema_type, sch_text },
    { { 3, 0 }, { 0, 0 }, sp_availablelanguage, t_text },
    { { 2, 0 }, { 0, 0 }, sp_availablelocation, t_schema_type, sch_administrativearea },
    { { 2, 0 }, { 0, 0 }, sp_availableondevice, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_availableondevice, t_text },
    { { 2, 0 }, { 0, 0 }, sp_availableservice, t_schema_type, sch_medicalprocedure },
    { { 2, 0 }, { 0, 0 }, sp_availableservice, t_schema_type, sch_medicaltest },
    { { 2, 0 }, { 0, 0 }, sp_availableservice, t_schema_type, sch_medicaltherapy },
    { { 2, 0 }, { 0, 0 }, sp_availablestrength, t_schema_type, sch_drugstrength },
    { { 2, 0 }, { 0, 0 }, sp_availabletest, t_schema_type, sch_medicaltest },
    { { 2, 0 }, { 0, 0 }, sp_availablethrough, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_availablethrough, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_award, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_award, t_text },
    { { 2, 0 }, { 0, 0 }, sp_awayteam, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_awayteam, t_schema_type, sch_sportsteam },
    { { 2, 0 }, { 0, 0 }, sp_background, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_background, t_text },
    { { 3, 5 }, { 0, 0 }, sp_backstory, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_backstory, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_backstory, t_text },
    { { 3, 5 }, { 0, 0 }, sp_bankaccounttype, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_bankaccounttype, t_text },
    { { 3, 5 }, { 0, 0 }, sp_bankaccounttype, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_bankaccounttype, t_url },
    { { 2, 0 }, { 0, 0 }, sp_basesalary, t_schema_type, sch_float },
    { { 2, 0 }, { 0, 0 }, sp_basesalary, t_fixedpoint },
    { { 3, 3 }, { 0, 0 }, sp_basesalary, t_schema_type, sch_monetaryamount },
    { { 2, 0 }, { 0, 0 }, sp_basesalary, t_schema_type, sch_pricespecification },
    { { 3, 3 }, { 0, 0 }, sp_bccrecipient, t_schema_type, sch_contactpoint },
    { { 3, 3 }, { 0, 0 }, sp_bccrecipient, t_schema_type, sch_organisation },
    { { 3, 3 }, { 0, 0 }, sp_bccrecipient, t_schema_type, sch_person },
    { { 3, 1 }, { 0, 0 }, sp_bed, t_schema_type, sch_beddetails },
    { { 3, 5 }, { 0, 0 }, sp_bed, t_schema_type, sch_bedtype },
    { { 3, 1 }, { 0, 0 }, sp_bed, t_schema_type, sch_text },
    { { 3, 1 }, { 0, 0 }, sp_bed, t_text },
    { { 3, 3 }, { 0, 0 }, sp_beforemedia, t_schema_type, sch_mediaobject },
    { { 3, 4 }, { 0, 0 }, sp_beforemedia, t_schema_type, sch_url },
    { { 3, 4 }, { 0, 0 }, sp_beforemedia, t_url },
    { { 3, 5 }, { 0, 0 }, sp_beneficiarybank, t_schema_type, sch_bankorcreditunion },
    { { 3, 5 }, { 0, 0 }, sp_beneficiarybank, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_beneficiarybank, t_text },
    { { 3, 5 }, { 0, 0 }, sp_benefitssummaryurl, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_benefitssummaryurl, t_url },
    { { 2, 0 }, { 0, 0 }, sp_bestrating, t_schema_type, sch_float },
    { { 2, 0 }, { 0, 0 }, sp_bestrating, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_bestrating, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_bestrating, t_text },
    { { 2, 0 }, { 0, 0 }, sp_billingaddress, t_schema_type, sch_postaladdress },
    { { 11, 0 }, { 0, 0 }, sp_billingduration, t_schema_type, sch_quantitativevalue },
    { { 11, 0 }, { 0, 0 }, sp_billingduration, t_schema_type, sch_number },
    { { 11, 0 }, { 0, 0 }, sp_billingduration, t_duration },
    { { 11, 0 }, { 0, 0 }, sp_billingduration, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_billingincrement, t_schema_type, sch_float },
    { { 2, 0 }, { 0, 0 }, sp_billingincrement, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_billingperiod, t_schema_type, sch_duration },
    { { 2, 0 }, { 0, 0 }, sp_billingperiod, t_duration },
    { { 11, 0 }, { 0, 0 }, sp_billingstart, t_schema_type, sch_number  },
    { { 11, 0 }, { 0, 0 }, sp_billingstart, t_fixedpoint },
    { { 13, 0 }, { 0, 0 }, sp_biocheminteraction, t_schema_type, sch_biochementity },
    { { 13, 0 }, { 0, 0 }, sp_biochemsimilarity, t_schema_type, sch_biochementity },
    { { 13, 0 }, { 0, 0 }, sp_biologicalrole, t_schema_type, sch_definedterm },
    { { 2, 0 }, { 0, 0 }, sp_biomechanicalclass, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_biomechanicalclass, t_text },
    { { 2, 0 }, { 0, 0 }, sp_birthdate, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_birthdate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_birthplace, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_bitrate, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_bitrate, t_text },
    { { 2, 0 }, { 0, 0 }, sp_blogpost, t_schema_type, sch_blogposting },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_bloodsupply, t_schema_type, sch_vessel },
    { { 2, 0 }, { 0, 0 }, sp_boardinggroup, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_boardinggroup, t_text },
    { { 2, 0 }, { 0, 0 }, sp_boardingpolicy, t_schema_type, sch_boardingpolicytype },
    { { 2, 0 }, { 0, 0 }, sp_boardingpolicy, t_schema_type, sch_boardingpolicytype},
    { { 2, 0 }, { 0, 0 }, sp_bodylocation, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_bodylocation, t_text },
    { { 3, 5 }, { 0, 0 }, sp_bodytype, t_schema_type, sch_qualitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_bodytype, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_bodytype, t_text },
    { { 3, 5 }, { 0, 0 }, sp_bodytype, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_bodytype, t_url },
    { { 2, 0 }, { 0, 0 }, sp_bookedition, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_bookedition, t_text },
    { { 2, 0 }, { 0, 0 }, sp_bookformat, t_schema_type, sch_bookformattype },
    { { 2, 0 }, { 0, 0 }, sp_bookingtime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_bookingtime, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_box, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_box, t_text },
    { { 2, 0 }, { 0, 0 }, sp_branch, t_schema_type, sch_nerve },
    { { 2, 2 }, { 0, 0 }, sp_branchcode, t_schema_type, sch_text },
    { { 2, 2 }, { 0, 0 }, sp_branchcode, t_text },
    { { 2, 0 }, { 0, 0 }, sp_brand, t_schema_type, sch_brand },
    { { 2, 0 }, { 0, 0 }, sp_brand, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_breadcrumb, t_schema_type, sch_breadcrumblist },
    { { 2, 0 }, { 0, 0 }, sp_breadcrumb, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_breadcrumb, t_text },
    { { 2, 0 }, { 0, 0 }, sp_breastfeedingwarning, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_breastfeedingwarning, t_text },
    { { 2, 0 }, { 0, 0 }, sp_broadcastaffiliateof, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_broadcastchannelid, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_broadcastchannelid, t_text },
    { { 2, 0 }, { 0, 0 }, sp_broadcastdisplayname, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_broadcastdisplayname, t_text },
    { { 3, 5 }, { 0, 0 }, sp_broadcastfrequency, t_schema_type, sch_broadcastfrequencyspecification },
    { { 3, 5 }, { 0, 0 }, sp_broadcastfrequency, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_broadcastfrequency, t_text },
    { { 3, 5 }, { 0, 0 }, sp_broadcastfrequencyvalue, t_schema_type, sch_float },
    { { 3, 5 }, { 0, 0 }, sp_broadcastfrequencyvalue, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_broadcastfrequencyvalue, t_schema_type, sch_quantitativevalue },
    { { 2, 2 }, { 0, 0 }, sp_broadcastofevent, t_schema_type, sch_event },
    { { 2, 0 }, { 0, 0 }, sp_broadcastservicetier, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_broadcastservicetier, t_text },
    { { 3, 5 }, { 0, 0 }, sp_broadcastsignalmodulation, t_schema_type, sch_float },
    { { 3, 5 }, { 0, 0 }, sp_broadcastsignalmodulation, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_broadcastsignalmodulation, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_broadcastsubchannel, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_broadcastsubchannel, t_text },
    { { 2, 0 }, { 0, 0 }, sp_broadcasttimezone, t_tz },
    { { 2, 0 }, { 0, 0 }, sp_broadcaster, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_broker, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_broker, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_browserrequirements, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_browserrequirements, t_text },
    { { 2, 0 }, { 0, 0 }, sp_busname, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_busname, t_text },
    { { 2, 0 }, { 0, 0 }, sp_busnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_busnumber, t_text },
    { { 8, 0 }, { 0, 0 }, sp_businessdays, t_schema_type, sch_openinghoursspecification },
    { { 2, 0 }, { 0, 0 }, sp_businessfunction, t_schema_type, sch_businessfunction },
    { { 13, 0 }, { 0, 0 }, sp_buyer, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_buyer, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_byartist, t_schema_type, sch_musicgroup },
    { { 3, 5 }, { 0, 0 }, sp_byartist, t_schema_type, sch_person },
    { { 3, 5 }, { 0, 0 }, sp_byday, t_schema_type, sch_dayofweek },
    { { 6, 0 }, { 0, 0 }, sp_byday, t_schema_type, sch_text },
    { { 6, 0 }, { 0, 0 }, sp_byday, t_text },
    { { 3, 5 }, { 0, 0 }, sp_bymonth, t_schema_type, sch_integer },
    { { 3, 5 }, { 0, 0 }, sp_bymonth, t_integer },
    { { 3, 5 }, { 0, 0 }, sp_bymonthday, t_schema_type, sch_integer },
    { { 3, 5 }, { 0, 0 }, sp_bymonthday, t_integer },
    { { 9, 0 }, { 0, 0 }, sp_bymonthweek, t_schema_type, sch_integer },
    { { 9, 0 }, { 0, 0 }, sp_bymonthweek, t_integer },
    { { 5, 0 }, { 0, 0 }, sp_callsign, t_schema_type, sch_text },
    { { 5, 0 }, { 0, 0 }, sp_callsign, t_text },
    { { 2, 0 }, { 0, 0 }, sp_calories, t_schema_type, sch_energy },
    { { 2, 0 }, { 0, 0 }, sp_candidate, t_schema_type, sch_person },
    { { 3, 5 }, { 0, 0 }, sp_caption, t_schema_type, sch_mediaobject },
    { { 2, 0 }, { 0, 0 }, sp_caption, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_caption, t_text },
    { { 2, 0 }, { 0, 0 }, sp_carbohydratecontent, t_schema_type, sch_mass },
    { { 2, 0 }, { 0, 0 }, sp_cargovolume, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_carrierrequirements, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_carrierrequirements, t_text },
    { { 3, 5 }, { 0, 0 }, sp_cashback, t_schema_type, sch_boolean },
    { { 3, 5 }, { 0, 0 }, sp_cashback, t_bool },
    { { 3, 5 }, { 0, 0 }, sp_cashback, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_cashback, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_cataloguenumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_cataloguenumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_category, t_schema_type, sch_physicalactivitycategory },
    { { 2, 0 }, { 0, 0 }, sp_category, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_category, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_category, t_text },
    { { 7, 4 }, { 0, 0 }, sp_category, t_schema_type, sch_url },
    { { 7, 4 }, { 0, 0 }, sp_category, t_url },
    { { 2, 0 }, { 6, 0 }, sp_cause, t_schema_type, sch_medicalcause },
    { { 2, 0 }, { 0, 0 }, sp_causeof, t_schema_type, sch_medicalentity },
    { { 3, 3 }, { 0, 0 }, sp_ccrecipient, t_schema_type, sch_contactpoint },
    { { 3, 3 }, { 0, 0 }, sp_ccrecipient, t_schema_type, sch_organisation },
    { { 3, 3 }, { 0, 0 }, sp_ccrecipient, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_character, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_characterattribute, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_charactername, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_charactername, t_text },
    { { 2, 0 }, { 0, 0 }, sp_cheatcode, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_checkintime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_checkintime, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_checkintime, t_schema_type, sch_time },
    { { 3, 8 }, { 0, 0 }, sp_checkintime, t_just_time },
    { { 2, 0 }, { 0, 0 }, sp_checkouttime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_checkouttime, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_checkouttime, t_schema_type, sch_time },
    { { 3, 8 }, { 0, 0 }, sp_checkouttime, t_just_time },
    { { 13, 0 }, { 0, 0 }, sp_chemicalcomposition, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_chemicalcomposition, t_text },
    { { 13, 0 }, { 0, 0 }, sp_chemicalrole, t_schema_type, sch_definedterm },
    { { 2, 0 }, { 0, 0 }, sp_childmaxage, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_childmaxage, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_childminage, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_childminage, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_children, t_schema_type, sch_person },
    { { 13, 0 }, { 0, 0 }, sp_childtaxon, t_schema_type, sch_taxon },
    { { 13, 0 }, { 0, 0 }, sp_childtaxon, t_schema_type, sch_url },
    { { 13, 0 }, { 0, 0 }, sp_childtaxon, t_url },
    { { 13, 0 }, { 0, 0 }, sp_childtaxon, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_childtaxon, t_text },
    { { 2, 0 }, { 0, 0 }, sp_cholesterolcontent, t_schema_type, sch_mass },
    { { 2, 0 }, { 0, 0 }, sp_circle, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_circle, t_text },
    { { 2, 0 }, { 0, 0 }, sp_citation, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_citation, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_citation, t_text },
    { { 13, 0 }, { 0, 0 }, sp_claiminterpreter, t_schema_type, sch_organisation },
    { { 13, 0 }, { 0, 0 }, sp_claiminterpreter, t_schema_type, sch_person },
    { { 3, 2 }, { 0, 0 }, sp_claimreviewed, t_schema_type, sch_text },
    { { 3, 2 }, { 0, 0 }, sp_claimreviewed, t_text },
    { { 2, 0 }, { 0, 0 }, sp_clinicalpharamcology, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_clinicalpharamcology, t_text },
    { { 2, 0 }, { 0, 0 }, sp_clipnumber, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_clipnumber, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_clipnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_clipnumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_closes, t_schema_type, sch_time },
    { { 2, 0 }, { 0, 0 }, sp_closes, t_just_time },
    { { 2, 0 }, { 0, 0 }, sp_coach, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_code, t_schema_type, sch_medicalcode },
    { { 2, 0 }, { 0, 0 }, sp_coderepository, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_coderepository, t_url },
    { { 2, 0 }, { 0, 0 }, sp_codesampletype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_codesampletype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_codevalue, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_codevalue, t_text },
    { { 2, 0 }, { 0, 0 }, sp_codingsystem, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_codingsystem, t_text },
    { { 2, 0 }, { 0, 0 }, sp_colleague, t_schema_type, sch_person },
    { { 3, 0 }, { 0, 0 }, sp_colleague, t_schema_type, sch_url },
    { { 3, 0 }, { 0, 0 }, sp_colleague, t_url },
    { { 3, 5 }, { 0, 0 }, sp_collectionsize, t_schema_type, sch_integer },
    { { 3, 5 }, { 0, 0 }, sp_collectionsize, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_colour, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_colour, t_text },
    { { 3, 5 }, { 0, 0 }, sp_colourist, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_comment, t_schema_type, sch_comment },
    { { 2, 0 }, { 0, 0 }, sp_commentcount, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_commentcount, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_commenttext, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_commenttext, t_text },
    { { 3, 1 }, { 0, 0 }, sp_commenttime, t_schema_type, sch_datetime },
    { { 3, 1 }, { 0, 0 }, sp_commenttime, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_commenttime, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_commenttime, t_just_date },
    { { 3, 5 }, { 0, 0 }, sp_competencyrequired, t_schema_type, sch_definedterm },
    { { 3, 5 }, { 0, 0 }, sp_competencyrequired, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_competencyrequired, t_text },
    { { 3, 5 }, { 0, 0 }, sp_competencyrequired, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_competencyrequired, t_url },
    { { 2, 0 }, { 0, 0 }, sp_competitor, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_competitor, t_schema_type, sch_sportsteam },
    { { 2, 0 }, { 0, 0 }, sp_composer, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_composer, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_comprisedof, t_schema_type, sch_anatomicalstructure },
    { { 2, 0 }, { 0, 0 }, sp_comprisedof, t_schema_type, sch_anatomicalsystem },
    { { 3, 8 }, { 0, 0 }, sp_conditionsofaccess, t_schema_type, sch_text },
    { { 3, 8 }, { 0, 0 }, sp_conditionsofaccess, t_text },
    { { 2, 0 }, { 0, 0 }, sp_confirmationnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_confirmationnumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_connectedto, t_schema_type, sch_anatomicalstructure },
    { { 3, 9 }, { 0, 0 }, sp_constrainingproperty, t_schema_type, sch_integer },
    { { 3, 9 }, { 0, 0 }, sp_constrainingproperty, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_contactoption, t_schema_type, sch_contactpointoption },
    { { 2, 0 }, { 0, 0 }, sp_contactpoint, t_schema_type, sch_contactpoint },
    { { 2, 0 }, { 0, 0 }, sp_contacttype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_contacttype, t_text },
    { { 3, 5 }, { 0, 0 }, sp_contactlesspayment },
    { { 2, 0 }, { 2, 1 }, sp_containedin, t_schema_type, sch_place },
    { { 2, 2 }, { 0, 0 }, sp_containedinplace, t_schema_type, sch_place },
    { { 2, 2 }, { 0, 0 }, sp_containsplace, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_containsseason, t_schema_type, sch_creativeworkseason },
    { { 2, 0 }, { 0, 0 }, sp_contentlocation, t_schema_type, sch_place },
    { { 3, 4 }, { 0, 0 }, sp_contentrating, t_schema_type, sch_rating },
    { { 2, 0 }, { 0, 0 }, sp_contentrating, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_contentrating, t_text },
    { { 3, 5 }, { 0, 0 }, sp_contentreferencetime, t_schema_type, sch_datetime },
    { { 3, 5 }, { 0, 0 }, sp_contentreferencetime, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_contentsize, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_contentsize, t_text },
    { { 2, 0 }, { 0, 0 }, sp_contenttype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_contenttype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_contenturl, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_contenturl, t_url },
    { { 2, 0 }, { 0, 0 }, sp_contraindication, t_schema_type, sch_medicalcontraindication },
    { { 3, 5 }, { 0, 0 }, sp_contraindication, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_contraindication, t_text },
    { { 2, 0 }, { 0, 0 }, sp_contributor, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_contributor, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_cooktime, t_schema_type, sch_duration },
    { { 2, 0 }, { 0, 0 }, sp_cooktime, t_duration },
    { { 2, 0 }, { 0, 0 }, sp_cookingmethod, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_cookingmethod, t_text },
    { { 2, 0 }, { 0, 0 }, sp_copyrightholder, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_copyrightholder, t_schema_type, sch_person },
    { { 11, 0 }, { 0, 0 }, sp_copyrightnotice, t_schema_type, sch_text },
    { { 11, 0 }, { 0, 0 }, sp_copyrightnotice, t_text },
    { { 2, 0 }, { 0, 0 }, sp_copyrightyear, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_copyrightyear, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_correction, t_schema_type, sch_correctioncomment },
    { { 3, 5 }, { 0, 0 }, sp_correction, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_correction, t_text },
    { { 3, 5 }, { 0, 0 }, sp_correction, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_correction, t_url },
    { { 3, 5 }, { 0, 0 }, sp_correctionspolicy, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_correctionspolicy, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_correctionspolicy, t_url },
    { { 2, 0 }, { 0, 0 }, sp_cost, t_schema_type, sch_drugcost },
    { { 2, 0 }, { 0, 0 }, sp_costcategory, t_schema_type, sch_drugcostcategory },
    { { 2, 0 }, { 0, 0 }, sp_costcurrency, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_costcurrency, t_text },
    { { 2, 0 }, { 0, 0 }, sp_costorigin, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_costorigin, t_text },
    { { 2, 0 }, { 0, 0 }, sp_costperunit, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_costperunit, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_costperunit, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_costperunit, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_costperunit, t_text },
    { { 2, 0 }, { 0, 0 }, sp_countriesnotsupported, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_countriesnotsupported, t_text },
    { { 2, 0 }, { 0, 0 }, sp_countriessupported, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_countriessupported, t_text },
    { { 13, 0 }, { 0, 0 }, sp_countryofassembly, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_countryofassembly, t_text },
    { { 13, 0 }, { 0, 0 }, sp_countryoflastprocessing, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_countryoflastprocessing, t_text },
    { { 2, 2 }, { 0, 0 }, sp_countryoforigin, t_schema_type, sch_country },
    { { 3, 4 }, { 0, 0 }, sp_coursecode, t_schema_type, sch_text },
    { { 3, 4 }, { 0, 0 }, sp_coursecode, t_text },
    { { 3, 4 }, { 0, 0 }, sp_coursemode, t_schema_type, sch_text },
    { { 3, 4 }, { 0, 0 }, sp_coursemode, t_text },
    { { 3, 4 }, { 0, 0 }, sp_coursemode, t_schema_type, sch_url },
    { { 3, 4 }, { 0, 0 }, sp_coursemode, t_url },
    { { 3, 4 }, { 0, 0 }, sp_coursepresequisites, t_schema_type, sch_alignmentobject },
    { { 3, 4 }, { 0, 0 }, sp_coursepresequisites, t_schema_type, sch_course },
    { { 3, 4 }, { 0, 0 }, sp_coursepresequisites, t_schema_type, sch_text },
    { { 3, 4 }, { 0, 0 }, sp_coursepresequisites, t_text },
    { { 3, 6 }, { 0, 0 }, sp_courseworkload, t_schema_type, sch_text },
    { { 3, 6 }, { 0, 0 }, sp_courseworkload, t_text },
    { { 2, 1 }, { 0, 0 }, sp_coverageendtime, t_schema_type, sch_datetime },
    { { 2, 1 }, { 0, 0 }, sp_coverageendtime, t_datetime },
    { { 2, 1 }, { 0, 0 }, sp_coveragestarttime, t_schema_type, sch_datetime },
    { { 2, 1 }, { 0, 0 }, sp_coveragestarttime, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_creativeworkstatus, t_schema_type, sch_definedterm },
    { { 3, 8 }, { 0, 0 }, sp_creativeworkstatus, t_schema_type, sch_text },
    { { 3, 8 }, { 0, 0 }, sp_creativeworkstatus, t_text },
    { { 2, 0 }, { 0, 0 }, sp_creator, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_creator, t_schema_type, sch_person },
    { { 3, 5 }, { 0, 0 }, sp_credentialcategory, t_schema_type, sch_definedterm },
    { { 3, 5 }, { 0, 0 }, sp_credentialcategory, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_credentialcategory, t_text },
    { { 3, 5 }, { 0, 0 }, sp_credentialcategory, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_credentialcategory, t_url },
    { { 2, 0 }, { 0, 0 }, sp_creditedto, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_creditedto, t_schema_type, sch_person },
    { { 11, 0 }, { 0, 0 }, sp_credittext, t_schema_type, sch_text },
    { { 11, 0 }, { 0, 0 }, sp_credittext, t_text },
    { { 3, 5 }, { 0, 0 }, sp_cssselector, t_schema_type, sch_cssselectortype },
    { { 2, 0 }, { 0, 0 }, sp_currenciesaccepted, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_currenciesaccepted, t_text },
    { { 2, 0 }, { 0, 0 }, sp_currency, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_currency, t_text },
    { { 3, 5 }, { 0, 0 }, sp_currentexchangerate, t_schema_type, sch_unitpricespecification },
    { { 2, 0 }, { 0, 0 }, sp_customer, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_customer, t_schema_type, sch_person },
    { { 13, 0 }, { 0, 0 }, sp_customerremorsereturnfees, t_schema_type, sch_returnfeesenumeration },
    { { 13, 0 }, { 0, 0 }, sp_customerremorsereturnlabelsource, t_schema_type, sch_returnlabelsourceenumeration },
    { { 13, 0 }, { 0, 0 }, sp_customerremorsereturnshippingfeesamount, t_schema_type, sch_monetaryamount },
    { { 8, 0 }, { 0, 0 }, sp_cutofftime, t_schema_type, sch_text },
    { { 8, 0 }, { 0, 0 }, sp_cutofftime, t_text },
    { { 7, 3 }, { 0, 0 }, sp_cvdcollectiondate, t_schema_type, sch_datetime },
    { { 7, 3 }, { 0, 0 }, sp_cvdcollectiondate, t_datetime_absolute },
    { { 7, 3 }, { 0, 0 }, sp_cvdcollectiondate, t_schema_type, sch_text },
    { { 7, 3 }, { 0, 0 }, sp_cvdcollectiondate, t_text },
    { { 7, 3 }, { 0, 0 }, sp_cvdfacilitycounty, t_schema_type, sch_text },
    { { 7, 3 }, { 0, 0 }, sp_cvdfacilitycounty, t_text },
    { { 7, 3 }, { 0, 0 }, sp_cvdfacilityid, t_schema_type, sch_text },
    { { 7, 3 }, { 0, 0 }, sp_cvdfacilityid, t_text },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumbeds, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumbeds, t_fixedpoint },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumbedsocc, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumbedsocc, t_fixedpoint },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumc19died, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumc19died, t_fixedpoint },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumc19hopats, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumc19hopats, t_fixedpoint },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumc19hosppats, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumc19hosppats, t_fixedpoint },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumc19mechventpats, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumc19mechventpats, t_fixedpoint },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumc19ofmechventpats, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumc19ofmechventpats, t_fixedpoint },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumc19overflowpats, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumc19overflowpats, t_fixedpoint },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumicubeds, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumicubeds, t_fixedpoint },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumicubedsocc, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumicubedsocc, t_fixedpoint },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumtotbeds, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumtotbeds, t_fixedpoint },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumvent, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumvent, t_fixedpoint },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumventuse, t_schema_type, sch_number },
    { { 7, 3 }, { 0, 0 }, sp_cvdnumventuse, t_fixedpoint },
    { { 2, 2 }, { 0, 0 }, sp_datafeedelement, t_schema_type, sch_datafeeditem },
    { { 2, 2 }, { 0, 0 }, sp_datafeedelement, t_schema_type, sch_thing },
    { { 2, 2 }, { 0, 0 }, sp_datafeedelement, t_schema_type, sch_text },
    { { 2, 2 }, { 0, 0 }, sp_datafeedelement, t_text },
    { { 2, 0 }, { 0, 0 }, sp_dataset, t_schema_type, sch_datadownload },
    { { 2, 0 }, { 3, 0 }, sp_datasettimeinterval, t_schema_type, sch_datacatalogue },
    { { 2, 0 }, { 0, 0 }, sp_datecreated, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_datecreated, t_datetime },
    { { 3, 0 }, { 0, 0 }, sp_datedeleted, t_schema_type, sch_datetime },
    { { 3, 0 }, { 0, 0 }, sp_datedeleted, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_dateissued, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_dateissued, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_dateissued, t_schema_type, sch_date },
    { { 3, 8 }, { 0, 0 }, sp_dateissued, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_datemodified, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_datemodified, t_datetime },
    { { 7, 3 }, { 0, 0 }, sp_dateposted, t_schema_type, sch_datetime },
    { { 7, 3 }, { 0, 0 }, sp_dateposted, t_datetime },
    { { 7, 3 }, { 0, 0 }, sp_dateposted, t_datetime_absolute },
    { { 2, 0 }, { 0, 0 }, sp_dateposted, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_dateposted, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_datepublished, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_datepublished, t_datetime },
    { { 3, 0 }, { 0, 0 }, sp_dateread, t_schema_type, sch_datetime },
    { { 3, 0 }, { 0, 0 }, sp_dateread, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_dateread, t_schema_type, sch_date },
    { { 3, 8 }, { 0, 0 }, sp_dateread, t_just_date },
    { { 3, 0 }, { 0, 0 }, sp_datereceived, t_schema_type, sch_datetime },
    { { 3, 0 }, { 0, 0 }, sp_datereceived, t_datetime },
    { { 3, 0 }, { 0, 0 }, sp_datesent, t_schema_type, sch_datetime },
    { { 3, 0 }, { 0, 0 }, sp_datesent, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_datevehiclefirstregistered, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_datevehiclefirstregistered, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_dateline, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_dateline, t_text },
    { { 2, 0 }, { 0, 0 }, sp_dayofweek, t_schema_type, sch_dayofweek },
    { { 2, 0 }, { 0, 0 }, sp_deathdate, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_deathdate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_deathplace, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_defaultvalue, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_defaultvalue, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_defaultvalue, t_text },
    { { 2, 0 }, { 0, 0 }, sp_deliveryaddress, t_schema_type, sch_postaladdress },
    { { 2, 0 }, { 0, 0 }, sp_deliveryleadtime, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_deliverymethod, t_schema_type, sch_deliverymethod },
    { { 2, 0 }, { 0, 0 }, sp_deliverystatus, t_schema_type, sch_deliveryevent },
    { { 8, 0 }, { 0, 0 }, sp_deliverytime, t_schema_type, sch_shippingdeliverytime },
    { { 2, 0 }, { 0, 0 }, sp_department, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_departureairport, t_schema_type, sch_airport },
    { { 9, 0 }, { 0, 0 }, sp_departureboatterminal, t_schema_type, sch_boatterminal },
    { { 2, 0 }, { 0, 0 }, sp_departurebusstop, t_schema_type, sch_busstation  },
    { { 2, 0 }, { 0, 0 }, sp_departurebusstop, t_schema_type, sch_busstop  },
    { { 2, 0 }, { 0, 0 }, sp_departuregate, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_departuregate, t_text },
    { { 2, 0 }, { 0, 0 }, sp_departureplatform, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_departureplatform, t_text },
    { { 2, 0 }, { 0, 0 }, sp_departurestation, t_schema_type, sch_trainstation },
    { { 2, 0 }, { 0, 0 }, sp_departureterminal, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_departureterminal, t_text },
    { { 2, 0 }, { 0, 0 }, sp_departuretime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_departuretime, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_departuretime, t_datetime_absolute },
    { { 2, 0 }, { 0, 0 }, sp_dependencies, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_dependencies, t_text },
    { { 2, 0 }, { 0, 0 }, sp_depth, t_schema_type, sch_distance },
    { { 2, 0 }, { 0, 0 }, sp_depth, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_description, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_description, t_text },
    { { 2, 0 }, { 0, 0 }, sp_diagnosis, t_schema_type, sch_medicalcondition },
    { { 2, 0 }, { 0, 0 }, sp_diagram, t_schema_type, sch_imageobject },
    { { 3, 5 }, { 0, 0 }, sp_diet, t_schema_type, sch_diet },
    { { 2, 0 }, { 0, 0 }, sp_dietfeatures, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_dietfeatures, t_text },
    { { 2, 0 }, { 0, 0 }, sp_differentialdiagnosis, t_schema_type, sch_ddxelement },
    { { 13, 0 }, { 0, 0 }, sp_directapply, t_schema_type, sch_boolean },
    { { 13, 0 }, { 0, 0 }, sp_directapply, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_director, t_schema_type, sch_person },
    { { 3, 0 }, { 0, 0 }, sp_disambiguatingdescription, t_schema_type, sch_text },
    { { 3, 0 }, { 0, 0 }, sp_disambiguatingdescription, t_text },
    { { 2, 0 }, { 0, 0 }, sp_discount, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_discount, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_discount, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_discount, t_text },
    { { 2, 0 }, { 0, 0 }, sp_discountcode, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_discountcode, t_text },
    { { 2, 0 }, { 0, 0 }, sp_discountcurrency, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_discountcurrency, t_text },
    { { 2, 0 }, { 0, 0 }, sp_discusses, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_discussionurl, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_discussionurl, t_url },
    { { 7, 0 }, { 0, 0 }, sp_diseasepreventioninfo, t_schema_type, sch_webcontent },
    { { 7, 0 }, { 0, 0 }, sp_diseasepreventioninfo, t_schema_type, sch_url },
    { { 7, 0 }, { 0, 0 }, sp_diseasepreventioninfo, t_url },
    { { 7, 0 }, { 0, 0 }, sp_diseasespreadstatistics, t_schema_type, sch_dataset },
    { { 7, 0 }, { 0, 0 }, sp_diseasespreadstatistics, t_schema_type, sch_observation },
    { { 7, 0 }, { 0, 0 }, sp_diseasespreadstatistics, t_schema_type, sch_webcontent },
    { { 7, 0 }, { 0, 0 }, sp_diseasespreadstatistics, t_schema_type, sch_url },
    { { 7, 0 }, { 0, 0 }, sp_diseasespreadstatistics, t_url },
    { { 2, 0 }, { 0, 0 }, sp_dissolutiondate, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_dissolutiondate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_distance, t_schema_type, sch_distance },
    { { 2, 0 }, { 0, 0 }, sp_distinguishingsign, t_schema_type, sch_medicalsignorsymptom },
    { { 2, 0 }, { 0, 0 }, sp_distribution, t_schema_type, sch_datadownload },
    { { 3, 5 }, { 0, 0 }, sp_diversitypolicy, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_diversitypolicy, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_diversitypolicy, t_url },
    { { 3, 5 }, { 0, 0 }, sp_diversitystaffingreport, t_schema_type, sch_article },
    { { 3, 5 }, { 0, 0 }, sp_diversitystaffingreport, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_diversitystaffingreport, t_url },
    { { 3, 5 }, { 0, 0 }, sp_documentation, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_documentation, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_documentation, t_url },
    { { 8, 0 }, { 0, 0 }, sp_doesnotship, t_schema_type, sch_boolean },
    { { 8, 0 }, { 0, 0 }, sp_doesnotship, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_domainincludes, t_schema_type, sch_class },
    { { 3, 5 }, { 0, 0 }, sp_domiciledmortgage, t_schema_type, sch_boolean },
    { { 3, 5 }, { 0, 0 }, sp_domiciledmortgage, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_doortime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_doortime, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_doortime, t_schema_type, sch_time },
    { { 3, 8 }, { 0, 0 }, sp_doortime, t_just_time },
    { { 2, 0 }, { 0, 0 }, sp_dosageform, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_dosageform, t_text },
    { { 2, 0 }, { 0, 0 }, sp_doseschedule, t_schema_type, sch_doseschedule },
    { { 2, 0 }, { 0, 0 }, sp_doseunit, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_doseunit, t_text },
    { { 2, 0 }, { 0, 0 }, sp_dosevalue, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_dosevalue, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_dosevalue, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_downpayment, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_downpayment, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_downpayment, t_schema_type, sch_monetaryamount },
    { { 2, 0 }, { 0, 0 }, sp_downloadurl, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_downloadurl, t_url },
    { { 2, 0 }, { 0, 0 }, sp_downvotecount, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_downvotecount, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_drainsto, t_schema_type, sch_vessel },
    { { 2, 0 }, { 0, 0 }, sp_drivewheelconfiguration, t_schema_type, sch_drivewheelconfigurationvalue },
    { { 2, 0 }, { 0, 0 }, sp_drivewheelconfiguration, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_drivewheelconfiguration, t_text },
    { { 2, 0 }, { 0, 0 }, sp_dropofflocation, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_dropofftime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_dropofftime, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_drug, t_schema_type, sch_drug },
    { { 2, 0 }, { 0, 0 }, sp_drugclass, t_schema_type, sch_drugclass },
    { { 2, 0 }, { 0, 0 }, sp_drugunit, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_drugunit, t_text },
    { { 2, 0 }, { 0, 0 }, sp_duns, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_duns, t_text },
    { { 2, 0 }, { 0, 0 }, sp_duplicatetherapy, t_schema_type, sch_medicaltherapy },
    { { 2, 0 }, { 0, 0 }, sp_duration, t_duration },
    { { 2, 0 }, { 0, 0 }, sp_durationofwarranty, t_schema_type, sch_quantitativevalue },
    { { 3, 3 }, { 0, 0 }, sp_duringmedia, t_schema_type, sch_mediaobject },
    { { 3, 4 }, { 0, 0 }, sp_duringmedia, t_schema_type, sch_url },
    { { 3, 4 }, { 0, 0 }, sp_duringmedia, t_url },
    { { 3, 5 }, { 0, 0 }, sp_earlyprepaymentpenalty, t_schema_type, sch_monetaryamount },
    { { 8, 0 }, { 0, 0 }, sp_editeidr, t_schema_type, sch_text },
    { { 8, 0 }, { 0, 0 }, sp_editeidr, t_text },
    { { 8, 0 }, { 0, 0 }, sp_editeidr, t_schema_type, sch_url },
    { { 8, 0 }, { 0, 0 }, sp_editeidr, t_url },
    { { 2, 0 }, { 0, 0 }, sp_editor, t_schema_type, sch_person },
    { { 3, 5 }, { 0, 0 }, sp_educationcredentialawarded, t_schema_type, sch_educationaloccupationalcredential },
    { { 3, 5 }, { 0, 0 }, sp_educationcredentialawarded, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_educationcredentialawarded, t_text },
    { { 3, 5 }, { 0, 0 }, sp_educationcredentialawarded, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_educationcredentialawarded, t_url },
    { { 3, 5 }, { 0, 0 }, sp_educationrequirements, t_schema_type, sch_educationaloccupationalcredential },
    { { 3, 6 }, { 0, 0 }, sp_educationrequirements, t_schema_type, sch_text },
    { { 3, 6 }, { 0, 0 }, sp_educationrequirements, t_text },
    { { 2, 0 }, { 0, 0 }, sp_educationalalignment, t_schema_type, sch_alignmentobject },
    { { 3, 9 }, { 0, 0 }, sp_educationalcredentialawarded, t_schema_type, sch_educationaloccupationalcredential },
    { { 3, 9 }, { 0, 0 }, sp_educationalcredentialawarded, t_schema_type, sch_text },
    { { 3, 9 }, { 0, 0 }, sp_educationalcredentialawarded, t_text },
    { { 3, 9 }, { 0, 0 }, sp_educationalcredentialawarded, t_schema_type, sch_url },
    { { 3, 9 }, { 0, 0 }, sp_educationalcredentialawarded, t_url },
    { { 2, 0 }, { 0, 0 }, sp_educationalframework, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_educationalframework, t_text },
    { { 3, 5 }, { 0, 0 }, sp_educationallevel, t_schema_type, sch_definedterm },
    { { 3, 5 }, { 0, 0 }, sp_educationallevel, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_educationallevel, t_text },
    { { 3, 5 }, { 0, 0 }, sp_educationallevel, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_educationallevel, t_url },
    { { 6, 0 }, { 0, 0 }, sp_educationalprogrammode, t_schema_type, sch_text },
    { { 6, 0 }, { 0, 0 }, sp_educationalprogrammode, t_text },
    { { 6, 0 }, { 0, 0 }, sp_educationalprogrammode, t_schema_type, sch_url },
    { { 6, 0 }, { 0, 0 }, sp_educationalprogrammode, t_url },
    { { 2, 0 }, { 0, 0 }, sp_educationalrole, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_educationalrole, t_text },
    { { 11, 0 }, { 0, 0 }, sp_educationaluse, t_schema_type, sch_definedterm },
    { { 2, 0 }, { 0, 0 }, sp_educationaluse, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_educationaluse, t_text },
    { { 10, 0 }, { 0, 0 }, sp_eduquestiontype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_elevation, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_elevation, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_elevation, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_elevation, t_text },
    { { 2, 0 }, { 0, 0 }, sp_eligibilecustomertype, t_schema_type, sch_businessentitytype },
    { { 8, 0 }, { 0, 0 }, sp_eligibilitytoworkrequirement, t_schema_type, sch_text },
    { { 8, 0 }, { 0, 0 }, sp_eligibilitytoworkrequirement, t_text },
    { { 2, 0 }, { 0, 0 }, sp_eligiblecustomertype, t_schema_type, sch_businessentitytype },
    { { 2, 0 }, { 0, 0 }, sp_eligibleduration, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_eligiblequantity, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_eligibleregion, t_schema_type, sch_geoshape },
    { { 2, 0 }, { 0, 0 }, sp_eligibleregion, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_eligibleregion, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_eligibleregion, t_text },
    { { 2, 0 }, { 0, 0 }, sp_eligibletransactionvolume, t_schema_type, sch_pricespecification },
    { { 2, 0 }, { 0, 0 }, sp_email, t_email },
    { { 13, 0 }, { 0, 0 }, sp_embeddedtextcaption, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_embeddedtextcaption, t_text },
    { { 2, 0 }, { 0, 0 }, sp_embedurl, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_embedurl, t_url },
    { { 3, 5 }, { 0, 0 }, sp_emmissionsco2, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_emmissionsco2, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_employee, t_schema_type, sch_person },
    { { 6, 0 }, { 0, 0 }, sp_employeroverview, t_schema_type, sch_text },
    { { 6, 0 }, { 0, 0 }, sp_employeroverview, t_text },
    { { 2, 0 }, { 0, 0 }, sp_employmenttype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_employmenttype, t_text },
    { { 4, 0 }, { 0, 0 }, sp_employmentunit, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_encodescreativework, t_schema_type, sch_creativework },
    { { 13, 0 }, { 0, 0 }, sp_encodesbiochementity, t_schema_type, sch_biochementity },
    { { 2, 0 }, { 0, 0 }, sp_encoding, t_schema_type, sch_mediaobject },
    { { 2, 0 }, { 0, 0 }, sp_encodingformat, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_encodingformat, t_text },
    { { 3, 4 }, { 0, 0 }, sp_encodingformat, t_schema_type, sch_url },
    { { 3, 4 }, { 0, 0 }, sp_encodingformat, t_url },
    { { 2, 0 }, { 0, 0 }, sp_encodingtype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_encodingtype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_enddate, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_enddate, t_datetime },
    { { 6, 0 }, { 0, 0 }, sp_enddate, t_schema_type, sch_date },
    { { 6, 0 }, { 0, 0 }, sp_enddate, t_just_date },
    { { 11, 0 }, { 0, 0 }, sp_endoffset, t_schema_type, sch_hypertocentry },
    { { 3, 5 }, { 0, 0 }, sp_endoffset, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_endoffset, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_endtime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_endtime, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_endtime, t_schema_type, sch_time },
    { { 3, 8 }, { 0, 0 }, sp_endtime, t_just_time },
    { { 2, 0 }, { 0, 0 }, sp_endorsee, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_endorsee, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_endorsers, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_endorsers, t_schema_type, sch_person },
    { { 10, 0 }, { 0, 0 }, sp_energyefficiencyscalemax, t_schema_type, sch_euenergyefficiencyenumeration },
    { { 10, 0 }, { 0, 0 }, sp_energyefficiencyscalemin, t_schema_type, sch_euenergyefficiencyenumeration },
    { { 3, 5 }, { 0, 0 }, sp_enginedisplacement, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_enginepower, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_enginetype, t_schema_type, sch_qualitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_enginetype, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_enginetype, t_text },
    { { 3, 5 }, { 0, 0 }, sp_enginetype, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_enginetype, t_url },
    { { 2, 0 }, { 0, 0 }, sp_entertainmentbusiness, t_schema_type, sch_entertainmentbusiness },
    { { 2, 0 }, { 0, 0 }, sp_epidemiology, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_epidemiology, t_text },
    { { 2, 0 }, { 0, 0 }, sp_episode, t_schema_type, sch_episode },
    { { 2, 0 }, { 0, 0 }, sp_episodenumber, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_episodenumber, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_episodenumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_episodenumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_equal, t_schema_type, sch_qualitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_error, t_schema_type, sch_thing },
    { { 3, 3 }, { 0, 0 }, sp_estimatedcost, t_schema_type, sch_monetaryamount },
    { { 3, 3 }, { 0, 0 }, sp_estimatedcost, t_schema_type, sch_text },
    { { 3, 3 }, { 0, 0 }, sp_estimatedcost, t_text },
    { { 2, 0 }, { 0, 0 }, sp_estimatedflightduration, t_schema_type, sch_duration },
    { { 2, 0 }, { 0, 0 }, sp_estimatedflightduration, t_duration },
    { { 2, 0 }, { 0, 0 }, sp_estimatedflightduration, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_estimatedflightduration, t_text },
    { { 3, 7 }, { 0, 0 }, sp_estimatedsalary, t_schema_type, sch_number },
    { { 3, 7 }, { 0, 0 }, sp_estimatedsalary, t_fixedpoint },
    { { 3, 7 }, { 0, 0 }, sp_estimatedsalary, t_schema_type, sch_monetaryamount },
    { { 3, 5 }, { 0, 0 }, sp_estimatedsalary, t_schema_type, sch_monetaryamountdistribution },
    { { 2, 0 }, { 0, 0 }, sp_estimatesriskof, t_schema_type, sch_medicalentity },
    { { 3, 5 }, { 0, 0 }, sp_ethicspolicy, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_ethicspolicy, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_ethicspolicy, t_url },
    { { 2, 0 }, { 0, 0 }, sp_event, t_schema_type, sch_event },
    { { 7, 0 }, { 0, 0 }, sp_eventattendancemode, t_schema_type, sch_eventattendancemodeenumeration },
    { { 2, 0 }, { 0, 0 }, sp_eventschedule, t_schema_type, sch_schedule },
    { { 2, 0 }, { 0, 0 }, sp_eventstatus, t_schema_type, sch_eventstatustype },
    { { 2, 0 }, { 0, 0 }, sp_evidencelevel, t_schema_type, sch_medicalevidencelevel },
    { { 2, 0 }, { 0, 0 }, sp_evidenceorigin, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_evidenceorigin, t_text },
    { { 2, 0 }, { 0, 0 }, sp_exampleofwork, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_exceptdate, t_schema_type, sch_datetime },
    { { 3, 5 }, { 0, 0 }, sp_exceptdate, t_datetime },
    { { 3, 5 }, { 0, 0 }, sp_exceptdate, t_schema_type, sch_date },
    { { 3, 5 }, { 0, 0 }, sp_exceptdate, t_just_date },
    { { 3, 5 }, { 0, 0 }, sp_exchangeratespread, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_exchangeratespread, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_exchangeratespread, t_schema_type, sch_monetaryamount },
    { { 2, 0 }, { 0, 0 }, sp_executablelibraryname, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_executablelibraryname, t_text },
    { { 2, 0 }, { 0, 0 }, sp_exercisecourse, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_exerciseplan, t_schema_type, sch_exerciseplan },
    { { 2, 0 }, { 0, 0 }, sp_exerciserelateddiet, t_schema_type, sch_diet },
    { { 2, 0 }, { 0, 0 }, sp_exercisetype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_exercisetype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_exifdata, t_schema_type, sch_propertyvalue },
    { { 2, 0 }, { 0, 0 }, sp_exifdata, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_exifdata, t_text },
    { { 2, 0 }, { 0, 0 }, sp_expectedarrivalfrom, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_expectedarrivalfrom, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_expectedarrivalfrom, t_schema_type, sch_date },
    { { 3, 8 }, { 0, 0 }, sp_expectedarrivalfrom, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_expectedarrivaluntil, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_expectedarrivaluntil, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_expectedarrivaluntil, t_schema_type, sch_date },
    { { 3, 8 }, { 0, 0 }, sp_expectedarrivaluntil, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_expectedprognosis, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_expectedprognosis, t_text },
    { { 2, 0 }, { 0, 0 }, sp_expectsacceptanceof, t_schema_type, sch_offer },
    { { 12, 0 }, { 0, 0 }, sp_experiencerequirements, t_schema_type, sch_occupationalexperiencerequirements },
    { { 2, 0 }, { 0, 0 }, sp_experiencerequirements, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_experiencerequirements, t_text },
    { { 12, 0 }, { 0, 0 }, sp_experienceinplaceofeducation, t_schema_type, sch_boolean },
    { { 12, 0 }, { 0, 0 }, sp_experienceinplaceofeducation, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_expires, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_expires, t_datetime },
    { { 13, 0 }, { 0, 0 }, sp_expressedin, t_schema_type, sch_anatomicalstructure },
    { { 13, 0 }, { 0, 0 }, sp_expressedin, t_schema_type, sch_anatomicalsystem },
    { { 13, 0 }, { 0, 0 }, sp_expressedin, t_schema_type, sch_biochementity },
    { { 13, 0 }, { 0, 0 }, sp_expressedin, t_schema_type, sch_definedterm },
    { { 2, 0 }, { 0, 0 }, sp_exportconsiderations, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_exportconsiderations, t_text },
    { { 2, 0 }, { 0, 0 }, sp_familyname, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_familyname, t_text },
    { { 2, 0 }, { 0, 0 }, sp_fatcontent, t_schema_type, sch_mass },
    { { 2, 0 }, { 0, 0 }, sp_faxnumber, t_tel },
    { { 2, 0 }, { 0, 0 }, sp_featurelist, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_featurelist, t_text },
    { { 2, 0 }, { 0, 0 }, sp_featurelist, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_featurelist, t_url },
    { { 3, 0 }, { 0, 0 }, sp_feesandcommissionsspecification, t_text },
    { { 3, 0 }, { 0, 0 }, sp_feesandcommissionsspecification, t_url },
    { { 2, 0 }, { 0, 0 }, sp_fibrecontent, t_schema_type, sch_mass },
    { { 2, 0 }, { 3, 3 }, sp_fileformat, t_schema_type, sch_text },
    { { 2, 0 }, { 3, 3 }, sp_fileformat, t_text },
    { { 2, 0 }, { 0, 0 }, sp_filesize, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_filesize, t_text },
    { { 6, 0 }, { 0, 0 }, sp_financialaideligible, t_schema_type, sch_definedterm },
    { { 6, 0 }, { 0, 0 }, sp_financialaideligible, t_schema_type, sch_text },
    { { 6, 0 }, { 0, 0 }, sp_financialaideligible, t_text },
    { { 3, 5 }, { 0, 0 }, sp_firstappearance, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_firstperformance, t_schema_type, sch_event },
    { { 2, 0 }, { 0, 0 }, sp_flightdistance, t_schema_type, sch_distance },
    { { 2, 0 }, { 0, 0 }, sp_flightdistance, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_flightdistance, t_text },
    { { 2, 0 }, { 0, 0 }, sp_flightnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_flightnumber, t_text },
    { { 5, 0 }, { 0, 0 }, sp_floorlevel, t_schema_type, sch_text },
    { { 5, 0 }, { 0, 0 }, sp_floorlevel, t_text },
    { { 3, 5 }, { 0, 0 }, sp_floorlimit, t_schema_type, sch_monetaryamount },
    { { 7, 0 }, { 0, 0 }, sp_floorsize, t_schema_type, sch_qualitativevalue },
    { { 3, 1 }, { 0, 0 }, sp_floorsize, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_followee, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_followee, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_follows, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_followup, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_followup, t_text },
    { { 2, 0 }, { 0, 0 }, sp_foodestablishment, t_schema_type, sch_foodestablishment },
    { { 2, 0 }, { 0, 0 }, sp_foodestablishment, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_foodevent, t_schema_type, sch_foodevent },
    { { 2, 0 }, { 0, 0 }, sp_foodwarning, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_foodwarning, t_text },
    { { 2, 0 }, { 0, 0 }, sp_founder, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_foundingdate, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_foundingdate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_foundinglocation, t_schema_type, sch_place },
    { { 8, 0 }, { 0, 0 }, sp_freeshippingthreshold, t_schema_type, sch_deliverychargespecification },
    { { 8, 0 }, { 0, 0 }, sp_freeshippingthreshold, t_schema_type, sch_monetaryamount },
    { { 2, 0 }, { 0, 0 }, sp_frequency, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_frequency, t_text },
    { { 2, 0 }, { 0, 0 }, sp_fromlocation, t_schema_type, sch_place },
    { { 3, 5 }, { 0, 0 }, sp_fuelcapacity, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_fuelconsumption, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_fuelefficiency, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_fueltype, t_schema_type, sch_qualitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_fueltype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_fueltype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_fueltype, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_fueltype, t_url },
    { { 3, 5 }, { 0, 0 }, sp_funcionalclass, t_schema_type, sch_medicalentity },
    { { 2, 0 }, { 0, 0 }, sp_funcionalclass, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_funcionalclass, t_text },
    { { 2, 0 }, { 6, 0 }, sp_function, t_schema_type, sch_text },
    { { 2, 0 }, { 6, 0 }, sp_function, t_text },
    { { 3, 5 }, { 0, 0 }, sp_fundeditem, t_schema_type, sch_thing },
    { { 3, 1 }, { 0, 0 }, sp_funder, t_schema_type, sch_organisation },
    { { 3, 1 }, { 0, 0 }, sp_funder, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_game, t_schema_type, sch_videogame },
    { { 2, 0 }, { 0, 0 }, sp_gameitem, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_gamelocation, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_gamelocation, t_schema_type, sch_postaladdress },
    { { 2, 0 }, { 0, 0 }, sp_gamelocation, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_gamelocation, t_url },
    { { 2, 0 }, { 0, 0 }, sp_gameplatform, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_gameplatform, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_gameplatform, t_text },
    { { 2, 0 }, { 0, 0 }, sp_gameplatform, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_gameplatform, t_url },
    { { 2, 0 }, { 0, 0 }, sp_gameserver, t_schema_type, sch_gameserver },
    { { 2, 0 }, { 0, 0 }, sp_gametip, t_schema_type, sch_creativework },
    { { 3, 0 }, { 0, 0 }, sp_gender, t_schema_type, sch_gendertype },
    { { 2, 0 }, { 0, 0 }, sp_gender, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_gender, t_text },
    { { 2, 0 }, { 0, 0 }, sp_genre, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_genre, t_text },
    { { 2, 0 }, { 0, 0 }, sp_genre, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_genre, t_url },
    { { 2, 0 }, { 0, 0 }, sp_geo, t_schema_type, sch_geocoordinates },
    { { 2, 0 }, { 0, 0 }, sp_geo, t_schema_type, sch_geoshape },
    { { 3, 5 }, { 0, 0 }, sp_geocontains, t_schema_type, sch_geospatialgeometry },
    { { 3, 5 }, { 0, 0 }, sp_geocontains, t_schema_type, sch_place },
    { { 3, 5 }, { 0, 0 }, sp_geocoveredby, t_schema_type, sch_geospatialgeometry },
    { { 3, 5 }, { 0, 0 }, sp_geocoveredby, t_schema_type, sch_place },
    { { 3, 5 }, { 0, 0 }, sp_geocovers, t_schema_type, sch_geospatialgeometry },
    { { 3, 5 }, { 0, 0 }, sp_geocovers, t_schema_type, sch_place },
    { { 3, 5 }, { 0, 0 }, sp_geocrosses, t_schema_type, sch_geospatialgeometry },
    { { 3, 5 }, { 0, 0 }, sp_geocrosses, t_schema_type, sch_place },
    { { 3, 5 }, { 0, 0 }, sp_geodisjoint, t_schema_type, sch_geospatialgeometry },
    { { 3, 5 }, { 0, 0 }, sp_geodisjoint, t_schema_type, sch_place },
    { { 3, 5 }, { 0, 0 }, sp_geoequals, t_schema_type, sch_geospatialgeometry },
    { { 3, 5 }, { 0, 0 }, sp_geoequals, t_schema_type, sch_place },
    { { 3, 5 }, { 0, 0 }, sp_geointersects, t_schema_type, sch_geospatialgeometry },
    { { 3, 5 }, { 0, 0 }, sp_geointersects, t_schema_type, sch_place },
    { { 2, 2 }, { 0, 0 }, sp_geomidpoint, t_schema_type, sch_geocoordinates },
    { { 3, 5 }, { 0, 0 }, sp_geooverlaps, t_schema_type, sch_geospatialgeometry },
    { { 3, 5 }, { 0, 0 }, sp_geooverlaps, t_schema_type, sch_place },
    { { 2, 2 }, { 0, 0 }, sp_georadius, t_schema_type, sch_number },
    { { 2, 2 }, { 0, 0 }, sp_georadius, t_fixedpoint },
    { { 2, 2 }, { 0, 0 }, sp_georadius, t_schema_type, sch_distance },
    { { 2, 2 }, { 0, 0 }, sp_georadius, t_schema_type, sch_text },
    { { 2, 2 }, { 0, 0 }, sp_georadius, t_text },
    { { 3, 5 }, { 0, 0 }, sp_geotouches, t_schema_type, sch_geospatialgeometry },
    { { 3, 5 }, { 0, 0 }, sp_geotouches, t_schema_type, sch_place },
    { { 3, 5 }, { 0, 0 }, sp_geowithin, t_schema_type, sch_geospatialgeometry },
    { { 3, 5 }, { 0, 0 }, sp_geowithin, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_geographicarea, t_schema_type, sch_administrativearea },
    { { 7, 0 }, { 0, 0 }, sp_gettingtestedinfo, t_schema_type, sch_webcontent },
    { { 7, 0 }, { 0, 0 }, sp_gettingtestedinfo, t_schema_type, sch_url },
    { { 7, 0 }, { 0, 0 }, sp_gettingtestedinfo, t_url },
    { { 2, 0 }, { 0, 0 }, sp_givenname, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_givenname, t_text },
    { { 2, 0 }, { 0, 0 }, sp_globallocationnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_globallocationnumber, t_text },
    { { 7, 4 }, { 0, 0 }, sp_governmentbenefitsinfo, t_schema_type, sch_governmentservice },
    { { 3, 5 }, { 0, 0 }, sp_graceperiod, t_schema_type, sch_duration },
    { { 3, 5 }, { 0, 0 }, sp_graceperiod, t_duration },
    { { 3, 0 }, { 0, 0 }, sp_grantee, t_schema_type, sch_audience },
    { { 3, 0 }, { 0, 0 }, sp_grantee, t_schema_type, sch_contactpoint },
    { { 3, 0 }, { 0, 0 }, sp_grantee, t_schema_type, sch_organisation },
    { { 3, 0 }, { 0, 0 }, sp_grantee, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_greater, t_schema_type, sch_qualitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_greaterorequal, t_schema_type, sch_qualitativevalue },
    { { 3, 8 }, { 0, 0 }, sp_gtin, t_schema_type, sch_text },
    { { 3, 8 }, { 0, 0 }, sp_gtin, t_text },
    { { 2, 0 }, { 0, 0 }, sp_gtin12, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_gtin12, t_text },
    { { 2, 0 }, { 0, 0 }, sp_gtin13, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_gtin13, t_text },
    { { 2, 0 }, { 0, 0 }, sp_gtin14, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_gtin14, t_text },
    { { 2, 0 }, { 0, 0 }, sp_gtin8, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_gtin8, t_text },
    { { 2, 0 }, { 0, 0 }, sp_guideline, t_schema_type, sch_medicalguideline },
    { { 2, 0 }, { 0, 0 }, sp_guidelinedate, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_guidelinedate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_guidelinesubject, t_schema_type, sch_medicalentity },
    { { 8, 0 }, { 0, 0 }, sp_handlingtime, t_schema_type, sch_quantitativevalue },
    { { 13, 0 }, { 0, 0 }, sp_hasbiochementitypart, t_schema_type, sch_biochementity },
    { { 13, 0 }, { 0, 0 }, sp_hasbiopolymersequence, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_hasbiopolymersequence, t_text },
    { { 3, 5 }, { 0, 0 }, sp_hasbroadcastchannel, t_schema_type, sch_broadcastchannel },
    { { 3, 5 }, { 0, 0 }, sp_hascategorycode, t_schema_type, sch_categorycode },
    { { 3, 4 }, { 0, 0 }, sp_hascourseinstance, t_schema_type, sch_courseinstance },
    { { 10, 0 }, { 0, 0 }, sp_hascourse, t_schema_type, sch_course },
    { { 3, 9 }, { 0, 0 }, sp_hascredential, t_schema_type, sch_educationaloccupationalcredential },
    { { 3, 5 }, { 0, 0 }, sp_hasdefinedterm, t_schema_type, sch_definedterm },
    { { 2, 0 }, { 0, 0 }, sp_hasdeliverymethod, t_schema_type, sch_deliverymethod },
    { { 3, 0 }, { 0, 0 }, sp_hasdigitaldocumentpermission, t_schema_type, sch_digitaldocumentpermission },
    { { 7, 0 }, { 0, 0 }, sp_hasdrivethroughservice, t_schema_type, sch_boolean },
    { { 7, 0 }, { 0, 0 }, sp_hasdrivethroughservice, t_bool },
    { { 10, 0 }, { 0, 0 }, sp_hasenergyconsumptiondetails, t_schema_type, sch_energyconsumptiondetails },
    { { 10, 0 }, { 0, 0 }, sp_hasenergyefficiencycategory, t_schema_type, sch_energyefficiencyenumeration },
    { { 5, 0 }, { 0, 0 }, sp_hashealthaspect, t_schema_type, sch_healthaspectenumeration },
    { { 2, 0 }, { 0, 0 }, sp_hasmap, t_schema_type, sch_map },
    { { 2, 0 }, { 0, 0 }, sp_hasmap, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_hasmap, t_url },
    { { 12, 0 }, { 0, 0 }, sp_hasmeasurement, t_schema_type, sch_quantitativevalue },
    { { 3, 2 }, { 0, 0 }, sp_hasmenu, t_schema_type, sch_menu },
    { { 3, 2 }, { 0, 0 }, sp_hasmenu, t_url },
    { { 3, 2 }, { 0, 0 }, sp_hasmenu, t_text },
    { { 3, 3 }, { 0, 0 }, sp_hasmenuitem, t_schema_type, sch_menuitem },
    { { 3, 3 }, { 0, 0 }, sp_hasmenusection, t_schema_type, sch_menusection },
    { { 6, 0 }, { 0, 0 }, sp_hasmerchantreturnpolicy, t_schema_type, sch_merchantreturnpolicy },
    { { 13, 0 }, { 0, 0 }, sp_hasmolecularfunction, t_schema_type, sch_definedterm },
    { { 13, 0 }, { 0, 0 }, sp_hasmolecularfunction, t_schema_type, sch_propertyvalue },
    { { 13, 0 }, { 0, 0 }, sp_hasmolecularfunction, t_schema_type, sch_url },
    { { 13, 0 }, { 0, 0 }, sp_hasmolecularfunction, t_url },
    { { 3, 5 }, { 0, 0 }, sp_hasoccupation, t_schema_type, sch_occupation },
    { { 2, 2 }, { 0, 0 }, sp_hasoffercatalogue, t_schema_type, sch_offercatalogue },
    { { 2, 0 }, { 0, 0 }, sp_haspos, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_haspart, t_schema_type, sch_creativework },
    { { 3, 8 }, { 0, 0 }, sp_hasproductreturnpolicy, t_schema_type, sch_productreturnpolicy },
    { { 13, 0 }, { 0, 0 }, sp_hasrepresentation, t_schema_type, sch_propertyvalue },
    { { 13, 0 }, { 0, 0 }, sp_hasrepresentation, t_schema_type, sch_url },
    { { 13, 0 }, { 0, 0 }, sp_hasrepresentation, t_url },
    { { 9, 0 }, { 0, 0 }, sp_hasvariant, t_schema_type, sch_product },
    { { 2, 0 }, { 0, 0 }, sp_headline, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_headline, t_text },
    { { 2, 0 }, { 0, 0 }, sp_healthcondition, t_schema_type, sch_medicalcondition },
    { { 3, 5 }, { 0, 0 }, sp_healthplancoinsuranceoption, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplancoinsuranceoption, t_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplancoinsurancerate, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_healthplancoinsurancerate, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_healthplancopay, t_schema_type, sch_pricespecification },
    { { 3, 5 }, { 0, 0 }, sp_healthplancopayoption, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplancopayoption, t_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplancostsharing, t_schema_type, sch_boolean },
    { { 3, 5 }, { 0, 0 }, sp_healthplancostsharing, t_bool },
    { { 3, 5 }, { 0, 0 }, sp_healthplandrugoption, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplandrugoption, t_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplandrugtier, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplandrugtier, t_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplanid, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplanid, t_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplanmarketingurl, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_healthplanmarketingurl, t_url },
    { { 3, 5 }, { 0, 0 }, sp_healthplannetworkid, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplannetworkid, t_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplannetworktier, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplannetworktier, t_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplanpharmacycategory, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_healthplanpharmacycategory, t_text },
    { { 7, 3 }, { 0, 0 }, sp_healthcarereportingdata, t_schema_type, sch_cdcpmdrecord },
    { { 7, 3 }, { 0, 0 }, sp_healthcarereportingdata, t_schema_type, sch_dataset },
    { { 2, 0 }, { 0, 0 }, sp_height, t_schema_type, sch_distance },
    { { 2, 0 }, { 0, 0 }, sp_height, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_highprice, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_highprice, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_hiringorganization, t_schema_type, sch_organisation },
    { { 3, 5 }, { 0, 0 }, sp_holdingarchive, t_schema_type, sch_archiveorganisation },
    { { 2, 0 }, { 0, 0 }, sp_homelocation, t_schema_type, sch_contactpoint },
    { { 2, 0 }, { 0, 0 }, sp_homelocation, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_hometeam, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_hometeam, t_schema_type, sch_sportsteam },
    { { 2, 0 }, { 0, 0 }, sp_honourificprefix, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_honourificprefix, t_text },
    { { 2, 0 }, { 0, 0 }, sp_honourificsuffix, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_honourificsuffix, t_text },
    { { 2, 0 }, { 0, 0 }, sp_hospitalaffiliation, t_schema_type, sch_hospital },
    { { 2, 0 }, { 0, 0 }, sp_hostingorganisation, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_hoursavailable, t_schema_type, sch_openinghoursspecification },
    { { 2, 0 }, { 0, 0 }, sp_howperformed, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_howperformed, t_text },
    { { 2, 0 }, { 0, 0 }, sp_httpmethod, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_httpmethod, t_text },
    { { 2, 0 }, { 0, 0 }, sp_iatacode, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_iatacode, t_text },
    { { 3, 2 }, { 0, 0 }, sp_identifier, t_schema_type, sch_propertyvalue },
    { { 3, 2 }, { 0, 0 }, sp_identifier, t_schema_type, sch_text },
    { { 3, 2 }, { 0, 0 }, sp_identifier, t_text },
    { { 3, 2 }, { 0, 0 }, sp_identifier, t_schema_type, sch_url },
    { { 3, 2 }, { 0, 0 }, sp_identifier, t_url },
    { { 2, 0 }, { 0, 0 }, sp_identifyingexam, t_schema_type, sch_physicalexam },
    { { 2, 0 }, { 0, 0 }, sp_identifyingtest, t_schema_type, sch_medicaltest },
    { { 2, 0 }, { 0, 0 }, sp_illustrator, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_image, t_schema_type, sch_imageobject },
    { { 2, 0 }, { 0, 0 }, sp_image, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_image, t_url },
    { { 2, 0 }, { 0, 0 }, sp_imagingtechnique, t_schema_type, sch_medicalimagingtechnique },
    { { 2, 0 }, { 0, 0 }, sp_inalbum, t_schema_type, sch_musicalbum },
    { { 2, 0 }, { 0, 0 }, sp_inbroadcastlineup, t_schema_type, sch_cableorsatelliteservice },
    { { 3, 5 }, { 0, 0 }, sp_incodeset, t_schema_type, sch_categorycodeset },
    { { 3, 5 }, { 0, 0 }, sp_incodeset, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_incodeset, t_url },
    { { 3, 5 }, { 0, 0 }, sp_indefinedtermset, t_schema_type, sch_definedtermset },
    { { 3, 5 }, { 0, 0 }, sp_indefinedtermset, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_indefinedtermset, t_url },
    { { 2, 0 }, { 0, 0 }, sp_inlanguage, t_schema_type, sch_language },
    { { 2, 0 }, { 0, 0 }, sp_inlanguage, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_inlanguage, t_text },
    { { 2, 0 }, { 0, 0 }, sp_inplaylist, t_schema_type, sch_musicplaylist },
    { { 9, 0 }, { 0, 0 }, sp_inproductgroupwithid },
    { { 6, 0 }, { 0, 0 }, sp_instorereturnsoffered, t_schema_type, sch_boolean },
    { { 6, 0 }, { 0, 0 }, sp_instorereturnsoffered, t_bool },
    { { 3, 5 }, { 0, 0 }, sp_insupportof, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_insupportof, t_text },
    { { 2, 0 }, { 0, 0 }, sp_incentivecompensation, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_incentivecompensation, t_text },
    { { 13, 0 }, { 0, 0 }, sp_inchi, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_inchi, t_text },
    { { 13, 0 }, { 0, 0 }, sp_inchikey, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_inchikey, t_text },
    { { 2, 0 }, { 0, 0 }, sp_includedcomposition, t_schema_type, sch_musiccomposition },
    { { 2, 0 }, { 0, 0 }, sp_includeddatacatalog, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_includeddatacatalog, t_datetime },
    { { 3, 5 }, { 0, 0 }, sp_includedinhealthinsuranceplan, t_schema_type, sch_dystopianhealthinsuranceplan },
    { { 2, 0 }, { 0, 0 }, sp_includedriskfactor, t_schema_type, sch_medicalriskfactor },
    { { 3, 5 }, { 0, 0 }, sp_includesattraction, t_schema_type, sch_touristattraction },
    { { 3, 5 }, { 0, 0 }, sp_includeshealthplanformulary, t_schema_type, sch_dystopianhealthplanformulary },
    { { 3, 5 }, { 0, 0 }, sp_includeshealthplannetwork, t_schema_type, sch_dystopianhealthplannetwork },
    { { 2, 0 }, { 0, 0 }, sp_includesobject, t_schema_type, sch_typeandquantitynode },
    { { 2, 0 }, { 0, 0 }, sp_increasesriskof, t_schema_type, sch_medicalentity },
    { { 2, 0 }, { 6, 0 }, sp_indication, t_schema_type, sch_medicalindication },
    { { 6, 0 }, { 0, 0 }, sp_industry, t_schema_type, sch_definedterm },
    { { 2, 0 }, { 0, 0 }, sp_industry, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_industry, t_text },
    { { 2, 0 }, { 0, 0 }, sp_ineligibleregion, t_schema_type, sch_geoshape },
    { { 2, 0 }, { 0, 0 }, sp_ineligibleregion, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_ineligibleregion, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_ineligibleregion, t_text },
    { { 2, 0 }, { 0, 0 }, sp_infectiousagent, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_infectiousagent, t_text },
    { { 2, 0 }, { 0, 0 }, sp_infectiousagentclass, t_schema_type, sch_infectiousagentclass },
    { { 3, 5 }, { 0, 0 }, sp_inker, t_schema_type, sch_person },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_insertion, t_schema_type, sch_anatomicalstructure },
    { { 2, 0 }, { 0, 0 }, sp_installurl, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_installurl, t_url },
    { { 3, 4 }, { 0, 0 }, sp_instructor, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_instrument, t_schema_type, sch_thing },
    { { 3, 5 }, { 0, 0 }, sp_intensity, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_intensity, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_intensity, t_text },
    { { 2, 0 }, { 0, 0 }, sp_interactingdrug, t_schema_type, sch_drug },
    { { 2, 0 }, { 0, 0 }, sp_interactionservice, t_schema_type, sch_softwareapplication },
    { { 2, 0 }, { 0, 0 }, sp_interactionservice, t_schema_type, sch_website },
    { { 2, 1 }, { 0, 0 }, sp_interactionstatistic, t_schema_type, sch_interactioncounter },
    { { 2, 0 }, { 0, 0 }, sp_interactiontype, t_schema_type, sch_action, SP_VALUE_TYPENAME },
    { { 2, 0 }, { 0, 0 }, sp_interactivitytype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_interactivitytype, t_text },
    { { 3, 0 }, { 0, 0 }, sp_interestrate, t_schema_type, sch_number },
    { { 3, 0 }, { 0, 0 }, sp_interestrate, t_fixedpoint },
    { { 3, 0 }, { 0, 0 }, sp_interestrate, t_schema_type, sch_quantitativevalue },
    { { 13, 0 }, { 0, 0 }, sp_interpretedasclaim, t_schema_type, sch_claim },
    { { 2, 0 }, { 0, 0 }, sp_inventorylevel, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_inverseof, t_schema_type, sch_property },
    { { 3, 5 }, { 0, 0 }, sp_isacceptingnewpatients, t_schema_type, sch_boolean },
    { { 3, 5 }, { 0, 0 }, sp_isacceptingnewpatients, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_isaccessibleforfree, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_isaccessibleforfree, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_isaccessoryorsparepartfor, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_isaccessoryorsparepartfor, t_text },
    { { 2, 0 }, { 0, 0 }, sp_isavailablegenerally, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_isavailablegenerally, t_bool },
    { { 3, 0 }, { 0, 0 }, sp_isbasedon, t_schema_type, sch_creativework },
    { { 3, 0 }, { 0, 0 }, sp_isbasedon, t_schema_type, sch_product },
    { { 3, 0 }, { 0, 0 }, sp_isbasedon, t_schema_type, sch_url },
    { { 3, 0 }, { 0, 0 }, sp_isbasedon, t_url },
    { { 2, 0 }, { 0, 0 }, sp_isbasedonurl, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_isbasedonurl, t_url },
    { { 2, 0 }, { 0, 0 }, sp_isconsumablefor, t_schema_type, sch_product },
    { { 13, 0 }, { 0, 0 }, sp_isencodedbybiochementity, t_schema_type, sch_gene },
    { { 2, 0 }, { 0, 0 }, sp_isfamilyfriendly, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_isfamilyfriendly, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_isgift, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_isgift, t_bool },
    { { 13, 0 }, { 0, 0 }, sp_isinvolvedinbiologicalprocess, t_schema_type, sch_definedterm },
    { { 13, 0 }, { 0, 0 }, sp_isinvolvedinbiologicalprocess, t_schema_type, sch_propertyvalue },
    { { 13, 0 }, { 0, 0 }, sp_isinvolvedinbiologicalprocess, t_schema_type, sch_url },
    { { 13, 0 }, { 0, 0 }, sp_isinvolvedinbiologicalprocess, t_url },
    { { 13, 0 }, { 0, 0 }, sp_islocatedinsubcellularlocation, t_schema_type, sch_definedterm },
    { { 13, 0 }, { 0, 0 }, sp_islocatedinsubcellularlocation, t_schema_type, sch_propertyvalue },
    { { 13, 0 }, { 0, 0 }, sp_islocatedinsubcellularlocation, t_schema_type, sch_url },
    { { 13, 0 }, { 0, 0 }, sp_islocatedinsubcellularlocation, t_url },
    { { 2, 0 }, { 0, 0 }, sp_islivebroadcast, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_islivebroadcast, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_ispartof, t_schema_type, sch_creativework },
    { { 6, 0 }, { 0, 0 }, sp_ispartof, t_schema_type, sch_url },
    { { 6, 0 }, { 0, 0 }, sp_ispartof, t_url },
    { { 13, 0 }, { 0, 0 }, sp_ispartofbiochementity, t_schema_type, sch_biochementity },
    { { 6, 0 }, { 0, 0 }, sp_isplanforapartment, t_schema_type, sch_accommodation },
    { { 2, 0 }, { 0, 0 }, sp_isproprietary, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_isproprietary, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_isrelatedto, t_schema_type, sch_product },
    { { 3, 0 }, { 0, 0 }, sp_isrelatedto, t_schema_type, sch_service },
    { { 6, 0 }, { 0, 0 }, sp_isresizable, t_schema_type, sch_boolean },
    { { 6, 0 }, { 0, 0 }, sp_isresizable, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_issimilarto, t_schema_type, sch_product },
    { { 3, 0 }, { 0, 0 }, sp_issimilarto, t_schema_type, sch_service },
    { { 8, 0 }, { 0, 0 }, sp_isunlabelledfallback, t_bool },
    { { 9, 0 }, { 0, 0 }, sp_isvariantof, t_schema_type, sch_productgroup },
    { { 2, 0 }, { 0, 0 }, sp_isvariantof, t_schema_type, sch_productmodel },
    { { 2, 0 }, { 0, 0 }, sp_isbn, t_isbn },
    { { 2, 0 }, { 0, 0 }, sp_isicv4, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_isicv4, t_text },
    { { 2, 0 }, { 0, 0 }, sp_isrccode, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_isrccode, t_text },
    { { 3, 5 }, { 0, 0 }, sp_issn, t_issn },
    { { 2, 0 }, { 0, 0 }, sp_issn, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_issn, t_text },
    { { 2, 0 }, { 0, 0 }, sp_issuenumber, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_issuenumber, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_issuenumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_issuenumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_issuedby, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_issuedthrough, t_schema_type, sch_service },
    { { 2, 0 }, { 0, 0 }, sp_iswccode, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_iswccode, t_text },
    { { 2, 0 }, { 0, 0 }, sp_item, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_itemcondition, t_schema_type, sch_offeritemcondition },
    { { 13, 0 }, { 0, 0 }, sp_itemdefectreturnfees, t_schema_type, sch_returnfeesenumeration },
    { { 13, 0 }, { 0, 0 }, sp_itemdefectreturnlabelsource, t_schema_type, sch_returnlabelsourceenumeration },
    { { 13, 0 }, { 0, 0 }, sp_itemdefectreturnshippingfeesamount, t_schema_type, sch_monetaryamount },
    { { 2, 0 }, { 0, 0 }, sp_itemlistelement, t_schema_type, sch_listitem },
    { { 2, 0 }, { 0, 0 }, sp_itemlistelement, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_itemlistelement, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_itemlistelement, t_text },
    { { 2, 0 }, { 0, 0 }, sp_itemlistorder, t_schema_type, sch_itemlistordertype },
    { { 2, 0 }, { 0, 0 }, sp_itemlistorder, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_itemlistorder, t_text },
    { { 3, 5 }, { 0, 0 }, sp_itemlocation, t_schema_type, sch_place },
    { { 3, 5 }, { 0, 0 }, sp_itemlocation, t_schema_type, sch_postaladdress },
    { { 3, 5 }, { 0, 0 }, sp_itemlocation, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_itemlocation, t_text },
    { { 2, 0 }, { 0, 0 }, sp_itemreviewed, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_itemshipped, t_schema_type, sch_product },
    { { 6, 0 }, { 0, 0 }, sp_itemoffered, t_schema_type, sch_aggregateoffer },
    { { 6, 0 }, { 0, 0 }, sp_itemoffered, t_schema_type, sch_creativework },
    { { 6, 0 }, { 0, 0 }, sp_itemoffered, t_schema_type, sch_event },
    { { 6, 0 }, { 0, 0 }, sp_itemoffered, t_schema_type, sch_menuitem },
    { { 2, 0 }, { 0, 0 }, sp_itemoffered, t_schema_type, sch_product },
    { { 2, 2 }, { 0, 0 }, sp_itemoffered, t_schema_type, sch_service },
    { { 6, 0 }, { 0, 0 }, sp_itemoffered, t_schema_type, sch_trip },
    { { 13, 0 }, { 0, 0 }, sp_itemreviewed, t_schema_type, sch_thing },
    { { 3, 5 }, { 0, 0 }, sp_itinerary, t_schema_type, sch_itemlist },
    { { 3, 5 }, { 0, 0 }, sp_itinerary, t_schema_type, sch_place },
    { { 13, 0 }, { 0, 0 }, sp_iupacname, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_iupacname, t_text },
    { { 2, 0 }, { 0, 0 }, sp_jobbenefits, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_jobbenefits, t_text },
    { { 3, 8 }, { 0, 0 }, sp_jobimmediatestart, t_schema_type, sch_boolean },
    { { 3, 8 }, { 0, 0 }, sp_jobimmediatestart, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_joblocation, t_schema_type, sch_place },
    { { 3, 5 }, { 0, 0 }, sp_joblocationtype, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_joblocationtype, t_text },
    { { 3, 8 }, { 0, 0 }, sp_jobstartdate, t_schema_type, sch_date },
    { { 3, 8 }, { 0, 0 }, sp_jobstartdate, t_just_date },
    { { 3, 8 }, { 0, 0 }, sp_jobstartdate, t_schema_type, sch_text },
    { { 3, 8 }, { 0, 0 }, sp_jobstartdate, t_text },
    { { 3, 7 }, { 0, 0 }, sp_jobtitle, t_schema_type, sch_definedterm },
    { { 2, 0 }, { 0, 0 }, sp_jobtitle, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_jobtitle, t_text },
    { { 7, 4 }, { 0, 0 }, sp_jurisdiction, t_schema_type, sch_administrativearea },
    { { 7, 4 }, { 0, 0 }, sp_jurisdiction, t_schema_type, sch_text },
    { { 7, 4 }, { 0, 0 }, sp_jurisdiction, t_text },
    { { 10, 0 }, { 0, 0 }, sp_keywords, t_schema_type, sch_definedterm },
    { { 10, 0 }, { 0, 0 }, sp_keywords, t_schema_type, sch_url },
    { { 10, 0 }, { 0, 0 }, sp_keywords, t_url },
    { { 2, 0 }, { 0, 0 }, sp_keywords, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_keywords, t_text },
    { { 2, 0 }, { 0, 0 }, sp_knownvehicledamages, t_text },
    { { 2, 0 }, { 0, 0 }, sp_knows, t_schema_type, sch_person },
    { { 3, 5 }, { 0, 0 }, sp_knowsabout, t_schema_type, sch_thing },
    { { 3, 5 }, { 0, 0 }, sp_knowsabout, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_knowsabout, t_text },
    { { 3, 5 }, { 0, 0 }, sp_knowsabout, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_knowsabout, t_url },
    { { 3, 5 }, { 0, 0 }, sp_knowslanguage, t_schema_type, sch_language },
    { { 3, 5 }, { 0, 0 }, sp_knowslanguage, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_knowslanguage, t_url },
    { { 2, 0 }, { 0, 0 }, sp_labeldetails, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_labeldetails, t_url },
    { { 2, 0 }, { 0, 0 }, sp_landlord, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_landlord, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_lastreviewed, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_lastreviewed, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_latitude, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_latitude, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_latitude, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_latitude, t_text },
    { { 10, 0 }, { 0, 0 }, sp_layoutimage, t_schema_type, sch_imageobject },
    { { 10, 0 }, { 0, 0 }, sp_layoutimage, t_schema_type, sch_url },
    { { 10, 0 }, { 0, 0 }, sp_layoutimage, t_url },
    { { 11, 0 }, { 0, 0 }, sp_learningresourcetype, t_schema_type, sch_definedterm  },
    { { 2, 0 }, { 0, 0 }, sp_learningresourcetype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_learningresourcetype, t_text },
    { { 5, 0 }, { 0, 0 }, sp_leaselength, t_schema_type, sch_duration },
    { { 5, 0 }, { 0, 0 }, sp_leaselength, t_duration },
    { { 5, 0 }, { 0, 0 }, sp_leaselength, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_legalname, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_legalname, t_text },
    { { 3, 5 }, { 0, 0 }, sp_legalstatus, t_schema_type, sch_druglegalstatus },
    { { 3, 5 }, { 0, 0 }, sp_legalstatus, t_schema_type, sch_medicalenumeration },
    { { 2, 0 }, { 0, 0 }, sp_legalstatus, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_legalstatus, t_text },
    { { 3, 5 }, { 0, 0 }, sp_legislationapplies, t_schema_type, sch_legislation },
    { { 3, 5 }, { 0, 0 }, sp_legislationchanges, t_schema_type, sch_legislation },
    { { 3, 5 }, { 0, 0 }, sp_legislationconsolidates, t_schema_type, sch_legislation },
    { { 3, 5 }, { 0, 0 }, sp_legislationdate, t_schema_type, sch_date },
    { { 3, 5 }, { 0, 0 }, sp_legislationdate, t_just_date },
    { { 3, 5 }, { 0, 0 }, sp_legislationdateversion, t_schema_type, sch_date },
    { { 3, 5 }, { 0, 0 }, sp_legislationdateversion, t_just_date },
    { { 3, 5 }, { 0, 0 }, sp_legislationidentifier, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_legislationidentifier, t_text },
    { { 3, 5 }, { 0, 0 }, sp_legislationidentifier, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_legislationidentifier, t_url },
    { { 3, 5 }, { 0, 0 }, sp_legislationjurisdiction, t_schema_type, sch_administrativearea },
    { { 3, 5 }, { 0, 0 }, sp_legislationjurisdiction, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_legislationjurisdiction, t_text },
    { { 3, 5 }, { 0, 0 }, sp_legislationlegalforce, t_schema_type, sch_legalforcestatus },
    { { 3, 5 }, { 0, 0 }, sp_legislationlegalvalue, t_schema_type, sch_legalvaluelevel },
    { { 3, 5 }, { 0, 0 }, sp_legislationpassedby, t_schema_type, sch_organisation },
    { { 3, 5 }, { 0, 0 }, sp_legislationpassedby, t_schema_type, sch_person },
    { { 3, 5 }, { 0, 0 }, sp_legislationresponsible, t_schema_type, sch_organisation },
    { { 3, 5 }, { 0, 0 }, sp_legislationresponsible, t_schema_type, sch_person },
    { { 3, 5 }, { 0, 0 }, sp_legislationtransposes, t_schema_type, sch_legislation },
    { { 3, 5 }, { 0, 0 }, sp_legislationtype, t_schema_type, sch_categorycode },
    { { 3, 5 }, { 0, 0 }, sp_legislationtype, t_text },
    { { 3, 5 }, { 0, 0 }, sp_legislationtype, t_text },
    { { 3, 0 }, { 0, 0 }, sp_leicode, t_text },
    { { 3, 0 }, { 0, 0 }, sp_leicode, t_text },
    { { 3, 5 }, { 0, 0 }, sp_lender, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_lender, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_lesser, t_schema_type, sch_qualitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_lesserorequal, t_schema_type, sch_qualitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_letterer, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_license, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_license, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_license, t_url },
    { { 2, 0 }, { 0, 0 }, sp_line, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_line, t_text },
    { { 3, 5 }, { 0, 0 }, sp_linkrelationship, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_linkrelationship, t_text },
    { { 2, 1 }, { 0, 0 }, sp_liveblogupdate, t_schema_type, sch_blogposting },
    { { 3, 5 }, { 0, 0 }, sp_loanmortgagamandateamount, t_schema_type, sch_monetaryamount },
    { { 3, 5 }, { 0, 0 }, sp_loanpaymentamount, t_schema_type, sch_monetaryamount },
    { { 3, 5 }, { 0, 0 }, sp_loanpaymentfrequency, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_loanpaymentfrequency, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_loanrepaymentform, t_schema_type, sch_repaymentspecification },
    { { 3, 0 }, { 0, 0 }, sp_loanterm, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_loantype, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_loantype, t_text },
    { { 3, 5 }, { 0, 0 }, sp_loantype, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_loantype, t_url },
    { { 2, 0 }, { 0, 0 }, sp_location, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_location, t_schema_type, sch_postaladdress },
    { { 7, 0 }, { 0, 0 }, sp_location, t_schema_type, sch_virtuallocation },
    { { 2, 2 }, { 0, 0 }, sp_location, t_schema_type, sch_text },
    { { 2, 2 }, { 0, 0 }, sp_location, t_text },
    { { 2, 2 }, { 0, 0 }, sp_locationcreated, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_lodgingunitdescription, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_lodgingunitdescription, t_text },
    { { 2, 0 }, { 0, 0 }, sp_lodgingunittype, t_schema_type, sch_qualitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_lodgingunittype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_lodgingunittype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_logo, t_schema_type, sch_imageobject },
    { { 2, 0 }, { 0, 0 }, sp_logo, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_logo, t_url },
    { { 2, 0 }, { 0, 0 }, sp_longitude, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_longitude, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_longitude, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_longitude, t_text },
    { { 2, 0 }, { 0, 0 }, sp_loser, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_lowprice, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_lowprice, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_lowprice, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_lowprice, t_text },
    { { 2, 0 }, { 0, 0 }, sp_lyricist, t_schema_type, sch_person },
    { { 2, 1 }, { 0, 0 }, sp_lyrics, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_maincontentofpage, t_schema_type, sch_webpageelement },
    { { 2, 0 }, { 0, 0 }, sp_mainentity, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_mainentityofpage, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_mainentityofpage, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_mainentityofpage, t_url },
    { { 6, 0 }, { 0, 0 }, sp_maintainer, t_schema_type, sch_organisation },
    { { 6, 0 }, { 0, 0 }, sp_maintainer, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_makesoffer, t_schema_type, sch_offer },
    { { 2, 0 }, { 0, 0 }, sp_manufacturer, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_maptype, t_schema_type, sch_mapcategorytype },
    { { 3, 9 }, { 11, 0 }, sp_marginoferror, t_schema_type, sch_datetime },
    { { 3, 9 }, { 11, 0 }, sp_marginoferror, t_datetime },
    { { 12, 0 }, { 0, 0 }, sp_marginoferror, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_masthead, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_masthead, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_masthead, t_url },
    { { 3, 2 }, { 0, 0 }, sp_material, t_schema_type, sch_product },
    { { 3, 2 }, { 0, 0 }, sp_material, t_schema_type, sch_text },
    { { 3, 2 }, { 0, 0 }, sp_material, t_text },
    { { 3, 2 }, { 0, 0 }, sp_material, t_schema_type, sch_url },
    { { 3, 2 }, { 0, 0 }, sp_material, t_url },
    { { 3, 5 }, { 0, 0 }, sp_materialextent, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_materialextent, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_materialextent, t_text },
    { { 11, 0 }, { 0, 0 }, sp_mathexpression, t_schema_type, sch_solvemathaction },
    { { 11, 0 }, { 0, 0 }, sp_mathexpression, t_schema_type, sch_text },
    { { 11, 0 }, { 0, 0 }, sp_mathexpression, t_text },
    { { 2, 0 }, { 0, 0 }, sp_maxprice, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_maxprice, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_maxvalue, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_maxvalue, t_fixedpoint },
    { { 3, 1 }, { 0, 0 }, sp_maximumattendeecapacity, t_schema_type, sch_integer },
    { { 3, 1 }, { 0, 0 }, sp_maximumattendeecapacity, t_integer },
    { { 6, 0 }, { 0, 0 }, sp_maximumenrollment, t_schema_type, sch_integer },
    { { 6, 0 }, { 0, 0 }, sp_maximumenrollment, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_maximumintake, t_schema_type, sch_maximumdoseschedule },
    { { 7, 0 }, { 0, 0 }, sp_maximumphysicalattendeecapacity, t_schema_type, sch_integer },
    { { 7, 0 }, { 0, 0 }, sp_maximumphysicalattendeecapacity, t_integer },
    { { 7, 0 }, { 0, 0 }, sp_maximumvirtualattendeecapacity, t_schema_type, sch_integer },
    { { 7, 0 }, { 0, 0 }, sp_maximumvirtualattendeecapacity, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_mealservice, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_mealservice, t_text },
    { { 3, 9 }, { 0, 0 }, sp_measuredproperty, t_schema_type, sch_property },
    { { 3, 9 }, { 0, 0 }, sp_measuredvalue, t_generic },
    { { 3, 5 }, { 0, 0 }, sp_measurementtechnique, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_measurementtechnique, t_text },
    { { 3, 5 }, { 0, 0 }, sp_measurementtechnique, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_measurementtechnique, t_url },
    { { 2, 0 }, { 0, 0 }, sp_mechanismofaction, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_mechanismofaction, t_text },
    { { 7, 0 }, { 0, 0 }, sp_mediaauthenticitycategory, t_schema_type, sch_mediamanipulationratingenumeration },
    { { 3, 5 }, { 0, 0 }, sp_median, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_median, t_fixedpoint },
    { { 9, 0 }, { 0, 0 }, sp_aspect, t_schema_type, sch_medicalaudience },
    { { 9, 0 }, { 0, 0 }, sp_aspect, t_schema_type, sch_medicalaudiencetype },
    { { 13, 0 }, { 0, 0 }, sp_mediaitemappearance, t_schema_type, sch_mediaobject },
    { { 9, 0 }, { 0, 0 }, sp_medicalaudience, t_schema_type, sch_medicalaudience },
    { { 9, 0 }, { 0, 0 }, sp_medicalaudience, t_schema_type, sch_medicalaudiencetype },
    { { 2, 0 }, { 0, 0 }, sp_medicalspeciality, t_schema_type, sch_medicalspeciality },
    { { 2, 0 }, { 0, 0 }, sp_medicinesystem, t_schema_type, sch_medicinesystem },
    { { 3, 5 }, { 0, 0 }, sp_meetsemissionstandard, t_schema_type, sch_qualitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_meetsemissionstandard, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_meetsemissionstandard, t_text },
    { { 3, 5 }, { 0, 0 }, sp_meetsemissionstandard, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_meetsemissionstandard, t_url },
    { { 2, 0 }, { 0, 0 }, sp_member, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_member, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_memberof, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_memberof, t_schema_type, sch_programmemembership },
    { { 2, 0 }, { 0, 0 }, sp_membershipnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_membershipnumber, t_text },
    { { 3, 5 }, { 0, 0 }, sp_membershippointsearned, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_membershippointsearned, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_memoryrequirements, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_memoryrequirements, t_text },
    { { 2, 0 }, { 0, 0 }, sp_memoryrequirements, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_memoryrequirements, t_url },
    { { 2, 0 }, { 0, 0 }, sp_mentions, t_schema_type, sch_thing },
    { { 3, 2 }, { 0, 0 }, sp_menu, t_schema_type, sch_menu },
    { { 2, 0 }, { 0, 0 }, sp_menu, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_menu, t_text },
    { { 2, 0 }, { 0, 0 }, sp_menu, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_menu, t_url },
    { { 3, 5 }, { 0, 0 }, sp_menuaddon, t_schema_type, sch_menuitem },
    { { 3, 5 }, { 0, 0 }, sp_menuaddon, t_schema_type, sch_menusection },
    { { 6, 0 }, { 0, 0 }, sp_merchantreturndays, t_schema_type, sch_integer },
    { { 6, 0 }, { 0, 0 }, sp_merchantreturndays, t_integer },
    { { 6, 0 }, { 0, 0 }, sp_merchantreturnlink, t_schema_type, sch_url },
    { { 6, 0 }, { 0, 0 }, sp_merchantreturnlink, t_url },
    { { 3, 0 }, { 0, 0 }, sp_messageattachment, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_mileagefromodometer, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_minprice, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_minprice, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_minvalue, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_minvalue, t_fixedpoint },
    { { 9, 0 }, { 0, 0 }, sp_minimummonthlyrepaymentamount, t_schema_type, sch_number },
    { { 9, 0 }, { 0, 0 }, sp_minimummonthlyrepaymentamount, t_fixedpoint },
    { { 9, 0 }, { 0, 0 }, sp_minimummonthlyrepaymentamount, t_schema_type, sch_monetaryamount },
    { { 3, 0 }, { 0, 0 }, sp_minimumpaymentdue, t_schema_type, sch_monetaryamount },
    { { 2, 0 }, { 0, 0 }, sp_minimumpaymentdue, t_schema_type, sch_pricespecification },
    { { 3, 5 }, { 0, 0 }, sp_missioncoverageprioritiespolicy, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_missioncoverageprioritiespolicy, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_missioncoverageprioritiespolicy, t_url },
    { { 2, 0 }, { 0, 0 }, sp_model, t_schema_type, sch_productmodel },
    { { 2, 0 }, { 0, 0 }, sp_model, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_model, t_text },
    { { 3, 5 }, { 0, 0 }, sp_modeldate, t_schema_type, sch_date },
    { { 3, 5 }, { 0, 0 }, sp_modeldate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_modifiedtime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_modifiedtime, t_datetime },
    { { 12, 0 }, { 0, 0 }, sp_monthsofexperience, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_mpn, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_mpn, t_text },
    { { 2, 0 }, { 0, 0 }, sp_multiplevalues, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_multiplevalues, t_bool },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_muscleaction, t_schema_type, sch_text },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_muscleaction, t_text },
    { { 2, 0 }, { 0, 0 }, sp_musicarrangement, t_schema_type, sch_musiccomposition },
    { { 2, 0 }, { 0, 0 }, sp_musicby, t_schema_type, sch_musicgroup },
    { { 2, 0 }, { 0, 0 }, sp_musicby, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_musiccompositionform, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_musiccompositionform, t_text },
    { { 2, 0 }, { 0, 0 }, sp_musicalkey, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_musicalkey, t_text },
    { { 2, 0 }, { 0, 0 }, sp_musicreleaseformat, t_schema_type, sch_musicreleaseformattype },
    { { 13, 0 }, { 0, 0 }, sp_molecularformula, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_molecularformula, t_text },
    { { 13, 0 }, { 0, 0 }, sp_molecularweight, t_schema_type, sch_quantitativevalue },
    { { 13, 0 }, { 0, 0 }, sp_molecularweight, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_molecularweight, t_text },
    { { 13, 0 }, { 0, 0 }, sp_monoistopicmolecularweight, t_schema_type, sch_quantitativevalue },
    { { 13, 0 }, { 0, 0 }, sp_monoistopicmolecularweight, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_monoistopicmolecularweight, t_text },
    { { 2, 0 }, { 0, 0 }, sp_naics, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_naics, t_text },
    { { 2, 0 }, { 0, 0 }, sp_name, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_name, t_text },
    { { 2, 0 }, { 0, 0 }, sp_nationality, t_schema_type, sch_country },
    { { 2, 0 }, { 0, 0 }, sp_naturalprogression, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_naturalprogression, t_text },
    { { 13, 0 }, { 0, 0 }, sp_negativevotes, t_schema_type, sch_itemlist },
    { { 13, 0 }, { 0, 0 }, sp_negativevotes, t_schema_type, sch_listitem },
    { { 13, 0 }, { 0, 0 }, sp_negativevotes, t_schema_type, sch_webcontent },
    { { 13, 0 }, { 0, 0 }, sp_negativevotes, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_negativevotes, t_text },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_nerve, t_schema_type, sch_nerve },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_nervemotor, t_schema_type, sch_muscle },
    { { 3, 0 }, { 0, 0 }, sp_networth, t_schema_type, sch_monetaryamount },
    { { 2, 0 }, { 0, 0 }, sp_networth, t_schema_type, sch_pricespecification },
    { { 7, 0 }, { 0, 0 }, sp_newupdatesandguidelines, t_schema_type, sch_webcontent },
    { { 7, 0 }, { 0, 0 }, sp_newupdatesandguidelines, t_schema_type, sch_url },
    { { 7, 0 }, { 0, 0 }, sp_newupdatesandguidelines, t_url },
    { { 2, 0 }, { 0, 0 }, sp_nextitem, t_schema_type, sch_listitem },
    { { 3, 5 }, { 0, 0 }, sp_nobylinespolicy, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_nobylinespolicy, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_nobylinespolicy, t_url },
    { { 2, 0 }, { 0, 0 }, sp_nonequal, t_schema_type, sch_qualitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_nonpropriatryname, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_nonpropriatryname, t_text },
    { { 2, 0 }, { 0, 0 }, sp_nonproprietaryname, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_nonproprietaryname, t_text },
    { { 8, 0 }, { 0, 0 }, sp_nonprofitstatus, t_schema_type, sch_nonprofittype },
    { { 3, 5 }, { 0, 0 }, sp_normalrange, t_schema_type, sch_medicalenumeration },
    { { 2, 0 }, { 0, 0 }, sp_normalrange, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_normalrange, t_text },
    { { 3, 5 }, { 0, 0 }, sp_nsn, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_nsn, t_text },
    { { 2, 0 }, { 0, 0 }, sp_numadults, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_numadults, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_numadults, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_numchildren, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_numchildren, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_numchildren, t_schema_type, sch_quantitativevalue },
    { { 3, 9 }, { 0, 0 }, sp_numconstraints, t_schema_type, sch_integer },
    { { 3, 9 }, { 0, 0 }, sp_numconstraints, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_numtracks, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_numtracks, t_integer },
    { { 6, 0 }, { 0, 0 }, sp_numberofaccommodationunits, t_schema_type, sch_qualitativevalue },
    { { 6, 0 }, { 0, 0 }, sp_numberofaccommodationunits, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_numberofairbags, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_numberofairbags, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_numberofairbags, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_numberofairbags, t_text },
    { { 6, 0 }, { 0, 0 }, sp_numberofavailableaccommodationunits, t_schema_type, sch_qualitativevalue },
    { { 6, 0 }, { 0, 0 }, sp_numberofavailableaccommodationunits, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_numberofaxles, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_numberofaxles, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_numberofaxles, t_fixedpoint },
    { { 5, 0 }, { 0, 0 }, sp_numberofbathroomstotal, t_schema_type, sch_integer },
    { { 5, 0 }, { 0, 0 }, sp_numberofbathroomstotal, t_integer },
    { { 7, 0 }, { 0, 0 }, sp_numberofbedrooms, t_schema_type, sch_number },
    { { 7, 0 }, { 0, 0 }, sp_numberofbedrooms, t_fixedpoint },
    { { 7, 0 }, { 0, 0 }, sp_numberofbedrooms, t_schema_type, sch_qualitativevalue },
    { { 7, 0 }, { 0, 0 }, sp_numberofbedrooms, t_schema_type, sch_quantitativevalue },
    { { 3, 1 }, { 0, 0 }, sp_numberofbeds, t_schema_type, sch_number },
    { { 3, 1 }, { 0, 0 }, sp_numberofbeds, t_fixedpoint },
    { { 6, 0 }, { 0, 0 }, sp_numberofcredits, t_schema_type, sch_integer },
    { { 6, 0 }, { 0, 0 }, sp_numberofcredits, t_integer },
    { { 6, 0 }, { 0, 0 }, sp_numberofcredits, t_schema_type, sch_structuredvalue },
    { { 2, 1 }, { 0, 0 }, sp_numberofdoors, t_schema_type, sch_number },
    { { 2, 1 }, { 0, 0 }, sp_numberofdoors, t_fixedpoint },
    { { 2, 1 }, { 0, 0 }, sp_numberofdoors, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_numberofemployees, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_numberofepisodes, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_numberofepisodes, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_numberofforwardgears, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_numberofforwardgears, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_numberofforwardgears, t_schema_type, sch_quantitativevalue },
    { { 5, 0 }, { 0, 0 }, sp_numberoffullbathrooms, t_schema_type, sch_number },
    { { 5, 0 }, { 0, 0 }, sp_numberoffullbathrooms, t_fixedpoint },
    { { 6, 0 }, { 0, 0 }, sp_numberoffullbathrooms, t_schema_type, sch_integer },
    { { 6, 0 }, { 0, 0 }, sp_numberoffullbathrooms, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_numberofitems, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_numberofitems, t_integer },
    { { 3, 5 }, { 0, 0 }, sp_numberofloanpayments, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_numberofloanpayments, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_numberofpages, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_numberofpages, t_integer },
    { { 6, 0 }, { 0, 0 }, sp_numberofpartialbathrooms, t_schema_type, sch_number },
    { { 6, 0 }, { 0, 0 }, sp_numberofpartialbathrooms, t_fixedpoint },
    { { 6, 0 }, { 0, 0 }, sp_numberofpartialbathrooms, t_schema_type, sch_integer },
    { { 6, 0 }, { 0, 0 }, sp_numberofpartialbathrooms, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_numberofplayers, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_numberofpreviousowners, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_numberofpreviousowners, t_schema_type, sch_quantitativevalue },
    { { 3, 1 }, { 0, 0 }, sp_numberofrooms, t_schema_type, sch_number },
    { { 3, 1 }, { 0, 0 }, sp_numberofrooms, t_fixedpoint },
    { { 3, 1 }, { 0, 0 }, sp_numberofrooms, t_schema_type, sch_number },
    { { 3, 1 }, { 0, 0 }, sp_numberofrooms, t_fixedpoint },
    { { 6, 0 }, { 0, 0 }, sp_numberofrooms, t_schema_type, sch_integer },
    { { 6, 0 }, { 0, 0 }, sp_numberofrooms, t_integer },
    { { 6, 0 }, { 0, 0 }, sp_numberofrooms, t_schema_type, sch_qualitativevalue },
    { { 3, 1 }, { 0, 0 }, sp_numberofrooms, t_schema_type, sch_quantitativevalue  },
    { { 3, 1 }, { 0, 0 }, sp_numberofrooms, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_numberofseasons, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_numberofseasons, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_numberedposition, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_numberedposition, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_nutrition, t_schema_type, sch_nutritioninformation },
    { { 2, 0 }, { 0, 0 }, sp_object, t_schema_type, sch_thing },
    { { 3, 9 }, { 0, 0 }, sp_observationdate, t_schema_type, sch_datetime },
    { { 3, 9 }, { 0, 0 }, sp_observationdate, t_datetime },
    { { 3, 9 }, { 0, 0 }, sp_observednode, t_schema_type, sch_statisticalpopulation },
    { { 2, 0 }, { 0, 0 }, sp_ocaocode, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_ocaocode, t_text },
    { { 3, 1 }, { 0, 0 }, sp_occupancy, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_occupationlocation, t_schema_type, sch_administrativearea },
    { { 7, 0 }, { 0, 0 }, sp_occupationalcategory, t_schema_type, sch_categorycode },
    { { 2, 0 }, { 0, 0 }, sp_occupationalcategory, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_occupationalcategory, t_text },
    { { 3, 9 }, { 0, 0 }, sp_occupationalcredentialawarded, t_schema_type, sch_educationaloccupationalcredential },
    { { 3, 9 }, { 0, 0 }, sp_occupationalcredentialawarded, t_schema_type, sch_text },
    { { 3, 9 }, { 0, 0 }, sp_occupationalcredentialawarded, t_text },
    { { 3, 9 }, { 0, 0 }, sp_occupationalcredentialawarded, t_schema_type, sch_url },
    { { 3, 9 }, { 0, 0 }, sp_occupationalcredentialawarded, t_url },
    { { 2, 0 }, { 0, 0 }, sp_offercount, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_offercount, t_integer },
    { { 2, 2 }, { 0, 0 }, sp_offeredby, t_schema_type, sch_organisation },
    { { 2, 2 }, { 0, 0 }, sp_offeredby, t_schema_type, sch_person },
    { { 6, 0 }, { 0, 0 }, sp_offers, t_schema_type, sch_demand },
    { { 2, 0 }, { 0, 0 }, sp_offers, t_schema_type, sch_offer },
    { { 3, 5 }, { 0, 0 }, sp_offersprescriptionbymail, t_schema_type, sch_boolean },
    { { 3, 5 }, { 0, 0 }, sp_offersprescriptionbymail, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_openinghours, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_openinghours, t_text },
    { { 2, 0 }, { 0, 0 }, sp_openinghoursspecification, t_schema_type, sch_openinghoursspecification },
    { { 2, 0 }, { 0, 0 }, sp_opens, t_schema_type, sch_time },
    { { 2, 0 }, { 0, 0 }, sp_opens, t_just_time },
    { { 2, 0 }, { 0, 0 }, sp_operatingsystem, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_operatingsystem, t_text },
    { { 2, 0 }, { 0, 0 }, sp_opponent, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_orderdate, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_orderdate, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_orderdate, t_schema_type, sch_date },
    { { 3, 8 }, { 0, 0 }, sp_orderdate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_orderdelivery, t_schema_type, sch_parceldelivery },
    { { 2, 0 }, { 0, 0 }, sp_orderitemnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_orderitemnumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_orderitemstatus, t_schema_type, sch_orderstatus },
    { { 2, 0 }, { 0, 0 }, sp_ordernumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_ordernumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_orderquantity, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_orderquantity, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_orderstatus, t_schema_type, sch_orderstatus },
    { { 2, 0 }, { 0, 0 }, sp_ordereditem, t_schema_type, sch_orderitem },
    { { 2, 0 }, { 0, 0 }, sp_ordereditem, t_schema_type, sch_product },
    { { 3, 6 }, { 0, 0 }, sp_ordereditem, t_schema_type, sch_service },
    { { 2, 0 }, { 0, 0 }, sp_organiser, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_organiser, t_schema_type, sch_person },
    { { 2, 0, SV_NOT_3034 }, { 6, 0 }, sp_origin, t_schema_type, sch_anatomicalstructure },
    { { 2, 0 }, { 0, 0 }, sp_originaddress, t_schema_type, sch_postaladdress },
    { { 13, 0 }, { 0, 0 }, sp_originalmediacontentdescription, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_originalmediacontentdescription, t_text },
    { { 13, 0 }, { 0, 0 }, sp_originalmedialink, t_schema_type, sch_mediaobject },
    { { 13, 0 }, { 0, 0 }, sp_originalmedialink, t_schema_type, sch_webpage },
    { { 13, 0 }, { 0, 0 }, sp_originalmedialink, t_schema_type, sch_url },
    { { 13, 0 }, { 0, 0 }, sp_originalmedialink, t_url },
    { { 2, 0 }, { 0, 0 }, sp_originatesfrom, t_schema_type, sch_vessel },
    { { 3, 5 }, { 0, 0 }, sp_outcome, t_schema_type, sch_medicalentity },
    { { 2, 0 }, { 0, 0 }, sp_outcome, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_outcome, t_text },
    { { 2, 0 }, { 0, 0 }, sp_overdosage, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_overdosage, t_text },
    { { 2, 0 }, { 0, 0 }, sp_overview, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_overview, t_text },
    { { 2, 0 }, { 0, 0 }, sp_ownedfrom, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_ownedfrom, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_ownedthrough, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_ownedthrough, t_datetime },
    { { 3, 5 }, { 0, 0 }, sp_ownershipfundinginfo, t_schema_type, sch_aboutpage },
    { { 3, 5 }, { 0, 0 }, sp_ownershipfundinginfo, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_ownershipfundinginfo, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_ownershipfundinginfo, t_text },
    { { 3, 5 }, { 0, 0 }, sp_ownershipfundinginfo, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_ownershipfundinginfo, t_url },
    { { 2, 0 }, { 0, 0 }, sp_owns, t_schema_type, sch_ownershipinfo },
    { { 2, 0 }, { 0, 0 }, sp_owns, t_schema_type, sch_product },
    { { 2, 0 }, { 0, 0 }, sp_pageend, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_pageend, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_pageend, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_pageend, t_text },
    { { 2, 0 }, { 0, 0 }, sp_pagestart, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_pagestart, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_pagestart, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_pagestart, t_text },
    { { 2, 0 }, { 0, 0 }, sp_pagination, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_pagination, t_text },
    { { 2, 0 }, { 0, 0 }, sp_parent, t_schema_type, sch_person },
    { { 2, 0 }, { 9, 0 }, sp_parentitem, t_schema_type, sch_question },
    { { 10, 0 }, { 0, 0 }, sp_parentitem, t_schema_type, sch_comment },
    { { 2, 0 }, { 0, 0 }, sp_parentorganisation, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_parentservice,  t_schema_type, sch_broadcastservice },
    { { 13, 0 }, { 0, 0 }, sp_parenttaxon, t_schema_type, sch_taxon },
    { { 13, 0 }, { 0, 0 }, sp_parenttaxon, t_schema_type, sch_url },
    { { 13, 0 }, { 0, 0 }, sp_parenttaxon, t_url },
    { { 13, 0 }, { 0, 0 }, sp_parenttaxon, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_parenttaxon, t_text },
    { { 2, 0 }, { 0, 0 }, sp_partofepisode, t_schema_type, sch_episode },
    { { 2, 0 }, { 0, 0 }, sp_partofinvoice, t_schema_type, sch_invoice },
    { { 2, 0 }, { 0, 0 }, sp_partofseason, t_schema_type, sch_creativeworkseason },
    { { 2, 0 }, { 0, 0 }, sp_partofseries, t_schema_type, sch_creativeworkseries },
    { { 2, 0 }, { 0, 0 }, sp_partofsystem, t_schema_type, sch_anatomicalsystem },
    { { 3, 5 }, { 0, 0 }, sp_partoftrip, t_schema_type, sch_trip },
    { { 2, 0 }, { 0, 0 }, sp_participant, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_participant, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_partoforder, t_schema_type, sch_order },
    { { 2, 0 }, { 0, 0 }, sp_partysize, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_partysize, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_partysize, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_passengerprioritystatus, t_schema_type, sch_qualitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_passengerprioritystatus, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_passengerprioritystatus, t_text },
    { { 2, 0 }, { 0, 0 }, sp_passengersequencenumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_passengersequencenumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_pathophysiology, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_pathophysiology, t_text },
    { { 9, 0 }, { 0, 0 }, sp_pattern, t_schema_type, sch_definedterm },
    { { 9, 0 }, { 0, 0 }, sp_pattern, t_schema_type, sch_text },
    { { 9, 0 }, { 0, 0 }, sp_pattern, t_text },
    { { 3, 5 }, { 0, 0 }, sp_payload, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_paymentaccepted, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_paymentaccepted, t_text },
    { { 2, 0 }, { 2, 0 }, sp_paymentdue, t_schema_type, sch_datetime },
    { { 2, 0 }, { 2, 0 }, sp_paymentdue, t_datetime },
    { { 2, 1 }, { 0, 0 }, sp_paymentduedate, t_schema_type, sch_datetime },
    { { 2, 1 }, { 0, 0 }, sp_paymentduedate, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_paymentduedate, t_schema_type, sch_date },
    { { 3, 8 }, { 0, 0 }, sp_paymentduedate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_paymentmethod, t_schema_type, sch_paymentmethod },
    { { 2, 0 }, { 0, 0 }, sp_paymentmethodid, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_paymentmethodid, t_text },
    { { 2, 0 }, { 0, 0 }, sp_paymentstatus, t_schema_type, sch_paymentstatustype },
    { { 2, 0 }, { 0, 0 }, sp_paymentstatus, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_paymentstatus, t_text },
    { { 2, 0 }, { 0, 0 }, sp_paymenturl, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_paymenturl, t_url },
    { { 3, 5 }, { 0, 0 }, sp_penciler, t_schema_type, sch_person },
    { { 3, 5 }, { 0, 0 }, sp_percentile10, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_percentile10, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_percentile25, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_percentile25, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_percentile75, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_percentile75, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_percentile90, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_percentile90, t_fixedpoint },
    { { 3, 3 }, { 0, 0 }, sp_performtime, t_schema_type, sch_duration },
    { { 3, 3 }, { 0, 0 }, sp_performtime, t_duration },
    { { 2, 0 }, { 0, 0 }, sp_performer, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_performer, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_performerin, t_schema_type, sch_event },
    { { 3, 0 }, { 0, 0 }, sp_permissiontype, t_schema_type, sch_digitaldocumentpermissiontype },
    { { 2, 0 }, { 0, 0 }, sp_permissions, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_permissions, t_text },
    { { 2, 0 }, { 0, 0 }, sp_permitaudience, t_schema_type, sch_audience },
    { { 3, 1 }, { 0, 0 }, sp_permittedusage, t_schema_type, sch_text },
    { { 3, 1 }, { 0, 0 }, sp_permittedusage, t_text },
    { { 3, 1 }, { 0, 0 }, sp_petsallowed, t_schema_type, sch_text },
    { { 3, 1 }, { 0, 0 }, sp_petsallowed, t_text },
    { { 3, 1 }, { 0, 0 }, sp_petsallowed, t_schema_type, sch_boolean },
    { { 3, 1 }, { 0, 0 }, sp_petsallowed, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_phase, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_phase, t_text },
    { { 6, 0 }, { 0, 0 }, sp_phonetictext, t_schema_type, sch_text },
    { { 6, 0 }, { 0, 0 }, sp_phonetictext, t_text },
    { { 2, 0 }, { 0, 0 }, sp_photo, t_schema_type, sch_imageobject },
    { { 2, 0 }, { 0, 0 }, sp_photo, t_schema_type, sch_photograph },
    { { 6, 0 }, { 0, 0 }, sp_physicalrequirement, t_schema_type, sch_definedterm },
    { { 6, 0 }, { 0, 0 }, sp_physicalrequirement, t_schema_type, sch_text },
    { { 6, 0 }, { 0, 0 }, sp_physicalrequirement, t_text },
    { { 6, 0 }, { 0, 0 }, sp_physicalrequirement, t_schema_type, sch_url },
    { { 6, 0 }, { 0, 0 }, sp_physicalrequirement, t_url },
    { { 2, 0 }, { 0, 0 }, sp_physiologicalbenefits, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_physiologicalbenefits, t_text },
    { { 2, 0 }, { 0, 0 }, sp_pickuplocation, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_pickuptime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_pickuptime, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_playmode, t_schema_type, sch_gameplaymode },
    { { 2, 0 }, { 0, 0 }, sp_playertype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_playertype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_playersonline, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_playersonline, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_polygon, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_polygon, t_text },
    { { 2, 0 }, { 0, 0 }, sp_population, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_population, t_text },
    { { 3, 9 }, { 0, 0 }, sp_populationtype, t_schema_type, sch_class },
    { { 2, 0 }, { 0, 0 }, sp_position, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_position, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_position, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_position, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_position, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_position, t_text },
    { { 13, 0 }, { 0, 0 }, sp_positivevotes, t_schema_type, sch_itemlist },
    { { 13, 0 }, { 0, 0 }, sp_positivevotes, t_schema_type, sch_listitem },
    { { 13, 0 }, { 0, 0 }, sp_positivevotes, t_schema_type, sch_webcontent },
    { { 13, 0 }, { 0, 0 }, sp_positivevotes, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_positivevotes, t_text },
    { { 2, 0 }, { 0, 0 }, sp_possiblecomplication, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_possiblecomplication, t_text },
    { { 2, 0 }, { 0, 0 }, sp_possibletreatment, t_schema_type, sch_medicaltherapy },
    { { 2, 0 }, { 0, 0 }, sp_postofficeboxnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_postofficeboxnumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_postop, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_postop, t_text },
    { { 2, 0 }, { 0, 0 }, sp_postalcode, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_postalcode, t_text },
    { { 8, 0 }, { 0, 0 }, sp_postalcodebegin, t_schema_type, sch_text },
    { { 8, 0 }, { 0, 0 }, sp_postalcodebegin, t_text },
    { { 8, 0 }, { 0, 0 }, sp_postalcodeend, t_schema_type, sch_text },
    { { 8, 0 }, { 0, 0 }, sp_postalcodeend, t_text },
    { { 8, 0 }, { 0, 0 }, sp_postalcodeprefix, t_schema_type, sch_text },
    { { 8, 0 }, { 0, 0 }, sp_postalcodeprefix, t_text },
    { { 8, 0 }, { 0, 0 }, sp_postalcoderange, t_schema_type, sch_postalcoderangespecification },
    { { 2, 0 }, { 0, 0 }, sp_potentialaction, t_schema_type, sch_action },
    { { 13, 0 }, { 0, 0 }, sp_potentialuse, t_schema_type, sch_definedterm },
    { { 2, 0 }, { 0, 0 }, sp_preop, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_preop, t_text },
    { { 2, 0 }, { 0, 0 }, sp_predecessorof, t_schema_type, sch_productmodel },
    { { 2, 0 }, { 0, 0 }, sp_pregnancycategory, t_schema_type, sch_drugpregnancycategory },
    { { 2, 0 }, { 0, 0 }, sp_pregnancywarning, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_pregnancywarning, t_text },
    { { 2, 0 }, { 0, 0 }, sp_preptime, t_schema_type, sch_duration },
    { { 2, 0 }, { 0, 0 }, sp_preptime, t_duration },
    { { 3, 5 }, { 0, 0 }, sp_preparation, t_schema_type, sch_medicalentity },
    { { 2, 0 }, { 0, 0 }, sp_preparation, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_preparation, t_text },
    { { 2, 0 }, { 0, 0 }, sp_prescribinginfo, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_prescribinginfo, t_url },
    { { 2, 0 }, { 0, 0 }, sp_prescriptionstatus, t_schema_type, sch_drugprescriptionstatus },
    { { 3, 5 }, { 0, 0 }, sp_prescriptionstatus, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_prescriptionstatus, t_text },
    { { 2, 0 }, { 0, 0 }, sp_previousitem, t_schema_type, sch_listitem },
    { { 2, 0 }, { 0, 0 }, sp_previousstartdate, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_previousstartdate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_price, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_price, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_price, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_price, t_text },
    { { 3, 0 }, { 0, 0 }, sp_pricecomponent, t_schema_type, sch_unitpricespecification },
    { { 11, 0 }, { 0, 0 }, sp_pricecomponenttype, t_schema_type, sch_pricecomponenttypeenumeration },
    { { 2, 0 }, { 0, 0 }, sp_pricecurrency, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_pricecurrency, t_text },
    { { 2, 0 }, { 0, 0 }, sp_pricerange, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_pricerange, t_text },
    { { 2, 0 }, { 0, 0 }, sp_pricespecification, t_schema_type, sch_pricespecification },
    { { 2, 0 }, { 0, 0 }, sp_pricetype, t_schema_type, sch_pricetypeenumeration },
    { { 2, 0 }, { 0, 0 }, sp_pricetype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_pricetype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_pricevaliduntil, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_pricevaliduntil, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_primaryimageofpage, t_schema_type, sch_imageobject },
    { { 2, 0 }, { 0, 0 }, sp_primaryprevention, t_schema_type, sch_medicaltherapy },
    { { 2, 0 }, { 0, 0 }, sp_printcolumn, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_printcolumn, t_text },
    { { 2, 0 }, { 0, 0 }, sp_printedition, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_printedition, t_text },
    { { 2, 0 }, { 0, 0 }, sp_printpage, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_printpage, t_text },
    { { 2, 0 }, { 0, 0 }, sp_printsection, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_printsection, t_text },
    { { 2, 0 }, { 0, 0 }, sp_procedure, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_procedure, t_text },
    { { 2, 0 }, { 0, 0 }, sp_proceduretype, t_schema_type, sch_medicalproceduretype },
    { { 2, 0 }, { 0, 0 }, sp_processingtime, t_schema_type, sch_duration },
    { { 2, 0 }, { 0, 0 }, sp_processingtime, t_duration },
    { { 2, 0 }, { 0, 0 }, sp_processorrequirements, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_processorrequirements, t_text },
    { { 2, 0 }, { 0, 0 }, sp_producer, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_producer, t_schema_type, sch_person },
    { { 9, 0 }, { 0, 0 }, sp_productgroupid, t_schema_type, sch_text },
    { { 9, 0 }, { 0, 0 }, sp_productgroupid, t_text },
    { { 2, 0 }, { 0, 0 }, sp_productid, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_productid, t_text },
    { { 2, 0 }, { 0, 0 }, sp_productsupported, t_schema_type, sch_product },
    { { 2, 0 }, { 0, 0 }, sp_productsupported, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_productsupported, t_text },
    { { 2, 0 }, { 0, 0 }, sp_productioncompany, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_productiondate, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_productiondate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_proficiencylevel, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_proficiencylevel, t_text },
    { { 2, 0 }, { 0, 0 }, sp_programmemembershipused, t_schema_type, sch_programmemembership },
    { { 2, 0 }, { 0, 0 }, sp_programmename, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_programmename, t_text },
    { { 3, 9 }, { 0, 0 }, sp_programprequisites, t_schema_type, sch_alignmentobject },
    { { 3, 9 }, { 0, 0 }, sp_programprequisites, t_schema_type, sch_course },
    { { 3, 9 }, { 0, 0 }, sp_programprequisites, t_schema_type, sch_educationaloccupationalcredential },
    { { 3, 9 }, { 0, 0 }, sp_programprequisites, t_schema_type, sch_text },
    { { 3, 9 }, { 0, 0 }, sp_programprequisites, t_text },
    { { 7, 0 }, { 0, 0 }, sp_programtype, t_schema_type, sch_definedterm },
    { { 7, 0 }, { 0, 0 }, sp_programtype, t_schema_type, sch_text },
    { { 7, 0 }, { 0, 0 }, sp_programtype, t_text },
    { { 3, 0 }, { 0, 0 }, sp_programminglanguage, t_schema_type, sch_computerlanguage },
    { { 2, 0 }, { 0, 0 }, sp_programminglanguage, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_programminglanguage, t_text },
    { { 2, 0 }, { 0, 0 }, sp_programmingmodel, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_programmingmodel, t_text },
    { { 2, 0 }, { 0, 0 }, sp_propertyid, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_propertyid, t_text },
    { { 2, 0 }, { 0, 0 }, sp_propertyid, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_propertyid, t_url },
    { { 3, 5 }, { 0, 0 }, sp_propriatryname, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_propriatryname, t_text },
    { { 2, 0 }, { 0, 0 }, sp_proprietaryname, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_proprietaryname, t_text },
    { { 2, 0 }, { 0, 0 }, sp_proteincontent, t_schema_type, sch_mass },
    { { 2, 0 }, { 0, 0 }, sp_provider, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_provider, t_schema_type, sch_person },
    { { 2, 2 }, { 0, 0 }, sp_providermobility, t_schema_type, sch_text },
    { { 2, 2 }, { 0, 0 }, sp_providermobility, t_text },
    { { 2, 0 }, { 0, 0 }, sp_providesbroadcastservice, t_schema_type, sch_broadcastservice },
    { { 2, 0 }, { 0, 0 }, sp_providesservice, t_schema_type, sch_service },
    { { 3, 3 }, { 0, 0 }, sp_publicaccess, t_schema_type, sch_boolean },
    { { 3, 3 }, { 0, 0 }, sp_publicaccess, t_bool },
    { { 7, 0 }, { 0, 0 }, sp_publictransportclosuresinfo, t_schema_type, sch_webcontent },
    { { 7, 0 }, { 0, 0 }, sp_publictransportclosuresinfo, t_schema_type, sch_url },
    { { 7, 0 }, { 0, 0 }, sp_publictransportclosuresinfo, t_url },
    { { 2, 0 }, { 0, 0 }, sp_publication, t_schema_type, sch_publicationevent },
    { { 2, 0 }, { 0, 0 }, sp_publicationtype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_publicationtype, t_text },
    { { 3, 5 }, { 0, 0 }, sp_publishedby, t_schema_type, sch_organisation },
    { { 3, 5 }, { 0, 0 }, sp_publishedby, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_publishedon, t_schema_type, sch_broadcastservice },
    { { 2, 0 }, { 0, 0 }, sp_publisher, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_publisher, t_schema_type, sch_person },
    { { 3, 5 }, { 0, 0 }, sp_publisherimprint, t_schema_type, sch_organisation },
    { { 3, 3 }, { 0, 0 }, sp_publishingprinciples, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_publishingprinciples, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_publishingprinciples, t_url },
    { { 2, 0 }, { 0, 0 }, sp_purchasedate, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_purchasedate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_purpose, t_schema_type, sch_medicaldevicepurpose },
    { { 2, 0 }, { 6, 0 }, sp_purpose, t_schema_type, sch_medicaldevicepurpose },
    { { 2, 0 }, { 0, 0 }, sp_purpose, t_schema_type, sch_thing },
    { { 2, 0 }, { 6, 0 }, sp_purpose, t_schema_type, sch_thing },
    { { 3, 5 }, { 0, 0 }, sp_qualifications, t_schema_type, sch_educationaloccupationalcredential },
    { { 2, 0 }, { 0, 0 }, sp_qualifications, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_qualifications, t_text },
    { { 7, 0 }, { 0, 0 }, sp_quarantineguidelines, t_schema_type, sch_webcontent },
    { { 7, 0 }, { 0, 0 }, sp_quarantineguidelines, t_schema_type, sch_url },
    { { 7, 0 }, { 0, 0 }, sp_quarantineguidelines, t_url },
    { { 2, 0 }, { 0, 0 }, sp_query, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_query, t_text },
    { { 2, 0 }, { 0, 0 }, sp_quest, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_question, t_schema_type, sch_question },
    { { 2, 0 }, { 0, 0 }, sp_rangeincludes, t_schema_type, sch_class },
    { { 2, 0 }, { 0, 0 }, sp_ratingcount, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_ratingcount, t_integer },
    { { 3, 9 }, { 0, 0 }, sp_ratingexplanation, t_schema_type, sch_text },
    { { 3, 9 }, { 0, 0 }, sp_ratingexplanation, t_text },
    { { 3, 1 }, { 0, 0 }, sp_ratingvalue, t_schema_type, sch_number },
    { { 3, 1 }, { 0, 0 }, sp_ratingvalue, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_ratingvalue, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_ratingvalue, t_text },
    { { 3, 5 }, { 0, 0 }, sp_readby, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_readonlyvalue, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_readonlyvalue, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_realestateagent, t_schema_type, sch_realestateagent },
    { { 2, 0 }, { 0, 0 }, sp_recipe, t_schema_type, sch_recipe },
    { { 2, 0 }, { 0, 0 }, sp_recipecategory, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_recipecategory, t_text },
    { { 2, 0 }, { 0, 0 }, sp_recipecuisine, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_recipecuisine, t_text },
    { { 2, 0 }, { 0, 0 }, sp_recipeingredient, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_recipeingredient, t_text },
    { { 3, 3 }, { 0, 0 }, sp_recipeinstructions, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_recipeinstructions, t_schema_type, sch_itemlist },
    { { 2, 0 }, { 0, 0 }, sp_recipeinstructions, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_recipeinstructions, t_text },
    { { 3, 3 }, { 0, 0 }, sp_recipeyield, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_recipeyield, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_recipeyield, t_text },
    { { 2, 0 }, { 0, 0 }, sp_recipient, t_schema_type, sch_audience },
    { { 3, 3 }, { 0, 0 }, sp_recipient, t_schema_type, sch_contactpoint },
    { { 2, 0 }, { 0, 0 }, sp_recipient, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_recipient, t_schema_type, sch_person },
    { { 3, 5 }, { 0, 0 }, sp_recognisedby, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_recognisingauthority, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_recommendationstrength, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_recommendationstrength, t_text },
    { { 2, 0 }, { 0, 0 }, sp_recommendedintake, t_schema_type, sch_recommendeddoseschedule },
    { { 2, 0 }, { 0, 0 }, sp_recordlabel, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_recordedas, t_schema_type, sch_musicrecording },
    { { 2, 0 }, { 0, 0 }, sp_recordedat, t_schema_type, sch_event },
    { { 2, 0 }, { 0, 0 }, sp_recordedin, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_recordingof, t_schema_type, sch_musiccomposition },
    { { 2, 0 }, { 0, 0 }, sp_recordingof, t_schema_type, sch_musiccomposition },
    { { 3, 5 }, { 0, 0 }, sp_recourseloan, t_schema_type, sch_boolean },
    { { 3, 5 }, { 0, 0 }, sp_recourseloan, t_bool },
    { { 3, 0 }, { 0, 0 }, sp_referencequantity, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_referencesorder, t_schema_type, sch_order },
    { { 6, 0 }, { 0, 0 }, sp_refundtype, t_schema_type, sch_refundtypeenumeration },
    { { 2, 0 }, { 0, 0 }, sp_regiondrained, t_schema_type, sch_anatomicalstructure },
    { { 2, 0 }, { 0, 0 }, sp_regiondrained, t_schema_type, sch_anatomicalsystem },
    { { 2, 0 }, { 0, 0 }, sp_regionsallowed, t_schema_type, sch_place },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_relatedanatomy, t_schema_type, sch_anatomicalstructure },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_relatedanatomy, t_schema_type, sch_anatomicalsystem },
    { { 2, 0 }, { 0, 0 }, sp_relatedcondition, t_schema_type, sch_medicalcondition },
    { { 2, 0 }, { 0, 0 }, sp_relateddrug, t_schema_type, sch_drug },
    { { 2, 0 }, { 0, 0 }, sp_relatedlink, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_relatedlink, t_url },
    { { 2, 0 }, { 0, 0 }, sp_relatedstructure, t_schema_type, sch_anatomicalstructure },
    { { 2, 0 }, { 0, 0 }, sp_relatedtherapy, t_schema_type, sch_medicaltherapy },
    { { 2, 0 }, { 0, 0 }, sp_relatedto, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_releasedate, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_releasedate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_releasenotes, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_releasenotes, t_text },
    { { 2, 0 }, { 0, 0 }, sp_releasenotes, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_releasenotes, t_url },
    { { 2, 0 }, { 0, 0 }, sp_releaseof, t_schema_type, sch_musicalbum },
    { { 2, 0 }, { 0, 0 }, sp_releasedevent, t_schema_type, sch_publicationevent },
    { { 3, 5 }, { 0, 0 }, sp_relevantoccupation, t_schema_type, sch_occupation },
    { { 2, 0 }, { 0, 0 }, sp_relevantspeciality, t_schema_type, sch_medicalspeciality },
    { { 3, 1 }, { 0, 0 }, sp_remainingattendeecapacity, t_integer },
    { { 3, 5 }, { 0, 0 }, sp_renegotiableloan, t_schema_type, sch_boolean },
    { { 3, 5 }, { 0, 0 }, sp_renegotiableloan, t_bool },
    { { 3, 5 }, { 0, 0 }, sp_repeatcount, t_schema_type, sch_integer },
    { { 3, 5 }, { 0, 0 }, sp_repeatcount, t_integer },
    { { 3, 5 }, { 0, 0 }, sp_repeatfrequency, t_schema_type, sch_duration },
    { { 3, 5 }, { 0, 0 }, sp_repeatfrequency, t_duration },
    { { 3, 5 }, { 0, 0 }, sp_repeatfrequency, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_repeatfrequency, t_text },
    { { 2, 0 }, { 0, 0 }, sp_repetitions, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_repetitions, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_repetitions, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_replacee, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_replacer, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_replytourl, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_replytourl, t_url },
    { { 2, 1 }, { 0, 0 }, sp_reportnumber, t_schema_type, sch_text },
    { { 2, 1 }, { 0, 0 }, sp_reportnumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_representativeofpage, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_representativeofpage, t_bool },
    { { 3, 0 }, { 0, 0 }, sp_requiredcollateral, t_schema_type, sch_thing },
    { { 3, 0 }, { 0, 0 }, sp_requiredcollateral, t_schema_type, sch_text },
    { { 3, 0 }, { 0, 0 }, sp_requiredcollateral, t_text },
    { { 2, 0 }, { 0, 0 }, sp_requiredgender, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_requiredgender, t_text },
    { { 2, 0 }, { 0, 0 }, sp_requiredmaxage, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_requiredmaxage, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_requiredminage, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_requiredminage, t_integer },
    { { 3, 3 }, { 0, 0 }, sp_requiredquantity, t_schema_type, sch_number },
    { { 3, 3 }, { 0, 0 }, sp_requiredquantity, t_fixedpoint },
    { { 3, 3 }, { 0, 0 }, sp_requiredquantity, t_schema_type, sch_quantitativevalue },
    { { 3, 3 }, { 0, 0 }, sp_requiredquantity, t_schema_type, sch_text },
    { { 3, 3 }, { 0, 0 }, sp_requiredquantity, t_text },
    { { 3, 5 }, { 0, 0 }, sp_requiressubscription, t_schema_type, sch_mediasubscription },
    { { 2, 0 }, { 0, 0 }, sp_requiressubscription, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_requiressubscription, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_reservationfor, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_reservationid, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_reservationid, t_text },
    { { 2, 0 }, { 0, 0 }, sp_reservationstatus, t_schema_type, sch_reservationstatustype },
    { { 2, 0 }, { 0, 0 }, sp_reservedticket, t_schema_type, sch_ticket },
    { { 13, 0 }, { 0, 0 }, sp_restockingfee, t_schema_type, sch_monetaryamount },
    { { 13, 0 }, { 0, 0 }, sp_restockingfee, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_responsibilities, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_responsibilities, t_text },
    { { 3, 5 }, { 0, 0 }, sp_restperiods, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_restperiods, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_restperiods, t_text },
    { { 2, 0 }, { 0, 0 }, sp_result, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_resultcomment, t_schema_type, sch_comment },
    { { 2, 0 }, { 0, 0 }, sp_resultreview, t_schema_type, sch_review },
    { { 6, 0 }, { 0, 0 }, sp_returnfees, t_schema_type, sch_returnfeesenumeration },
    { { 13, 0 }, { 0, 0 }, sp_returnlabelsource, t_schema_type, sch_returnlabelsourceenumeration },
    { { 13, 0 }, { 0, 0 }, sp_returnmethod, t_schema_type, sch_returnmethodenumeration },
    { { 6, 0 }, { 0, 0 }, sp_returnpolicycategory, t_schema_type, sch_merchantreturnenumeration },
    { { 13, 0 }, { 0, 0 }, sp_returnpolicycountry, t_schema_type, sch_country },
    { { 13, 0 }, { 0, 0 }, sp_returnpolicycountry, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_returnpolicycountry, t_text },
    { { 13, 0 }, { 0, 0 }, sp_returnpolicyseasonaloverride, t_schema_type, sch_merchantreturnpolicyseasonaloverride },
    { { 13, 0 }, { 0, 0 }, sp_returnshippingfeesamount, t_schema_type, sch_monetaryamount },
    { { 2, 0 }, { 0, 0 }, sp_review, t_schema_type, sch_review },
    { { 3, 5 }, { 0, 0 }, sp_reviewaspect, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_reviewaspect, t_text },
    { { 2, 0 }, { 0, 0 }, sp_reviewbody, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_reviewbody, t_text },
    { { 2, 0 }, { 0, 0 }, sp_reviewcount, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_reviewcount, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_reviewrating, t_schema_type, sch_rating },
    { { 2, 0 }, { 0, 0 }, sp_reviewedby, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_reviewedby, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_riskfactor, t_schema_type, sch_medicalriskfactor},
    { { 2, 0 }, { 0, 0 }, sp_risks, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_risks, t_text },
    { { 2, 0 }, { 0, 0 }, sp_rolename, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_rolename, t_text },
    { { 2, 0 }, { 0, 0 }, sp_rolename, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_rolename, t_url },
    { { 3, 5 }, { 0, 0 }, sp_roofload, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_rsvpresponse, t_schema_type, sch_rsvpresponsetype },
    { { 2, 0 }, { 0, 0 }, sp_runsto, t_schema_type, sch_vessel },
    { { 2, 0 }, { 0, 0 }, sp_runtimeplatform, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_runtimeplatform, t_text },
    { { 3, 5 }, { 0, 0 }, sp_rxcui, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_rxcui, t_text },
    { { 2, 0 }, { 0, 0 }, sp_safetyconsideration, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_safetyconsideration, t_text },
    { { 2, 0 }, { 0, 0 }, sp_salarycurrency, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_salarycurrency, t_text },
    { { 3, 9 }, { 0, 0 }, sp_salaryuponcompletion, t_schema_type, sch_monetaryamountdistribution },
    { { 2, 0 }, { 0, 0 }, sp_sameas, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_sameas, t_url },
    { { 2, 0 }, { 0, 0 }, sp_saturatedfatcontent, t_schema_type, sch_mass },
    { { 2, 0 }, { 0, 0 }, sp_scheduledpaymentdate, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_scheduledpaymentdate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_scheduledtime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_scheduledtime, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_schemaversion, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_schemaversion, t_text },
    { { 2, 0 }, { 0, 0 }, sp_schemaversion, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_schemaversion, t_url },
    { { 7, 0 }, { 0, 0 }, sp_schoolclosuresinfo, t_schema_type, sch_webcontent },
    { { 7, 0 }, { 0, 0 }, sp_schoolclosuresinfo, t_schema_type, sch_url },
    { { 7, 0 }, { 0, 0 }, sp_schoolclosuresinfo, t_url },
    { { 2, 0 }, { 0, 0 }, sp_screencount, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_screencount, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_screenshot, t_schema_type, sch_imageobject },
    { { 2, 0 }, { 0, 0 }, sp_screenshot, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_screenshot, t_url },
    { { 3, 5 }, { 0, 0 }, sp_sddatepublished, t_schema_type, sch_datetime },
    { { 3, 5 }, { 0, 0 }, sp_sddatepublished, t_datetime },
    { { 3, 5 }, { 0, 0 }, sp_sdlicence, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_sdlicence, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_sdlicence, t_url },
    { { 3, 5 }, { 0, 0 }, sp_sdpublisher, t_schema_type, sch_organisation },
    { { 3, 5 }, { 0, 0 }, sp_sdpublisher, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_seasonnumber, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_seasonnumber, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_seasonnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_seasonnumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_seatnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_seatnumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_seatrow, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_seatrow, t_text },
    { { 2, 0 }, { 0, 0 }, sp_seatsection, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_seatsection, t_text },
    { { 3, 5 }, { 0, 0 }, sp_seatingcapacity, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_seatingcapacity, t_fixedpoint },
    { { 3, 5 }, { 0, 0 }, sp_seatingcapacity, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_seatingtype, t_schema_type, sch_qualitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_seatingtype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_seatingtype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_secondaryprevention, t_schema_type, sch_medicaltherapy },
    { { 6, 0 }, { 0, 0 }, sp_securityclearancerequirement, t_schema_type, sch_text },
    { { 6, 0 }, { 0, 0 }, sp_securityclearancerequirement, t_text },
    { { 6, 0 }, { 0, 0 }, sp_securityclearancerequirement, t_schema_type, sch_url },
    { { 6, 0 }, { 0, 0 }, sp_securityclearancerequirement, t_url },
    { { 2, 0 }, { 0, 0 }, sp_securityscreening, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_securityscreening, t_text },
    { { 2, 0 }, { 0, 0 }, sp_seeks, t_schema_type, sch_demand },
    { { 2, 0 }, { 0, 0 }, sp_seller, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_seller, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_sender, t_schema_type, sch_audience },
    { { 2, 0 }, { 0, 0 }, sp_sender, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_sender, t_schema_type, sch_person },
    { { 6, 0 }, { 0, 0 }, sp_sensoryclearancerequirement, t_schema_type, sch_definedterm },
    { { 6, 0 }, { 0, 0 }, sp_sensoryclearancerequirement, t_schema_type, sch_text },
    { { 6, 0 }, { 0, 0 }, sp_sensoryclearancerequirement, t_text },
    { { 6, 0 }, { 0, 0 }, sp_sensoryclearancerequirement, t_schema_type, sch_url },
    { { 6, 0 }, { 0, 0 }, sp_sensoryclearancerequirement, t_url },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_sensoryunit, t_schema_type, sch_anatomicalstructure },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_sensoryunit, t_schema_type, sch_superficialanatomy },
    { { 2, 0 }, { 0, 0 }, sp_serialnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_serialnumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_seriousadverseoutcome, t_schema_type, sch_medicalentity },
    { { 2, 0 }, { 0, 0 }, sp_serverstatus, t_schema_type, sch_gameserverstatus },
    { { 2, 0 }, { 0, 0 }, sp_servescuisine, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_servescuisine, t_text },
    { { 2, 0 }, { 0, 0 }, sp_servicearea, t_schema_type, sch_administrativearea },
    { { 2, 0 }, { 0, 0 }, sp_servicelocation, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_serviceoperator, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_serviceoutput, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_servicephone, t_schema_type, sch_contactpoint },
    { { 2, 0 }, { 0, 0 }, sp_servicepostaladdress, t_schema_type, sch_postaladdress },
    { { 2, 0 }, { 0, 0 }, sp_servicesmsnumber, t_schema_type, sch_contactpoint },
    { { 7, 4 }, { 0, 0 }, sp_servicetype, t_schema_type, sch_governmentbenefitstype },
    { { 2, 0 }, { 0, 0 }, sp_servicetype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_servicetype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_serviceurl, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_serviceurl, t_url },
    { { 2, 0 }, { 0, 0 }, sp_servingsize, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_servingsize, t_text },
    { { 13, 0 }, { 0, 0 }, sp_sha256, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_sha256, t_sha256 },
    { { 2, 1 }, { 0, 0 }, sp_sharedcontent, t_schema_type, sch_creativework },
    { { 7, 2 }, { 0, 0 }, sp_shippingdestination, t_schema_type, sch_definedregion },
    { { 8, 0 }, { 0, 0 }, sp_shippinglabel, t_schema_type, sch_text },
    { { 8, 0 }, { 0, 0 }, sp_shippinglabel, t_text },
    { { 8, 0 }, { 0, 0 }, sp_shippingrate, t_schema_type, sch_monetaryamount },
    { { 8, 0 }, { 0, 0 }, sp_shippingsettingslink, t_schema_type, sch_url },
    { { 8, 0 }, { 0, 0 }, sp_shippingsettingslink, t_url },
    { { 2, 0 }, { 0, 0 }, sp_sibling, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_signdetected, t_schema_type, sch_medicalsign },
    { { 2, 0 }, { 0, 0 }, sp_signorsymptom, t_schema_type, sch_medicalsignorsymptom},
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_significance, t_schema_type, sch_text },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_significance, t_text },
    { { 2, 0 }, { 0, 0 }, sp_significantlink, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_significantlink, t_url },
    { { 9, 0 }, { 0, 0 }, sp_size, t_schema_type, sch_definedterm },
    { { 9, 0 }, { 0, 0 }, sp_size, t_schema_type, sch_quantitativevalue },
    { { 12, 0 }, { 0, 0 }, sp_size, t_schema_type, sch_sizespecification },
    { { 9, 0 }, { 0, 0 }, sp_size, t_schema_type, sch_text },
    { { 9, 0 }, { 0, 0 }, sp_size, t_text },
    { { 12, 0 }, { 0, 0 }, sp_sizegroup, t_schema_type, sch_sizegroupenumeration },
    { { 12, 0 }, { 0, 0 }, sp_sizegroup, t_schema_type, sch_text },
    { { 12, 0 }, { 0, 0 }, sp_sizegroup, t_text },
    { { 12, 0 }, { 0, 0 }, sp_sizesystem, t_schema_type, sch_sizesystemenumeration },
    { { 12, 0 }, { 0, 0 }, sp_sizesystem, t_schema_type, sch_text },
    { { 12, 0 }, { 0, 0 }, sp_sizesystem, t_text },
    { { 5, 0 }, { 0, 0 }, sp_skills, t_schema_type, sch_definedterm },
    { { 2, 0 }, { 0, 0 }, sp_skills, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_skills, t_text },
    { { 2, 0 }, { 0, 0 }, sp_sku, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_sku, t_text },
    { { 3, 5 }, { 0, 0 }, sp_slogan, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_slogan, t_text },
    { { 13, 0 }, { 0, 0 }, sp_smiles, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_smiles, t_text },
    { { 3, 1 }, { 0, 0 }, sp_smokingallowed, t_schema_type, sch_boolean },
    { { 3, 1 }, { 0, 0 }, sp_smokingallowed, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_sodiumcontent, t_schema_type, sch_mass },
    { { 2, 0 }, { 0, 0 }, sp_softwareaddon, t_schema_type, sch_softwareapplication },
    { { 2, 0 }, { 0, 0 }, sp_softwarehelp, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_softwarerequirements, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_softwarerequirements, t_text },
    { { 2, 0 }, { 0, 0 }, sp_softwarerequirements, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_softwarerequirements, t_url },
    { { 2, 0 }, { 0, 0 }, sp_softwareversion, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_softwareversion, t_text },
    { { 2, 0 }, { 6, 0 }, sp_source, t_schema_type, sch_anatomicalstructure },
    { { 2, 0 }, { 0, 0 }, sp_sourceorganisation, t_schema_type, sch_organisation },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_sourcedfrom, t_schema_type, sch_brainstructure },
    { { 2, 0 }, { 0, 0 }, sp_spatial, t_schema_type, sch_place },
    { { 3, 1 }, { 0, 0 }, sp_spatialcoverage, t_schema_type, sch_place },
    { { 3, 5 }, { 0, 0 }, sp_speakable, t_schema_type, sch_speakablespecification },
    { { 3, 5 }, { 0, 0 }, sp_speakable, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_speakable, t_url },
    { { 2, 0 }, { 0, 0 }, sp_specialcommitments, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_specialcommitments, t_text },
    { { 3, 0 }, { 0, 0 }, sp_specialopeninghoursspecification, t_schema_type, sch_openinghoursspecification },
    { { 2, 0 }, { 0, 0 }, sp_speciality, t_schema_type, sch_speciality },
    { { 6, 0 }, { 0, 0 }, sp_speechtotextmarkup, t_schema_type, sch_text },
    { { 6, 0 }, { 0, 0 }, sp_speechtotextmarkup, t_text },
    { { 3, 5 }, { 0, 0 }, sp_speed, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_spokenbycharacter, t_schema_type, sch_organisation },
    { { 3, 5 }, { 0, 0 }, sp_spokenbycharacter, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_sponsor, t_schema_type, sch_organisation },
    { { 3, 1 }, { 0, 0 }, sp_sponsor, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_sport, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_sport, t_text },
    { { 2, 0 }, { 0, 0 }, sp_sport, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_sport, t_url },
    { { 2, 0 }, { 0, 0 }, sp_sportsactivitylocation, t_schema_type, sch_sportsactivitylocation },
    { { 2, 0 }, { 0, 0 }, sp_sportsevent, t_schema_type, sch_sportsevent },
    { { 2, 0 }, { 0, 0 }, sp_sportsteam, t_schema_type, sch_sportsteam },
    { { 2, 0 }, { 0, 0 }, sp_spouse, t_schema_type, sch_person },
    { { 6, 0 }, { 0, 0 }, sp_scheduletimezone, t_schema_type, sch_text },
    { { 6, 0 }, { 0, 0 }, sp_scheduletimezone, t_text },
    { { 2, 0 }, { 0, 0 }, sp_stage, t_schema_type, sch_medicalconditionstage },
    { { 2, 0 }, { 0, 0 }, sp_stageasnumber, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_stageasnumber, t_fixedpoint },
    { { 3, 1 }, { 0, 0 }, sp_starrating, t_schema_type, sch_rating },
    { { 2, 0 }, { 0, 0 }, sp_startdate, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_startdate, t_datetime },
    { { 6, 0 }, { 0, 0 }, sp_startdate, t_schema_type, sch_date },
    { { 6, 0 }, { 0, 0 }, sp_startdate, t_just_date },
    { { 11, 0 }, { 0, 0 }, sp_startoffset, t_schema_type, sch_hypertocentry },
    { { 3, 5 }, { 0, 0 }, sp_startoffset, t_schema_type, sch_number },
    { { 3, 5 }, { 0, 0 }, sp_startoffset, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_starttime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_starttime, t_datetime },
    { { 3, 8 }, { 0, 0 }, sp_starttime, t_schema_type, sch_time },
    { { 3, 8 }, { 0, 0 }, sp_starttime, t_just_time },
    { { 3, 5 }, { 0, 0 }, sp_status, t_schema_type, sch_eventstatustype },
    { { 2, 0 }, { 0, 0 }, sp_status, t_schema_type, sch_medicalstudystatus },
    { { 3, 5 }, { 0, 0 }, sp_status, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_status, t_text },
    { { 2, 0 }, { 0, 0 }, sp_steeringposition, t_schema_type, sch_steeringpositionvalue },
    { { 2, 0 }, { 0, 0 }, sp_stepvalue, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_stepvalue, t_fixedpoint },
    { { 3, 4 }, { 0, 0 }, sp_step, t_schema_type, sch_creativework },
    { { 3, 4 }, { 0, 0 }, sp_step, t_schema_type, sch_howtosection },
    { { 3, 4 }, { 0, 0 }, sp_step, t_schema_type, sch_itemlist },
    { { 3, 4 }, { 0, 0 }, sp_step, t_schema_type, sch_text },
    { { 3, 4 }, { 0, 0 }, sp_step, t_text },
    { { 3, 3 }, { 3, 3 }, sp_steps, t_schema_type, sch_creativework },
    { { 3, 3 }, { 3, 3 }, sp_steps, t_schema_type, sch_itemlist },
    { { 3, 3 }, { 3, 3 }, sp_steps, t_schema_type, sch_text },
    { { 3, 3 }, { 3, 3 }, sp_steps, t_text },
    { { 2, 0 }, { 0, 0 }, sp_storagerequirements, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_storagerequirements, t_text },
    { { 2, 0 }, { 0, 0 }, sp_storagerequirements, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_storagerequirements, t_url },
    { { 2, 0 }, { 0, 0 }, sp_streetaddress, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_streetaddress, t_text },
    { { 2, 0 }, { 0, 0 }, sp_strengthunit, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_strengthunit, t_text },
    { { 2, 0 }, { 0, 0 }, sp_strengthvalue, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_strengthvalue, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_structuralclass, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_structuralclass, t_text },
    { { 2, 0 }, { 0, 0 }, sp_study, t_schema_type, sch_medicalstudy },
    { { 2, 0 }, { 0, 0 }, sp_studydesign, t_schema_type, sch_medicalobservationalstudydesign },
    { { 2, 0 }, { 0, 0 }, sp_studylocation, t_schema_type, sch_administrativearea },
    { { 2, 0 }, { 0, 0 }, sp_studysubject, t_schema_type, sch_medicalentity },
    { { 3, 5 }, { 0, 0 }, sp_stupidproperty, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_subevent, t_schema_type, sch_event },
    { { 2, 0 }, { 0, 0 }, sp_suborganisation, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_subreservation, t_schema_type, sch_reservation },
    { { 2, 0 }, { 0, 0 }, sp_substagesuffix, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_substagesuffix, t_text },
    { { 2, 0 }, { 0, 0 }, sp_substructure, t_schema_type, sch_anatomicalstructure },
    { { 3, 5 }, { 0, 0 }, sp_subtrip, t_schema_type, sch_trip },
    { { 3, 5 }, { 0, 0 }, sp_subjectof, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_subjectof, t_schema_type, sch_event },
    { { 2, 0 }, { 0, 0 }, sp_subtitlelanguage, t_schema_type, sch_language },
    { { 2, 0 }, { 0, 0 }, sp_subtitlelanguage, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_subtitlelanguage, t_text },
    { { 2, 0 }, { 6, 0 }, sp_subtype, t_schema_type, sch_text },
    { { 2, 0 }, { 6, 0 }, sp_subtype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_successorof, t_schema_type, sch_productmodel },
    { { 2, 0 }, { 0, 0 }, sp_sugarcontent, t_schema_type, sch_mass },
    { { 12, 0 }, { 0, 0 }, sp_suggestedage, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_suggestedanswer, t_schema_type, sch_answer },
    { { 3, 4 }, { 0, 0 }, sp_suggestedanswer, t_schema_type, sch_itemlist },
    { { 12, 0 }, { 0, 0 }, sp_suggestedgender, t_schema_type, sch_gendertype },
    { { 2, 0 }, { 0, 0 }, sp_suggestedgender, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_suggestedgender, t_text },
    { { 2, 0 }, { 0, 0 }, sp_suggestedmaxage, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_suggestedmaxage, t_fixedpoint },
    { { 12, 0 }, { 0, 0 }, sp_suggestedmeasurement, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_suggestedminage, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_suggestedminage, t_fixedpoint },
    { { 3, 0 }, { 0, 0 }, sp_suitablefordiet, t_schema_type, sch_restricteddiet },
    { { 2, 0 }, { 0, 0 }, sp_superevent, t_schema_type, sch_event },
    { { 2, 0 }, { 0, 0 }, sp_supersededby, t_schema_type, sch_class },
    { { 2, 0 }, { 0, 0 }, sp_supersededby, t_schema_type, sch_enumeration },
    { { 2, 0 }, { 0, 0 }, sp_supersededby, t_schema_type, sch_property },
    { { 3, 3 }, { 0, 0 }, sp_supply, t_schema_type, sch_howtosupply },
    { { 3, 3 }, { 0, 0 }, sp_supply, t_schema_type, sch_text },
    { { 3, 3 }, { 0, 0 }, sp_supply, t_text },
    { { 2, 0 }, { 0, 0 }, sp_supplyto, t_schema_type, sch_anatomicalstructure },
    { { 2, 2 }, { 0, 0 }, sp_supportingdata, t_schema_type, sch_datafeed },
    { { 2, 0 }, { 0, 0 }, sp_suspersededby, t_schema_type, sch_class },
    { { 2, 0 }, { 0, 0 }, sp_suspersededby, t_schema_type, sch_enumeration },
    { { 2, 0 }, { 0, 0 }, sp_suspersededby, t_schema_type, sch_property },
    { { 2, 0 }, { 0, 0 }, sp_target, t_schema_type, sch_entrypoint },
    { { 2, 0 }, { 0, 0 }, sp_targetcollection, t_schema_type, sch_thing },
    { { 2, 0 }, { 0, 0 }, sp_targetdescription, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_targetdescription, t_text },
    { { 2, 0 }, { 0, 0 }, sp_targetname, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_targetname, t_text },
    { { 2, 0 }, { 0, 0 }, sp_targetplatform, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_targetplatform, t_text },
    { { 2, 0 }, { 0, 0 }, sp_targetpopulation, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_targetpopulation, t_text },
    { { 2, 0 }, { 0, 0 }, sp_targeturl, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_targeturl, t_url },
    { { 2, 0 }, { 0, 0 }, sp_taxid, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_taxid, t_text },
    { { 13, 0 }, { 0, 0 }, sp_taxonomicrange, t_schema_type, sch_definedterm },
    { { 13, 0 }, { 0, 0 }, sp_taxonomicrange, t_schema_type, sch_taxon },
    { { 13, 0 }, { 0, 0 }, sp_taxonomicrange, t_schema_type, sch_url },
    { { 13, 0 }, { 0, 0 }, sp_taxonomicrange, t_url },
    { { 13, 0 }, { 0, 0 }, sp_taxonrank, t_schema_type, sch_propertyvalue },
    { { 13, 0 }, { 0, 0 }, sp_taxonrank, t_schema_type, sch_url },
    { { 13, 0 }, { 0, 0 }, sp_taxonrank, t_url },
    { { 13, 0 }, { 0, 0 }, sp_taxonrank, t_schema_type, sch_text },
    { { 13, 0 }, { 0, 0 }, sp_taxonrank, t_text },
    { { 8, 0 }, { 0, 0 }, sp_teaches, t_schema_type, sch_definedterm },
    { { 8, 0 }, { 0, 0 }, sp_teaches, t_schema_type, sch_text },
    { { 8, 0 }, { 0, 0 }, sp_teaches, t_text },
    { { 2, 0 }, { 0, 0 }, sp_telephone, t_tel },
    { { 3, 5 }, { 0, 0 }, sp_temporal, t_schema_type, sch_datetime },
    { { 3, 5 }, { 0, 0 }, sp_temporal, t_datetime },
    { { 3, 5 }, { 0, 0 }, sp_temporal, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_temporal, t_text },
    { { 3, 1 }, { 0, 0 }, sp_temporalcoverage, t_schema_type, sch_datetime },
    { { 3, 1 }, { 0, 0 }, sp_temporalcoverage, t_datetime },
    { { 3, 1 }, { 0, 0 }, sp_temporalcoverage, t_schema_type, sch_text },
    { { 3, 1 }, { 0, 0 }, sp_temporalcoverage, t_text },
    { { 3, 1 }, { 0, 0 }, sp_temporalcoverage, t_schema_type, sch_url },
    { { 3, 1 }, { 0, 0 }, sp_temporalcoverage, t_url },
    { { 3, 5 }, { 0, 0 }, sp_termcode, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_termcode, t_text },
    { { 6, 0 }, { 0, 0 }, sp_termduration, t_schema_type, sch_duration },
    { { 6, 0 }, { 0, 0 }, sp_termduration, t_duration },
    { { 3, 5 }, { 0, 0 }, sp_termsofservice, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_termsofservice, t_text },
    { { 3, 5 }, { 0, 0 }, sp_termsofservice, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_termsofservice, t_url },
    { { 6, 0 }, { 0, 0 }, sp_termsperyear, t_schema_type, sch_number },
    { { 6, 0 }, { 0, 0 }, sp_termsperyear, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_text, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_text, t_text },
    { { 6, 0 }, { 0, 0 }, sp_textvalue, t_schema_type, sch_text },
    { { 6, 0 }, { 0, 0 }, sp_textvalue, t_text },
    { { 2, 0 }, { 0, 0 }, sp_thumbnail, t_schema_type, sch_imageobject },
    { { 2, 0 }, { 0, 0 }, sp_thumbnailurl, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_thumbnailurl, t_url },
    { { 2, 0 }, { 0, 0 }, sp_tickersymbol, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_tickersymbol, t_text },
    { { 2, 0 }, { 0, 0 }, sp_ticketnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_ticketnumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_tickettoken, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_tickettoken, t_text },
    { { 2, 0 }, { 0, 0 }, sp_tickettoken, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_tickettoken, t_url },
    { { 2, 0 }, { 0, 0 }, sp_ticketedseat, t_schema_type, sch_seat },
    { { 6, 0 }, { 0, 0 }, sp_timeofday, t_schema_type, sch_text },
    { { 6, 0 }, { 0, 0 }, sp_timeofday, t_text },
    { { 2, 0 }, { 0, 0 }, sp_timerequired, t_schema_type, sch_duration },
    { { 2, 0 }, { 0, 0 }, sp_timerequired, t_duration },
    { { 3, 9 }, { 0, 0 }, sp_timetocomplete, t_schema_type, sch_duration },
    { { 3, 9 }, { 0, 0 }, sp_timetocomplete, t_duration },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_tissuesample, t_schema_type, sch_text },
    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sp_tissuesample, t_text },
    { { 2, 0 }, { 0, 0 }, sp_title, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_title, t_text },
    { { 8, 0 }, { 0, 0 }, sp_titleeidr, t_schema_type, sch_text },
    { { 8, 0 }, { 0, 0 }, sp_titleeidr, t_text },
    { { 8, 0 }, { 0, 0 }, sp_titleeidr, t_schema_type, sch_url },
    { { 8, 0 }, { 0, 0 }, sp_titleeidr, t_url },
    { { 11, 0 }, { 0, 0 }, sp_toccontinuation, t_schema_type, sch_hypertocentry },
    { { 11, 0 }, { 0, 0 }, sp_tocentry, t_schema_type, sch_hypertocentry },
    { { 2, 0 }, { 0, 0 }, sp_tolocation, t_schema_type, sch_place },
    { { 3, 3 }, { 0, 0 }, sp_torecipient, t_schema_type, sch_audience },
    { { 3, 3 }, { 0, 0 }, sp_torecipient, t_schema_type, sch_contactpoint },
    { { 3, 3 }, { 0, 0 }, sp_torecipient, t_schema_type, sch_organisation },
    { { 3, 3 }, { 0, 0 }, sp_torecipient, t_schema_type, sch_person },
    { { 3, 5 }, { 0, 0 }, sp_tongueweight, t_schema_type, sch_quantitativevalue },
    { { 3, 3 }, { 0, 0 }, sp_tool, t_schema_type, sch_howtotool },
    { { 3, 3 }, { 0, 0 }, sp_tool, t_schema_type, sch_text },
    { { 3, 3 }, { 0, 0 }, sp_tool, t_text },
    { { 3, 5 }, { 0, 0 }, sp_torque, t_schema_type, sch_quantitativevalue },
    { { 4, 0 }, { 0, 0 }, sp_totaljobopenings, t_schema_type, sch_integer },
    { { 4, 0 }, { 0, 0 }, sp_totaljobopenings, t_integer },
    { { 3, 0 }, { 0, 0 }, sp_totalpaymentdue, t_schema_type, sch_monetaryamount },
    { { 2, 0 }, { 0, 0 }, sp_totalpaymentdue, t_schema_type, sch_pricespecification },
    { { 2, 0 }, { 0, 0 }, sp_totalprice, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_totalprice, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_totalprice, t_schema_type, sch_pricespecification },
    { { 2, 0 }, { 0, 0 }, sp_totalprice, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_totalprice, t_text },
    { { 2, 0 }, { 0, 0 }, sp_totaltime, t_schema_type, sch_duration },
    { { 2, 0 }, { 0, 0 }, sp_totaltime, t_duration },
    { { 7, 0 }, { 0, 0 }, sp_tourbookingpage, t_schema_type, sch_url },
    { { 7, 0 }, { 0, 0 }, sp_tourbookingpage, t_url },
    { { 3, 3 }, { 0, 0 }, sp_touristtype, t_schema_type, sch_audience },
    { { 3, 3 }, { 0, 0 }, sp_touristtype, t_schema_type, sch_text },
    { { 3, 3 }, { 0, 0 }, sp_touristtype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_track, t_schema_type, sch_itemlist },
    { { 2, 0 }, { 0, 0 }, sp_track, t_schema_type, sch_musicrecording },
    { { 2, 0 }, { 0, 0 }, sp_trackingnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_trackingnumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_trackingurl, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_trackingurl, t_url },
    { { 2, 0 }, { 0, 0 }, sp_trailer, t_schema_type, sch_videoobject },
    { { 3, 5 }, { 0, 0 }, sp_trailerweight, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_trainname, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_trainname, t_text },
    { { 2, 0 }, { 0, 0 }, sp_trainnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_trainnumber, t_text },
    { { 3, 9 }, { 0, 0 }, sp_trainingsalary, t_schema_type, sch_monetaryamountdistribution },
    { { 2, 0 }, { 0, 0 }, sp_transfatcontent, t_schema_type, sch_mass },
    { { 2, 0 }, { 0, 0 }, sp_transcript, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_transcript, t_text },
    { { 8, 0 }, { 0, 0 }, sp_transittime, t_schema_type, sch_quantitativevalue },
    { { 8, 0 }, { 0, 0 }, sp_transittimelabel, t_schema_type, sch_text },
    { { 8, 0 }, { 0, 0 }, sp_transittimelabel, t_text },
    { { 2, 0 }, { 0, 0 }, sp_translationofwork, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_translator, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_translator, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_transmissionmethod, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_transmissionmethod, t_text },
    { { 7, 0 }, { 0, 0 }, sp_travelbans, t_schema_type, sch_webcontent },
    { { 7, 0 }, { 0, 0 }, sp_travelbans, t_schema_type, sch_url },
    { { 7, 0 }, { 0, 0 }, sp_travelbans, t_url },
    { { 2, 0 }, { 0, 0 }, sp_trialdesign, t_schema_type, sch_medicaltrialdesign },
    { { 2, 0 }, { 0, 0 }, sp_tributary, t_schema_type, sch_anatomicalstructure },
    { { 3, 5 }, { 0, 0 }, sp_typeofbed, t_schema_type, sch_bedtype },
    { { 3, 1 }, { 0, 0 }, sp_typeofbed, t_schema_type, sch_text },
    { { 3, 1 }, { 0, 0 }, sp_typeofbed, t_text },
    { { 2, 0 }, { 0, 0 }, sp_typeofgood, t_schema_type, sch_product },
    { { 3, 1 }, { 0, 0 }, sp_typeofgood, t_schema_type, sch_service },
    { { 2, 0 }, { 0, 0 }, sp_typicalagerange, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_typicalagerange, t_text },
    { { 6, 0 }, { 0, 0 }, sp_typicalcreditsperterm, t_schema_type, sch_integer },
    { { 6, 0 }, { 0, 0 }, sp_typicalcreditsperterm, t_integer },
    { { 6, 0 }, { 0, 0 }, sp_typicalcreditsperterm, t_schema_type, sch_structuredvalue },
    { { 2, 0 }, { 0, 0 }, sp_typicaltest, t_schema_type, sch_medicaltest },
    { { 2, 0 }, { 0, 0 }, sp_undername, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_undername, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_unitcode, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_unitcode, t_text },
    { { 2, 0 }, { 0, 0 }, sp_unitcode, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_unitcode, t_url },
    { { 2, 0 }, { 0, 0 }, sp_unittext, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_unittext, t_text },
    { { 3, 5 }, { 0, 0 }, sp_unnamedsourcespolicy, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_unnamedsourcespolicy, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_unnamedsourcespolicy, t_url },
    { { 2, 0 }, { 0, 0 }, sp_unsaturatedfatcontent, t_schema_type, sch_mass },
    { { 2, 0 }, { 0, 0 }, sp_uploaddate, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_uploaddate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_upvotecount, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_upvotecount, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_url, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_url, t_url },
    { { 2, 0 }, { 0, 0 }, sp_urltemplate, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_urltemplate, t_text },
    { { 7, 0 }, { 0, 0 }, sp_usageinfo, t_schema_type, sch_creativework },
    { { 7, 0 }, { 0, 0 }, sp_usageinfo, t_schema_type, sch_url },
    { { 7, 0 }, { 0, 0 }, sp_usageinfo, t_url },
    { { 2, 0 }, { 0, 0 }, sp_usedtodiagnose, t_schema_type, sch_medicalcondition },
    { { 2, 0 }, { 0, 0 }, sp_userinteractioncount, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_userinteractioncount, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_usesdevice, t_schema_type, sch_medicaldevice },
    { { 3, 5 }, { 0, 0 }, sp_useshealthplanidstandard, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_useshealthplanidstandard, t_text },
    { { 3, 5 }, { 0, 0 }, sp_useshealthplanidstandard, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_useshealthplanidstandard, t_url },
    { { 11, 0 }, { 0, 0 }, sp_utterances, t_schema_type, sch_text },
    { { 11, 0 }, { 0, 0 }, sp_utterances, t_text },
    { { 2, 0 }, { 0, 0 }, sp_validfor, t_schema_type, sch_duration },
    { { 2, 0 }, { 0, 0 }, sp_validfor, t_duration },
    { { 2, 0 }, { 0, 0 }, sp_validfrom, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_validfrom, t_datetime },
    { { 2, 0 }, { 0, 0 }, sp_validin, t_schema_type, sch_administrativearea },
    { { 2, 0 }, { 0, 0 }, sp_validthrough, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_validthrough, t_datetime },
    { { 4, 0 }, { 0, 0 }, sp_validthrough, t_schema_type, sch_date },
    { { 4, 0 }, { 0, 0 }, sp_validthrough, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_validuntil, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_validuntil, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_value, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_value, t_fixedpoint },
    { { 3, 0 }, { 0, 0 }, sp_value, t_schema_type, sch_structuredvalue },
    { { 3, 0 }, { 0, 0 }, sp_value, t_schema_type, sch_boolean },
    { { 3, 0 }, { 0, 0 }, sp_value, t_bool },
    { { 3, 0 }, { 0, 0 }, sp_value, t_schema_type, sch_text },
    { { 3, 0 }, { 0, 0 }, sp_value, t_text },
    { { 2, 0 }, { 0, 0 }, sp_valueaddedtaxincluded, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_valueaddedtaxincluded, t_bool },
    { { 2, 0 }, { 0, 0 }, sp_valuemaxlength, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_valuemaxlength, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_valueminlength, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_valueminlength, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_valuename, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_valuename, t_text },
    { { 2, 0 }, { 0, 0 }, sp_valuepattern, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_valuepattern, t_text },
    { { 2, 0 }, { 0, 0 }, sp_valuereference, t_schema_type, sch_enumeration },
    { { 12, 0 }, { 0, 0 }, sp_valuereference, t_schema_type, sch_measurementtypeenumeration },
    { { 2, 0 }, { 0, 0 }, sp_valuereference, t_schema_type, sch_propertyvalue },
    { { 2, 0 }, { 0, 0 }, sp_valuereference, t_schema_type, sch_qualitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_valuereference, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_valuereference, t_schema_type, sch_structuredvalue },
    { { 2, 0 }, { 0, 0 }, sp_valuerequired, t_schema_type, sch_boolean },
    { { 2, 0 }, { 0, 0 }, sp_valuerequired, t_bool },
    { { 3, 5 }, { 0, 0 }, sp_variablemeasured, t_schema_type, sch_propertyvalue },
    { { 3, 5 }, { 0, 0 }, sp_variablemeasured, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_variablemeasured, t_text },
    { { 3, 5 }, { 0, 0 }, sp_variablesmeasured, t_schema_type, sch_propertyvalue },
    { { 3, 5 }, { 0, 0 }, sp_variablesmeasured, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_variablesmeasured, t_text },
    { { 3, 5 }, { 0, 0 }, sp_variantcover, t_schema_type, sch_text },
    { { 3, 5 }, { 0, 0 }, sp_variantcover, t_text },
    { { 9, 0 }, { 0, 0 }, sp_variesby, t_schema_type, sch_definedterm },
    { { 9, 0 }, { 0, 0 }, sp_variesby, t_schema_type, sch_text },
    { { 9, 0 }, { 0, 0 }, sp_variesby, t_text },
    { { 2, 0 }, { 0, 0 }, sp_vatid, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_vatid, t_text },
    { { 2, 0 }, { 0, 0 }, sp_vehicleconfiguration, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_vehicleconfiguration, t_text },
    { { 2, 0 }, { 0, 0 }, sp_vehicleengine, t_schema_type, sch_enginespecification },
    { { 2, 0 }, { 0, 0 }, sp_vehicleidentificationnumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_vehicleidentificationnumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_vehicleinteriorcolour, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_vehicleinteriorcolour, t_text },
    { { 2, 0 }, { 0, 0 }, sp_vehicleinteriortype, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_vehicleinteriortype, t_text },
    { { 2, 0 }, { 0, 0 }, sp_vehiclemodeldate, t_schema_type, sch_date },
    { { 2, 0 }, { 0, 0 }, sp_vehiclemodeldate, t_just_date },
    { { 2, 0 }, { 0, 0 }, sp_vehicleseatingcapacity, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_vehicleseatingcapacity, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_vehicleseatingcapacity, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_vehiclespecialusage, t_schema_type, sch_carusagetype },
    { { 3, 0 }, { 0, 0 }, sp_vehiclespecialusage, t_schema_type, sch_text },
    { { 3, 0 }, { 0, 0 }, sp_vehiclespecialusage, t_text },
    { { 2, 0 }, { 0, 0 }, sp_vehicletransmission, t_schema_type, sch_qualitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_vehicletransmission, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_vehicletransmission, t_text },
    { { 2, 0 }, { 0, 0 }, sp_vehicletransmission, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_vehicletransmission, t_url },
    { { 3, 5 }, { 0, 0 }, sp_verificationfactcheckingpolicy, t_schema_type, sch_creativework },
    { { 3, 5 }, { 0, 0 }, sp_verificationfactcheckingpolicy, t_schema_type, sch_url },
    { { 3, 5 }, { 0, 0 }, sp_verificationfactcheckingpolicy, t_url },
    { { 2, 0 }, { 0, 0 }, sp_version, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_version, t_fixedpoint },
    { { 3, 1 }, { 0, 0 }, sp_version, t_schema_type, sch_text },
    { { 3, 1 }, { 0, 0 }, sp_version, t_text },
    { { 3, 5 }, { 0, 0 }, sp_video, t_schema_type, sch_clip },
    { { 2, 0 }, { 0, 0 }, sp_video, t_schema_type, sch_videoobject },
    { { 2, 0 }, { 0, 0 }, sp_videoformat, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_videoformat, t_text },
    { { 2, 0 }, { 0, 0 }, sp_videoframesize, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_videoframesize, t_text },
    { { 2, 0 }, { 0, 0 }, sp_videoquality, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_videoquality, t_text },
    { { 2, 0 }, { 0, 0 }, sp_volumenumber, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_volumenumber, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_volumenumber, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_volumenumber, t_text },
    { { 2, 0 }, { 0, 0 }, sp_warning, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_warning, t_text },
    { { 2, 0 }, { 0, 0 }, sp_warning, t_schema_type, sch_url },
    { { 2, 0 }, { 0, 0 }, sp_warning, t_url },
    { { 2, 0 }, { 0, 0 }, sp_warranty, t_schema_type, sch_warrantypromise },
    { { 2, 0 }, { 0, 0 }, sp_warrantyscope, t_schema_type, sch_warrantyscope },
    { { 2, 0 }, { 0, 0 }, sp_webcheckintime, t_schema_type, sch_datetime },
    { { 2, 0 }, { 0, 0 }, sp_webcheckintime, t_datetime },
    { { 7, 1 }, { 0, 0 }, sp_webfeed, t_schema_type, sch_datafeed },
    { { 4, 0 }, { 0, 0 }, sp_webfeed, t_schema_type, sch_url },
    { { 4, 0 }, { 0, 0 }, sp_webfeed, t_url },
    { { 2, 0 }, { 0, 0 }, sp_weight, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_weighttotal, t_schema_type, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sp_wheelbase, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_width, t_schema_type, sch_distance },
    { { 2, 0 }, { 0, 0 }, sp_width, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_winner, t_schema_type, sch_person },
    { { 2, 0 }, { 0, 0 }, sp_wordcount, t_schema_type, sch_integer },
    { { 2, 0 }, { 0, 0 }, sp_wordcount, t_integer },
    { { 2, 0 }, { 0, 0 }, sp_workexample, t_schema_type, sch_creativework },
    { { 2, 2 }, { 0, 0 }, sp_workfeatured, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_workhours, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_workhours, t_text },
    { { 2, 0 }, { 0, 0 }, sp_worklocation, t_schema_type, sch_contactpoint },
    { { 2, 0 }, { 0, 0 }, sp_worklocation, t_schema_type, sch_place },
    { { 2, 0 }, { 0, 0 }, sp_workperformed, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_workpresented, t_schema_type, sch_movie },
    { { 3, 5 }, { 0, 0 }, sp_worktranslation, t_schema_type, sch_creativework },
    { { 2, 0 }, { 0, 0 }, sp_workload, t_schema_type, sch_energy },
    { { 3, 5 }, { 0, 0 }, sp_workload, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_worksfor, t_schema_type, sch_organisation },
    { { 2, 0 }, { 0, 0 }, sp_worstrating, t_schema_type, sch_number },
    { { 2, 0 }, { 0, 0 }, sp_worstrating, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_worstrating, t_schema_type, sch_text },
    { { 2, 0 }, { 0, 0 }, sp_worstrating, t_text },
    { { 3, 5 }, { 0, 0 }, sp_xpath, t_schema_type, sch_xpathtype },
    { { 6, 0 }, { 0, 0 }, sp_yearbuilt, t_schema_type, sch_number },
    { { 6, 0 }, { 0, 0 }, sp_yearbuilt, t_fixedpoint },
    { { 2, 0 }, { 0, 0 }, sp_yearlyrevenue, t_schema_type, sch_quantitativevalue },
    { { 2, 0 }, { 0, 0 }, sp_yearsinoperation, t_schema_type, sch_quantitativevalue },
    { { 3, 3 }, { 0, 0 }, sp_yield, t_schema_type, sch_quantitativevalue },
    { { 3, 4 }, { 0, 0 }, sp_yield, t_schema_type, sch_text },
    { { 3, 4 }, { 0, 0 }, sp_yield, t_text },

    // open tag
    { { s_video, 1, 0 }, { 0, 0 }, og_video_actor, t_text },
    { { s_video, 1, 0 }, { 0, 0 }, og_video_actor_role, t_text },
    { { s_video, 1, 0 }, { 0, 0 }, og_video_director, t_text },
    { { s_video, 1, 0 }, { 0, 0 }, og_video_duration, t_1_more },
    { { s_video, 1, 0 }, { 0, 0 }, og_video_release_date, t_datetime },
    { { s_video, 1, 0 }, { 0, 0 }, og_video_series, t_url },
    { { s_video, 1, 0 }, { 0, 0 }, og_video_tag, t_text },
    { { s_video, 1, 0 }, { 0, 0 }, og_video_writer, t_text },

    // WDRS
    { { s_wdrs, 1, 0 }, { 0, 0 }, wdrs_describedby, t_url },

    // living standard
    { { s_whatwg, 1, 0 }, { 0, 0 }, wp_author, t_text },
    { { s_whatwg, 1, 0 }, { 0, 0 }, wp_license, t_absolute_url },
    { { s_whatwg, 1, 0 }, { 0, 0 }, wp_title, t_text },
    { { s_whatwg, 1, 0 }, { 0, 0 }, wp_work, t_absolute_url },

    { { 0, 0 }, { 0, 0 }, sp_illegal } };

typedef ::std::vector < e_schema_type > c;
typedef ::std::multimap < e_schema_property, property_gen* > mpp_t;
mpp_t mpp;

void schema_property_init (nitpick& )
{   PRESUME (mpp.empty (), __FILE__, __LINE__);
    for (property_gen* p = &gentab [0]; p -> prop_ != sp_illegal; ++p)
    {   VERIFY_NOT_NULL (p, __FILE__, __LINE__);
        mpp.insert (mpp_t::value_type (p -> prop_, p)); } }

e_schema_property identify_schema_property (nitpick& nits, const schema_version& sv, const ::std::string& name)
{   ::std::string n (::boost::to_lower_copy (name));
    e_schema_property p = get_schema_property (n);
    if (p == sp_illegal)
    {   ::std::string::size_type ends_at = 0;
        if (schema_names.starts_with (SCHEMA_CURIE, true, n, &ends_at) != s_error)
        {   n = n.substr (ends_at);
            p = get_schema_property (n); }
        if (p == sp_illegal)
            nits.pick (nit_not_schema_property, es_error, ec_schema, quote (name), " is not a ", sv.report (), " property"); }
    return p; }

vsp_t identify_schema_properties (nitpick& nits, const schema_version& , const ::std::string& name)
{   vsp_t res;
    ::std::string n (::boost::to_lower_copy (name));
    res = get_schema_properties (n);
    if (res.empty ())
    {   ::std::string::size_type ends_at = 0;
        e_schema x = schema_names.starts_with (SCHEMA_CURIE, true, n, &ends_at);
        if (x != s_error)
        {   n = n.substr (ends_at);
            res = get_schema_properties (n, x); }
        if (res.empty ())
            nits.pick (nit_not_schema_property, es_error, ec_schema, quote (name), " is not a recognised property"); }
    return res; }

bool check_schema_property_version (const schema_version& from, const schema_version& to, const e_schema_property prop, bool& found)
{   for (mpp_t::const_iterator i = mpp.find (prop); (i != mpp.cend ()) && (i -> first == prop); ++i)
    {   found = true;
        VERIFY_NOT_NULL (i -> second, __FILE__, __LINE__);
        if (does_apply < schema_version > (from, i -> second -> from_, i -> second -> to_) &&
            does_apply < schema_version > (to, i -> second -> from_, i -> second -> to_)) return true; }
    return false; }

bool test_enumerated_type (nitpick& nits, const html_version& v, const bool ex, const ::std::string& value)
{   if (value.empty ()) return false;
    if (ex && (schema_names.starts_with (SCHEMA_CURIE, true, value) != s_schema)) return true;
    ::std::string test_value (value);
    if (test_value.find (':') == ::std::string::npos)
        test_value = ::std::string ("https://" SCHEMA_ORG "/") + value;
    type_master < t_itemtype > tm;
    tm.set_value (nits, v, test_value);
    return (tm.good ()); }

bool is_valid_schema_property_int (nitpick& nits, const html_version& v, const e_schema_type schema, const e_schema_property prop, const ::std::string& value, const bool is_link)
{   nitpick knots;
    ::std::string expected, unfound;
    bool many = false, valid_type = false;
    PRESUME (! vsv.empty (), __FILE__, __LINE__);
    ::std::string name (quote (schema_property_name (prop)) + " (" + schema_names.get (static_cast < e_schema > (get_property_root (prop)), SCHEMA_CURIE) + ")");
    ssch_t ssch (generalise (schema));
    for (e_schema_type gen : ssch)
    {   if (is_schema_property (gen, prop))
            for (mpp_t::const_iterator i = mpp.find (prop); (i != mpp.cend ()) && (i -> first == prop); ++i)
            {   VERIFY_NOT_NULL (i -> second, __FILE__, __LINE__);
                if (i -> second -> field_type_ == t_schema_type)
                {   if ((i -> second -> flags_ & SP_VALUE_TYPENAME) == SP_VALUE_TYPENAME)
                        if (test_enumerated_type (knots, v, false, value))
                        {   itemtype_index ii = find_itemtype_index (knots, v, value);
                            if (ii == make_itemtype_index (i -> second -> field_schema_)) return true;
                                if (type_category (ii) == itemtype_schema)
                                    if (is_specific_type_of (context.schema_ver (), i -> second -> field_schema_, static_cast < e_schema_type > (ndx_item (ii))))
                                        return true; }
                    flags_t flags = sch :: flags (i -> second -> field_schema_);
                    bool en = enumerated_schema_type (flags);
                    bool ex = external_enumerated_schema_type (flags);
                    if (ex || en) if (test_enumerated_type (knots, v, ex, value)) return true;
                    if (is_link)
                    {   nitpick ohsnoes;
                        url u (ohsnoes, v, value);
                        if (! u.invalid ())
                        {   if (i -> second -> from_.deprecated ())
                                ohsnoes.pick (nit_deprecated_schema, es_warning, ec_schema, name, " is deprecated (here)");
                            if (! u.is_local () || u.has_args () || u.has_query () || u.empty ()) { nits.merge (ohsnoes); return true; }
                            if (! u.is_http () && ! u.is_https ()) { nits.merge (ohsnoes); return true; }
                            if (u.has_id () && (u.has_path () || u.has_file ())) { nits.merge (ohsnoes); return true; } } }
                    if (has_simple_schema_type (flags))
                    {   nitpick nuts;
                        if (i -> second -> from_.deprecated ())
                                nuts.pick (nit_deprecated_schema, es_warning, ec_schema, name, " is deprecated (here)");
                        if (test_value (nuts, v, get_simple_schema_type (flags), value))
                        {   nits.merge (nuts);
                            return true; }
                        valid_type = true;
                        knots.merge (nuts); }
                    else
                    {   if (! expected.empty ()) { many = true; expected += ", "; }
                        expected += quote (::std::string (schema_names.get (i -> second -> from_.root (), SCHEMA_CURIE)) + "/" + sch :: name (i -> second -> field_schema_)); } }
                else if (! vsv [i -> second -> from_.root ()].invalid ())
                    if (does_apply < schema_version > (vsv [i -> second -> from_.root ()], i -> second -> from_, i -> second -> to_))
                    {   nitpick nuts;
                        if (i -> second -> from_.deprecated ())
                            nuts.pick (nit_deprecated_schema, es_warning, ec_schema, name, " is deprecated (here)");
                        if (test_value (nuts, v, i -> second -> field_type_, value))
                        {   nits.merge (nuts);
                            return true; }
                        valid_type = true;
                        knots.merge (nuts); } }
        if (! expected.empty ())
        {   ::std::string msg ("consider specifying ");
            nitpick nuts;
            msg += name + " in greater detail using ";
            if (many) msg += "one of ";
            msg += expected;
            nuts.pick (nit_schema_property, es_comment, ec_schema, msg);
            if (test_value (nuts, v, t_text, value))
            {   nits.merge (nuts);
                return true; }
            valid_type = true;
            knots.merge (nuts); }
        if (unfound.empty () && (get_property_root (prop) == sch::category (gen)))
            unfound = name + " is not a valid property of " + quote (sch::name (schema)); }
    if (! unfound.empty () && ! valid_type)
        knots.pick (nit_not_schema_property, es_error, ec_schema, unfound);
    nits.merge (knots);
    return false; }

bool is_valid_schema_property (nitpick& nits, const html_version& v, const e_schema_type schema, const e_schema_property prop, const ::std::string& value, const bool is_link)
{   if (! is_valid_schema_property_int (nits, v, schema, prop, value, is_link)) return false;
    return true; }

bool is_valid_schema_property_int (nitpick& nits, const html_version& , const e_schema_type schema, const e_schema_property prop, const e_schema_type value)
{   ssch_t ssch (generalise (schema));
    for (e_schema_type gen : ssch)
        if (is_schema_property (gen, prop))
            for (mpp_t::const_iterator i = mpp.find (prop); (i != mpp.cend ()) && (i -> first == prop); ++i)
            {   VERIFY_NOT_NULL (i -> second, __FILE__, __LINE__);
                if (i -> second -> field_type_ == t_schema_type)
                    if ((i -> second -> field_schema_ == value) || (i -> second -> field_schema_ == sty_context))
                        if (does_apply < schema_version > (context.schema_ver (), i -> second -> from_, i -> second -> to_))
                            return true; }
    nits.pick (nit_bad_property, es_error, ec_schema, quote (schema_property_name (prop)), " cannot have sub-values, or is not a property of ", quote (sch::name (schema)), ", or both");
    return false; }

bool is_valid_schema_property (nitpick& nits, const html_version& v, const e_schema_type schema, const e_schema_property prop, const e_schema_type value)
{   if (! is_valid_schema_property_int (nits, v, schema, prop, value)) return false;
    return true; }

vit_t sought_schema_types (const e_schema_property prop)
{   vit_t res;
    for (mpp_t::const_iterator i = mpp.find (prop); (i != mpp.cend ()) && (i -> first == prop); ++i)
    {   VERIFY_NOT_NULL (i -> second, __FILE__, __LINE__);
        if (i -> second -> field_type_ == t_schema_type)
            if (does_apply < schema_version > (context.schema_ver (), i -> second -> from_, i -> second -> to_))
                res.push_back (make_itemtype_index (i -> second -> field_schema_)); }
    return res; }
